@@ -13,38 +13,21 @@
 #ifndef __AUDACITY_AUDIO_IO__
 #define __AUDACITY_AUDIO_IO__
 
-#include "Audacity.h" // for USE_* macros
+
 
 #include "AudioIOBase.h" // to inherit
+#include "PlaybackSchedule.h" // member variable
 
-#include "Experimental.h"
-
+#include <functional>
 #include <memory>
+#include <mutex>
+#include <thread>
 #include <utility>
 #include <wx/atomic.h> // member variable
 
-#ifdef USE_MIDI
-
-// TODO: Put the relative paths into automake.
-
-#ifdef EXPERIMENTAL_MIDI_OUT
-typedef void PmStream;
-typedef int32_t PmTimestamp;
-
-class Alg_seq;
-class Alg_event;
-class Alg_iterator;
-
-class NoteTrack;
-using NoteTrackArray = std::vector < std::shared_ptr< NoteTrack > >;
-using NoteTrackConstArray = std::vector < std::shared_ptr< const NoteTrack > >;
-
-#endif // EXPERIMENTAL_MIDI_OUT
-
-#endif // USE_MIDI
-
-#include <wx/event.h> // to declare custom event types
-
+#include "PluginProvider.h" // for PluginID
+#include "Observer.h"
+#include "SampleCount.h"
 #include "SampleFormat.h"
 
 class wxArrayString;
@@ -52,12 +35,16 @@ class AudioIOBase;
 class AudioIO;
 class RingBuffer;
 class Mixer;
+class RealtimeEffectState;
 class Resample;
-class AudioThread;
-class SelectedRegion;
 
 class AudacityProject;
 
+class PlayableTrack;
+using PlayableTrackConstArray =
+   std::vector < std::shared_ptr < const PlayableTrack > >;
+
+class Track;
 class WaveTrack;
 using WaveTrackArray = std::vector < std::shared_ptr < WaveTrack > >;
 using WaveTrackConstArray = std::vector < std::shared_ptr < const WaveTrack > >;
@@ -66,43 +53,37 @@ struct PaStreamCallbackTimeInfo;
 typedef unsigned long PaStreamCallbackFlags;
 typedef int PaError;
 
+namespace RealtimeEffects {
+   class ProcessingScope;
+}
+
 bool ValidateDeviceNames();
 
-#define MAX_MIDI_BUFFER_SIZE 5000
-#define DEFAULT_SYNTH_LATENCY 5
+enum class Acknowledge { eNone = 0, eStart, eStop };
 
-wxDECLARE_EXPORTED_EVENT(AUDACITY_DLL_API,
-                         EVT_AUDIOIO_PLAYBACK, wxCommandEvent);
-wxDECLARE_EXPORTED_EVENT(AUDACITY_DLL_API,
-                         EVT_AUDIOIO_CAPTURE, wxCommandEvent);
-wxDECLARE_EXPORTED_EVENT(AUDACITY_DLL_API,
-                         EVT_AUDIOIO_MONITOR, wxCommandEvent);
-
-// PRL:
-// If we always run a portaudio output stream (even just to produce silence)
-// whenever we play Midi, then we might use just one thread for both.
-// I thought this would improve MIDI synch problems on Linux/ALSA, but RBD
-// convinced me it was neither a necessary nor sufficient fix.  Perhaps too the
-// MIDI thread might block in some error situations but we should then not
-// also block the audio thread.
-// So leave the separate thread ENABLED.
-#define USE_MIDI_THREAD
+/*!
+ Emitted by the global AudioIO object when play, recording, or monitoring
+ starts or stops
+*/
+struct AudioIOEvent {
+   AudacityProject *pProject;
+   enum Type {
+      PLAYBACK,
+      CAPTURE,
+      MONITOR,
+   } type;
+   bool on;
+};
 
 struct TransportTracks {
    WaveTrackArray playbackTracks;
    WaveTrackArray captureTracks;
-#ifdef EXPERIMENTAL_MIDI_OUT
-   NoteTrackConstArray midiTracks;
-#endif
+   PlayableTrackConstArray otherPlayableTracks;
 
    // This is a subset of playbackTracks
    WaveTrackConstArray prerollTracks;
 };
 
-// This workaround makes pause and stop work when output is to GarageBand,
-// which seems not to implement the notes-off message correctly.
-#define AUDIO_IO_GB_MIDI_WORKAROUND
-
 /** brief The function which is called from PortAudio's callback thread
  * context to collect and deliver audio for / from the sound device.
  *
@@ -131,108 +112,7 @@ int audacityAudioCallback(
    const PaStreamCallbackTimeInfo *timeInfo,
    PaStreamCallbackFlags statusFlags, void *userData );
 
-// Communicate data from one writer to one reader.
-// This is not a queue: it is not necessary for each write to be read.
-// Rather loss of a message is allowed:  writer may overwrite.
-// Data must be default-constructible and either copyable or movable.
-template<typename Data>
-class MessageBuffer {
-   struct alignas( 64
-      //std::hardware_destructive_interference_size // C++17
-   ) UpdateSlot {
-      std::atomic<bool> mBusy{ false };
-      Data mData;
-   } mSlots[2];
-
-   std::atomic<unsigned char> mLastWrittenSlot{ 0 };
-
-public:
-   void Initialize();
-
-   // Move data out (if available), or else copy it out
-   Data Read();
-   
-   // Copy data in
-   void Write( const Data &data );
-   // Move data in
-   void Write( Data &&data );
-};
-
-template<typename Data>
-void MessageBuffer<Data>::Initialize()
-{
-   for (auto &slot : mSlots)
-      // Lock both slots first, maybe spinning a little
-      while ( slot.mBusy.exchange( true, std::memory_order_acquire ) )
-         {}
-
-   mSlots[0].mData = {};
-   mSlots[1].mData = {};
-   mLastWrittenSlot.store( 0, std::memory_order_relaxed );
-
-   for (auto &slot : mSlots)
-      slot.mBusy.exchange( false, std::memory_order_release );
-}
-
-template<typename Data>
-Data MessageBuffer<Data>::Read()
-{
-   // Whichever slot was last written, prefer to read that.
-   auto idx = mLastWrittenSlot.load( std::memory_order_relaxed );
-   idx = 1 - idx;
-   bool wasBusy = false;
-   do {
-      // This loop is unlikely to execute twice, but it might because the
-      // producer thread is writing a slot.
-      idx = 1 - idx;
-      wasBusy = mSlots[idx].mBusy.exchange( true, std::memory_order_acquire );
-   } while ( wasBusy );
-
-   // Copy the slot
-   auto result = std::move( mSlots[idx].mData );
-
-   mSlots[idx].mBusy.store( false, std::memory_order_release );
-
-   return result;
-}
-
-template<typename Data>
-void MessageBuffer<Data>::Write( const Data &data )
-{
-   // Whichever slot was last written, prefer to write the other.
-   auto idx = mLastWrittenSlot.load( std::memory_order_relaxed );
-   bool wasBusy = false;
-   do {
-      // This loop is unlikely to execute twice, but it might because the
-      // consumer thread is reading a slot.
-      idx = 1 - idx;
-      wasBusy = mSlots[idx].mBusy.exchange( true, std::memory_order_acquire );
-   } while ( wasBusy );
-
-   mSlots[idx].mData = data;
-   mLastWrittenSlot.store( idx, std::memory_order_relaxed );
-
-   mSlots[idx].mBusy.store( false, std::memory_order_release );
-}
-
-template<typename Data>
-void MessageBuffer<Data>::Write( Data &&data )
-{
-   // Whichever slot was last written, prefer to write the other.
-   auto idx = mLastWrittenSlot.load( std::memory_order_relaxed );
-   bool wasBusy = false;
-   do {
-      // This loop is unlikely to execute twice, but it might because the
-      // consumer thread is reading a slot.
-      idx = 1 - idx;
-      wasBusy = mSlots[idx].mBusy.exchange( true, std::memory_order_acquire );
-   } while ( wasBusy );
-
-   mSlots[idx].mData = std::move( data );
-   mLastWrittenSlot.store( idx, std::memory_order_relaxed );
-
-   mSlots[idx].mBusy.store( false, std::memory_order_release );
-}
+class AudioIOExt;
 
 class AUDACITY_DLL_API AudioIoCallback /* not final */
    : public AudioIOBase
@@ -244,11 +124,56 @@ public:
 public:
    // This function executes in a thread spawned by the PortAudio library
    int AudioCallback(
-      const void *inputBuffer, void *outputBuffer,
+      constSamplePtr inputBuffer, float *outputBuffer,
       unsigned long framesPerBuffer,
       const PaStreamCallbackTimeInfo *timeInfo,
       const PaStreamCallbackFlags statusFlags, void *userData);
 
+   //! @name iteration over extensions, supporting range-for syntax
+   //! @{
+   class AUDACITY_DLL_API AudioIOExtIterator {
+   public:
+      using difference_type = ptrdiff_t;
+      using value_type = AudioIOExt &;
+      using pointer = AudioIOExt *;
+      using reference = AudioIOExt &;
+      using iterator_category = std::forward_iterator_tag;
+
+      explicit AudioIOExtIterator( AudioIoCallback &audioIO, bool end )
+         : mIterator{ end
+            ? audioIO.mAudioIOExt.end()
+            : audioIO.mAudioIOExt.begin() }
+      {}
+      AudioIOExtIterator &operator ++ () { ++mIterator; return *this; }
+      auto operator *() const -> AudioIOExt &;
+      friend inline bool operator == (
+         const AudioIOExtIterator &xx, const AudioIOExtIterator &yy)
+      {
+         return xx.mIterator == yy.mIterator;
+      }
+      friend inline bool operator != (
+         const AudioIOExtIterator &xx, const AudioIOExtIterator &yy)
+      {
+         return !(xx == yy);
+      }
+   private:
+      std::vector<std::unique_ptr<AudioIOExtBase>>::const_iterator mIterator;
+   };
+   struct AudioIOExtRange {
+      AudioIOExtIterator first;
+      AudioIOExtIterator second;
+      AudioIOExtIterator begin() const { return first; }
+      AudioIOExtIterator end() const { return second; }
+   };
+
+   AudioIOExtRange Extensions() {
+      return {
+         AudioIOExtIterator{ *this, false },
+         AudioIOExtIterator{ *this, true }
+      };
+   }
+   //! @}
+
    std::shared_ptr< AudioIOListener > GetListener() const
       { return mListener.lock(); }
    void SetListener( const std::shared_ptr< AudioIOListener > &listener);
@@ -268,27 +193,25 @@ public:
    bool TrackHasBeenFadedOut( const WaveTrack &wt );
    bool AllTracksAlreadySilent();
 
-   // These eight functions do different parts of AudioCallback().
-   void ComputeMidiTimings(
-      const PaStreamCallbackTimeInfo *timeInfo,
-      unsigned long framesPerBuffer);
-   void CheckSoundActivatedRecordingLevel(const void *inputBuffer);
+   void CheckSoundActivatedRecordingLevel(
+      float *inputSamples,
+      unsigned long framesPerBuffer
+   );
    void AddToOutputChannel( unsigned int chan,
       float * outputMeterFloats,
       float * outputFloats,
-      float * tempFloats,
-      float * tempBuf,
+      const float * tempBuf,
       bool drop,
       unsigned long len,
       WaveTrack *vt
       );
    bool FillOutputBuffers(
-      void *outputBuffer,
+      float *outputBuffer,
       unsigned long framesPerBuffer,
-      float * tempFloats, float *outputMeterFloats
+      float *outputMeterFloats
    );
-   void FillInputBuffers(
-      const void *inputBuffer, 
+   void DrainInputBuffers(
+      constSamplePtr inputBuffer, 
       unsigned long framesPerBuffer,
       const PaStreamCallbackFlags statusFlags,
       float * tempFloats
@@ -297,28 +220,20 @@ public:
       unsigned long framesPerBuffer
    );
    void DoPlaythrough(
-      const void *inputBuffer, 
-      void *outputBuffer,
+      constSamplePtr inputBuffer, 
+      float *outputBuffer,
       unsigned long framesPerBuffer,
       float *outputMeterFloats
    );
    void SendVuInputMeterData(
-      float *tempFloats,
-      const void *inputBuffer,
+      const float *inputSamples,
       unsigned long framesPerBuffer
    );
    void SendVuOutputMeterData(
-      float *outputMeterFloats,
+      const float *outputMeterFloats,
       unsigned long framesPerBuffer
    );
 
-
-// Required by these functions...
-#ifdef EXPERIMENTAL_MIDI_OUT
-   double AudioTime() { return mPlaybackSchedule.mT0 + mNumFrames / mRate; }
-#endif
-
-
    /** \brief Get the number of audio samples ready in all of the playback
    * buffers.
    *
@@ -326,80 +241,10 @@ public:
    * they are different. */
    size_t GetCommonlyReadyPlayback();
 
+   size_t GetCommonlyWrittenForPlayback();
 
-#ifdef EXPERIMENTAL_MIDI_OUT
-   //   MIDI_PLAYBACK:
-   PmStream        *mMidiStream;
-   int              mLastPmError;
-
-   /// Latency of MIDI synthesizer
-   long             mSynthLatency; // ms
-
-   // These fields are used to synchronize MIDI with audio:
-
-   /// PortAudio's clock time
-   volatile double  mAudioCallbackClockTime;
-
-   /// Number of frames output, including pauses
-   volatile long    mNumFrames;
    /// How many frames of zeros were output due to pauses?
-   volatile long    mNumPauseFrames;
-   /// total of backward jumps
-   volatile int     mMidiLoopPasses;
-   inline double MidiLoopOffset() {
-      return mMidiLoopPasses * (mPlaybackSchedule.mT1 - mPlaybackSchedule.mT0);
-   }
-
-   volatile long    mAudioFramesPerBuffer;
-   /// Used by Midi process to record that pause has begun,
-   /// so that AllNotesOff() is only delivered once
-   volatile bool    mMidiPaused;
-   /// The largest timestamp written so far, used to delay
-   /// stream closing until last message has been delivered
-   PmTimestamp mMaxMidiTimestamp;
-
-   /// Offset from ideal sample computation time to system time,
-   /// where "ideal" means when we would get the callback if there
-   /// were no scheduling delays or computation time
-   double mSystemMinusAudioTime;
-   /// audio output latency reported by PortAudio
-   /// (initially; for Alsa, we adjust it to the largest "observed" value)
-   double mAudioOutLatency;
-
-   // Next two are used to adjust the previous two, if
-   // PortAudio does not provide the info (using ALSA):
-
-   /// time of first callback
-   /// used to find "observed" latency
-   double mStartTime;
-   /// number of callbacks since stream start
-   long mCallbackCount;
-
-   /// Make just one variable to communicate from audio to MIDI thread,
-   /// to avoid problems of atomicity of updates
-   volatile double mSystemMinusAudioTimePlusLatency;
-
-   Alg_seq      *mSeq;
-   std::unique_ptr<Alg_iterator> mIterator;
-   /// The next event to play (or null)
-   Alg_event    *mNextEvent;
-
-#ifdef AUDIO_IO_GB_MIDI_WORKAROUND
-   std::vector< std::pair< int, int > > mPendingNotesOff;
-#endif
-
-   /// Real time at which the next event should be output, measured in seconds.
-   /// Note that this could be a note's time+duration for note offs.
-   double           mNextEventTime;
-   /// Track of next event
-   NoteTrack        *mNextEventTrack;
-   /// Is the next event a note-on?
-   bool             mNextIsNoteOn;
-   /// when true, mSendMidiState means send only updates, not note-on's,
-   /// used to send state changes that precede the selected notes
-   bool             mSendMidiState;
-   NoteTrackConstArray mMidiPlaybackTracks;
-#endif
+   long    mNumPauseFrames;
 
 #ifdef EXPERIMENTAL_AUTOMATED_INPUT_LEVEL_ADJUSTMENT
    bool           mAILAActive;
@@ -418,26 +263,30 @@ public:
    unsigned short mAILALastChangeType;  //0 - no change, 1 - increase change, 2 - decrease change
 #endif
 
-   std::unique_ptr<AudioThread> mThread;
-#ifdef EXPERIMENTAL_MIDI_OUT
-#ifdef USE_MIDI_THREAD
-   std::unique_ptr<AudioThread> mMidiThread;
-#endif
-#endif
+   std::thread mAudioThread;
+   std::atomic<bool> mFinishAudioThread{ false };
+
    ArrayOf<std::unique_ptr<Resample>> mResample;
    ArrayOf<std::unique_ptr<RingBuffer>> mCaptureBuffers;
    WaveTrackArray      mCaptureTracks;
+   /*! Read by worker threads but unchanging during playback */
    ArrayOf<std::unique_ptr<RingBuffer>> mPlaybackBuffers;
    WaveTrackArray      mPlaybackTracks;
+   // Temporary buffers, each as large as the playback buffers
+   std::vector<SampleBuffer> mScratchBuffers;
+   std::vector<float *> mScratchPointers; //!< pointing into mScratchBuffers
 
-   ArrayOf<std::unique_ptr<Mixer>> mPlaybackMixers;
+   std::vector<std::unique_ptr<Mixer>> mPlaybackMixers;
+
+   std::atomic<float>  mMixerOutputVol{ 1.0 };
    static int          mNextStreamToken;
    double              mFactor;
    unsigned long       mMaxFramesOutput; // The actual number of frames output.
-   bool                mbMicroFades; 
+   /*! Read by a worker thread but unchanging during playback */
+   bool                mbMicroFades;
 
    double              mSeek;
-   double              mPlaybackRingBufferSecs;
+   PlaybackPolicy::Duration mPlaybackRingBufferSecs;
    double              mCaptureRingBufferSecs;
 
    /// Preferred batch size for replenishing the playback RingBuffer
@@ -446,40 +295,60 @@ public:
    size_t              mPlaybackQueueMinimum;
 
    double              mMinCaptureSecsToCopy;
+   /*! Read by a worker thread but unchanging during playback */
    bool                mSoftwarePlaythrough;
    /// True if Sound Activated Recording is enabled
+   /*! Read by a worker thread but unchanging during playback */
    bool                mPauseRec;
    float               mSilenceLevel;
+   /*! Read by a worker thread but unchanging during playback */
    unsigned int        mNumCaptureChannels;
+   /*! Read by a worker thread but unchanging during playback */
    unsigned int        mNumPlaybackChannels;
    sampleFormat        mCaptureFormat;
    unsigned long long  mLostSamples{ 0 };
-   volatile bool       mAudioThreadShouldCallFillBuffersOnce;
-   volatile bool       mAudioThreadFillBuffersLoopRunning;
-   volatile bool       mAudioThreadFillBuffersLoopActive;
+   std::atomic<bool>   mAudioThreadShouldCallTrackBufferExchangeOnce;
+   std::atomic<bool>   mAudioThreadTrackBufferExchangeLoopRunning;
+   std::atomic<bool>   mAudioThreadTrackBufferExchangeLoopActive;
+      
+   std::atomic<Acknowledge>  mAudioThreadAcknowledge;
+
+   // Sync start/stop of AudioThread processing
+   void StartAudioThreadAndWait();
+   void StopAudioThreadAndWait();
+
+   // Async start/stop + wait of AudioThread processing.
+   // Provided to allow more flexibility, however use with caution:
+   // never call Stop between Start and the wait for Started (and the converse)
+   void StartAudioThread();
+   void WaitForAudioThreadStarted();
+   void StopAudioThread();
+   void WaitForAudioThreadStopped();
+
+   void ProcessOnceAndWait( std::chrono::milliseconds sleepTime = std::chrono::milliseconds(50) );
+
+
+
+   std::atomic<bool>   mForceFadeOut{ false };
 
    wxLongLong          mLastPlaybackTimeMillis;
 
-#ifdef EXPERIMENTAL_MIDI_OUT
-   volatile bool       mMidiThreadFillBuffersLoopRunning;
-   volatile bool       mMidiThreadFillBuffersLoopActive;
-#endif
-
-   volatile double     mLastRecordingOffset;
+   //! Not (yet) used; should perhaps be atomic when it is
+   double              mLastRecordingOffset;
    PaError             mLastPaError;
 
 protected:
 
-   bool                mUpdateMeters;
-   volatile bool       mUpdatingMeters;
+   float GetMixerOutputVol() {
+      return mMixerOutputVol.load(std::memory_order_relaxed); }
+   void SetMixerOutputVol(float value) {
+      mMixerOutputVol.store(value, std::memory_order_relaxed); }
 
+   /*! Pointer is read by a worker thread but unchanging during playback.
+    (Whether its overriding methods are race-free is not for AudioIO to ensure.)
+    */
    std::weak_ptr< AudioIOListener > mListener;
 
-   friend class AudioThread;
-#ifdef EXPERIMENTAL_MIDI_OUT
-   friend class MidiThread;
-#endif
-
    bool mUsingAlsa { false };
 
    // For cacheing supported sample rates
@@ -491,16 +360,6 @@ protected:
    // the state used by the third, Audio thread.
    wxMutex mSuspendAudioThread;
 
-#ifdef EXPERIMENTAL_SCRUBBING_SUPPORT
-public:
-   struct ScrubState;
-   std::unique_ptr<ScrubState> mScrubState;
-
-   bool mSilentScrub;
-   double mScrubSpeed;
-   sampleCount mScrubDuration;
-#endif
-
 protected:
    // A flag tested and set in one thread, cleared in another.  Perhaps
    // this guarantee of atomicity is more cautious than necessary.
@@ -511,6 +370,7 @@ protected:
       { if (mRecordingException) wxAtomicDec( mRecordingException ); }
 
    std::vector< std::pair<double, double> > mLostCaptureIntervals;
+   /*! Read by a worker thread but unchanging during playback */
    bool mDetectDropouts{ true };
 
 public:
@@ -523,45 +383,59 @@ public:
 
    // Whether to check the error code passed to audacityAudioCallback to
    // detect more dropouts
-   bool mDetectUpstreamDropouts{ true };
+   std::atomic<bool> mDetectUpstreamDropouts{ true };
 
 protected:
    RecordingSchedule mRecordingSchedule{};
+   PlaybackSchedule mPlaybackSchedule;
 
-   // Another circular buffer
-   // Holds track time values corresponding to every nth sample in the playback
-   // buffers, for some large n
-   struct TimeQueue {
-      Doubles mData;
-      size_t mSize{ 0 };
-      double mLastTime {};
-      // These need not be updated atomically, because we rely on the atomics
-      // in the playback ring buffers to supply the synchronization.  Still,
-      // align them to avoid false sharing.
-      alignas(64) struct Cursor {
-         size_t mIndex {};
-         size_t mRemainder {};
-      } mHead, mTail;
-
-      void Producer(
-         const PlaybackSchedule &schedule, double rate, double scrubSpeed,
-         size_t nSamples );
-      double Consumer( size_t nSamples, double rate );
-   } mTimeQueue;
+   struct TransportState;
+   //! Holds some state for duration of playback or recording
+   std::unique_ptr<TransportState> mpTransportState;
 
+private:
+   /*!
+    Privatize the inherited array but give access by Extensions().
+    This class guarantees that this array is populated only with non-null
+    pointers to the subtype AudioIOExt
+    */
+   using AudioIOBase::mAudioIOExt;
 };
 
+struct PaStreamInfo;
+
 class AUDACITY_DLL_API AudioIO final
    : public AudioIoCallback
+   , public Observer::Publisher<AudioIOEvent>
 {
 
    AudioIO();
    ~AudioIO();
+   void StartThread();
 
 public:
    // This might return null during application startup or shutdown
    static AudioIO *Get();
 
+   //! Forwards to RealtimeEffectManager::AddState with proper init scope
+   /*!
+    @post result: `!result || result->GetEffect() != nullptr`
+    */
+   std::shared_ptr<RealtimeEffectState>
+   AddState(AudacityProject &project, Track *pTrack, const PluginID & id);
+
+   //! Forwards to RealtimeEffectManager::ReplaceState with proper init scope
+   /*!
+    @post result: `!result || result->GetEffect() != nullptr`
+    */
+   std::shared_ptr<RealtimeEffectState>
+   ReplaceState(AudacityProject &project,
+      Track *pTrack, size_t index, const PluginID & id);
+
+   //! Forwards to RealtimeEffectManager::RemoveState with proper init scope
+   void RemoveState(AudacityProject &project,
+      Track *pTrack, std::shared_ptr<RealtimeEffectState> pState);
+
    /** \brief Start up Portaudio for capture and recording as needed for
     * input monitoring and software playthrough only
     *
@@ -579,8 +453,9 @@ public:
     * instance.  For use with IsStreamActive() */
 
    int StartStream(const TransportTracks &tracks,
-                   double t0, double t1,
-                   const AudioIOStartStreamOptions &options);
+      double t0, double t1,
+      double mixerLimit, //!< Time at which mixer stops producing, maybe > t1
+      const AudioIOStartStreamOptions &options);
 
    /** \brief Stop recording, playback or input monitoring.
     *
@@ -592,51 +467,18 @@ public:
     * by the specified amount from where it is now */
    void SeekStream(double seconds) { mSeek = seconds; }
 
-#ifdef EXPERIMENTAL_SCRUBBING_SUPPORT
-   bool IsScrubbing() const { return IsBusy() && mScrubState != 0; }
-
-   /** \brief Notify scrubbing engine of desired position or speed.
-   * If options.adjustStart is true, then when mouse movement exceeds maximum
-   * scrub speed, adjust the beginning of the scrub interval rather than the
-   * end, so that the scrub skips or "stutters" to stay near the cursor.
-   */
-   void UpdateScrub(double endTimeOrSpeed, const ScrubbingOptions &options);
-
-   void StopScrub();
-
-   /** \brief return the ending time of the last scrub interval.
-   */
-   double GetLastScrubTime() const;
-#endif
+   using PostRecordingAction = std::function<void()>;
+   
+   //! Enqueue action for main thread idle time, not before the end of any recording in progress
+   /*! This may be called from non-main threads */
+   void CallAfterRecording(PostRecordingAction action);
 
 public:
    wxString LastPaErrorString();
 
    wxLongLong GetLastPlaybackTime() const { return mLastPlaybackTimeMillis; }
-   AudacityProject *GetOwningProject() const { return mOwningProject; }
-
-#ifdef EXPERIMENTAL_MIDI_OUT
-   /** \brief Compute the current PortMidi timestamp time.
-    *
-    * This is used by PortMidi to synchronize midi time to audio samples
-    */
-   PmTimestamp MidiTime();
-
-   // Note: audio code solves the problem of soloing/muting tracks by scanning
-   // all playback tracks on every call to the audio buffer fill routine.
-   // We do the same for Midi, but it seems wasteful for at least two
-   // threads to be frequently polling to update status. This could be
-   // eliminated (also with a reduction in code I think) by updating mHasSolo
-   // each time a solo button is activated or deactivated. For now, I'm
-   // going to do this polling in the FillMidiBuffer routine to localize
-   // changes for midi to the midi code, but I'm declaring the variable
-   // here so possibly in the future, Audio code can use it too. -RBD
- private:
-   bool  mHasSolo; // is any playback solo button pressed?
- public:
-   bool SetHasSolo(bool hasSolo);
-   bool GetHasSolo() { return mHasSolo; }
-#endif
+   std::shared_ptr<AudacityProject> GetOwningProject() const
+   { return mOwningProject.lock(); }
 
    /** \brief Pause and un-pause playback and recording */
    void SetPaused(bool state);
@@ -659,14 +501,6 @@ public:
     */
    bool InputMixerWorks();
 
-   /** @brief Find out if the output level control is being emulated via software attenuation
-    *
-    * Checks the mEmulateMixerOutputVol variable, which is set up in
-    * AudioIOBase::HandleDeviceChange(). External classes care, because we want to
-    * modify the UI if it doesn't work.
-    */
-   bool OutputMixerEmulated();
-
    /** \brief Get the list of inputs to the current mixer device
     *
     * Returns an array of strings giving the names of the inputs to the
@@ -697,7 +531,7 @@ public:
       double AILAGetLastDecisionTime();
    #endif
 
-   bool IsAvailable(AudacityProject *projecT) const;
+   bool IsAvailable(AudacityProject &project) const;
 
    /** \brief Return a valid sample rate that is supported by the current I/O
    * device(s).
@@ -711,23 +545,31 @@ public:
    * and playing is true if one or more channels are being played. */
    double GetBestRate(bool capturing, bool playing, double sampleRate);
 
-   friend class AudioThread;
-#ifdef EXPERIMENTAL_MIDI_OUT
-   friend class MidiThread;
-#endif
+   /** \brief During playback, the track time most recently played
+    *
+    * When playing looped, this will start from t0 again,
+    * too. So the returned time should be always between
+    * t0 and t1
+    */
+   double GetStreamTime();
+
+   static void AudioThread(std::atomic<bool> &finish);
 
    static void Init();
    static void Deinit();
 
-
+   /*! For purposes of CallAfterRecording, treat time from now as if
+    recording (when argument is true) or not necessarily so (false) */
+   void DelayActions(bool recording);
 
 private:
 
+   bool DelayingActions() const;
+
    /** \brief Set the current VU meters - this should be done once after
     * each call to StartStream currently */
    void SetMeters();
 
-
    /** \brief Opens the portaudio stream(s) used to do playback or recording
     * (or both) through.
     *
@@ -742,22 +584,28 @@ private:
                              unsigned int numPlaybackChannels,
                              unsigned int numCaptureChannels,
                              sampleFormat captureFormat);
-   void FillBuffers();
 
-#ifdef EXPERIMENTAL_MIDI_OUT
-   void PrepareMidiIterator(bool send = true, double offset = 0);
-   bool StartPortMidiStream();
+   void SetOwningProject( const std::shared_ptr<AudacityProject> &pProject );
+   void ResetOwningProject();
 
-   // Compute nondecreasing real time stamps, accounting for pauses, but not the
-   // synth latency.
-   double UncorrectedMidiEventTime();
+   /*!
+    Called in a loop from another worker thread that does not have the low-latency constraints
+    of the PortAudio callback thread.  Does less frequent and larger batches of work that may
+    include memory allocations and database operations.  RingBuffer objects mediate the transfer
+    between threads, to overcome the mismatch of their batch sizes.
+    */
+   void TrackBufferExchange();
 
-   void OutputEvent();
-   void FillMidiBuffers();
-   void GetNextEvent();
-   double PauseTime();
-   void AllNotesOff(bool looping = false);
-#endif
+   //! First part of TrackBufferExchange
+   void FillPlayBuffers();
+   void TransformPlayBuffers(
+      std::optional<RealtimeEffects::ProcessingScope> &scope);
+   bool ProcessPlaybackSlices(
+      std::optional<RealtimeEffects::ProcessingScope> &pScope,
+      size_t available);
+
+   //! Second part of TrackBufferExchange
+   void DrainRecordBuffers();
 
    /** \brief Get the number of audio samples free in all of the playback
    * buffers.
@@ -781,15 +629,17 @@ private:
      */
    bool AllocateBuffers(
       const AudioIOStartStreamOptions &options,
-      const TransportTracks &tracks, double t0, double t1, double sampleRate,
-      bool scrubbing );
+      const TransportTracks &tracks, double t0, double t1, double sampleRate );
 
    /** \brief Clean up after StartStream if it fails.
      *
      * If bOnlyBuffers is specified, it only cleans up the buffers. */
    void StartStreamCleanup(bool bOnlyBuffers = false);
+
+   std::mutex mPostRecordingActionMutex;
+   PostRecordingAction mPostRecordingAction;
+
+   bool mDelayingActions{ false };
 };
 
-static constexpr unsigned ScrubPollInterval_ms = 50;
-
 #endif
