@@ -16,18 +16,18 @@
 #include "SelectedRegion.h"
 #include "Track.h"
 
+#include <wx/event.h> // to inherit
 
 class wxTextFile;
 
 class AudacityProject;
-class DirManager;
-class NotifyingSelectedRegion;
 class TimeWarper;
 
+class LabelTrack;
 struct LabelTrackHit;
 struct TrackPanelDrawingContext;
 
-class LabelStruct
+class AUDACITY_DLL_API LabelStruct
 {
 public:
    LabelStruct() = default;
@@ -88,8 +88,25 @@ class AUDACITY_DLL_API LabelTrack final
    , public wxEvtHandler
 {
  public:
-   LabelTrack(const std::shared_ptr<DirManager> &projDirManager);
-   LabelTrack(const LabelTrack &orig);
+   static wxString GetDefaultName();
+
+   // Construct and also build all attachments
+   static LabelTrack *New(AudacityProject &project);
+
+   /**
+    * \brief Create a new LabelTrack with specified @p name and append it to the @p trackList
+    * \return New LabelTrack with custom name
+    */
+   static LabelTrack* Create(TrackList& trackList, const wxString& name);
+
+   /**
+    * \brief Create a new LabelTrack with unique default name and append it to the @p trackList
+    * \return New LabelTrack with unique default name
+    */
+   static LabelTrack* Create(TrackList& trackList);
+
+   LabelTrack();
+   LabelTrack(const LabelTrack &orig, ProtectedCreationArg&&);
 
    virtual ~ LabelTrack();
 
@@ -104,20 +121,15 @@ class AUDACITY_DLL_API LabelTrack final
    double GetEndTime() const override;
 
    using Holder = std::shared_ptr<LabelTrack>;
-   
+
 private:
    Track::Holder Clone() const override;
 
 public:
-   bool HandleXMLTag(const wxChar *tag, const wxChar **attrs) override;
-   XMLTagHandler *HandleXMLChild(const wxChar *tag) override;
+   bool HandleXMLTag(const std::string_view& tag, const AttributesList& attrs) override;
+   XMLTagHandler *HandleXMLChild(const std::string_view& tag) override;
    void WriteXML(XMLWriter &xmlFile) const override;
 
-#if LEGACY_PROJECT_FILE_SUPPORT
-   bool Load(wxTextFile * in, DirManager * dirManager) override;
-   bool Save(wxTextFile * out, bool overwrite) override;
-#endif
-
    Track::Holder Cut  (double t0, double t1) override;
    Track::Holder Copy (double t0, double t1, bool forClipboard = true) const override;
    void Clear(double t0, double t1) override;
@@ -161,12 +173,24 @@ public:
    int FindNextLabel(const SelectedRegion& currentSelection);
    int FindPrevLabel(const SelectedRegion& currentSelection);
 
+   const TypeInfo &GetTypeInfo() const override;
+   static const TypeInfo &ClassTypeInfo();
+
+   Track::Holder PasteInto( AudacityProject & ) const override;
+
+   struct IntervalData final : Track::IntervalData {
+      size_t index;
+      explicit IntervalData(size_t index) : index{index} {};
+   };
+   ConstInterval MakeInterval ( size_t index ) const;
+   Interval MakeInterval ( size_t index );
+   ConstIntervals GetIntervals() const override;
+   Intervals GetIntervals() override;
+
  public:
    void SortLabels();
 
  private:
-   TrackKind GetKind() const override { return TrackKind::Label; }
-
    LabelArray mLabels;
 
    // Set in copied label tracks
@@ -175,7 +199,9 @@ public:
    int miLastLabel;                 // used by FindNextLabel and FindPrevLabel
 };
 
-struct LabelTrackEvent : TrackListEvent
+ENUMERATE_TRACK_TYPE(LabelTrack);
+
+struct LabelTrackEvent : public wxEvent
 {
    explicit
    LabelTrackEvent(
@@ -184,7 +210,8 @@ struct LabelTrackEvent : TrackListEvent
       int formerPosition,
       int presentPosition
    )
-   : TrackListEvent{ commandType, pTrack }
+   : wxEvent{ 0, commandType }
+   , mpTrack{ pTrack }
    , mTitle{ title }
    , mFormerPosition{ formerPosition }
    , mPresentPosition{ presentPosition }
@@ -195,6 +222,8 @@ struct LabelTrackEvent : TrackListEvent
       // wxWidgets will own the event object
       return safenew LabelTrackEvent(*this); }
 
+   const std::weak_ptr<Track> mpTrack;
+
    // invalid for selection events
    wxString mTitle;
 
