@@ -25,29 +25,38 @@
 
 *//*******************************************************************/
 
-#include "Audacity.h" // for USE_* macros
+
 #include "Menus.h"
 
-#include "Experimental.h"
+
 
 #include <wx/frame.h>
 
 #include "Project.h"
 #include "ProjectHistory.h"
 #include "ProjectSettings.h"
+#include "ProjectWindows.h"
 #include "UndoManager.h"
 #include "commands/CommandManager.h"
 #include "toolbars/ToolManager.h"
 #include "widgets/AudacityMessageBox.h"
-#include "widgets/ErrorDialog.h"
+#include "BasicUI.h"
 
 #include <unordered_set>
 
 #include <wx/menu.h>
 #include <wx/windowptr.h>
+#include <wx/log.h>
 
 MenuCreator::MenuCreator()
 {
+   mLastAnalyzerRegistration = repeattypenone;
+   mLastToolRegistration = repeattypenone;
+   
+   mRepeatGeneratorFlags = 0;
+   mRepeatEffectFlags = 0;
+   mRepeatAnalyzerFlags = 0;
+   mRepeatToolFlags = 0;
 }
 
 MenuCreator::~MenuCreator()
@@ -73,16 +82,12 @@ MenuManager::MenuManager( AudacityProject &project )
    : mProject{ project }
 {
    UpdatePrefs();
-   mProject.Bind( EVT_UNDO_OR_REDO, &MenuManager::OnUndoRedo, this );
-   mProject.Bind( EVT_UNDO_RESET, &MenuManager::OnUndoRedo, this );
-   mProject.Bind( EVT_UNDO_PUSHED, &MenuManager::OnUndoRedo, this );
+   mUndoSubscription = UndoManager::Get(project)
+      .Subscribe(*this, &MenuManager::OnUndoRedo);
 }
 
 MenuManager::~MenuManager()
 {
-   mProject.Unbind( EVT_UNDO_OR_REDO, &MenuManager::OnUndoRedo, this );
-   mProject.Unbind( EVT_UNDO_RESET, &MenuManager::OnUndoRedo, this );
-   mProject.Unbind( EVT_UNDO_PUSHED, &MenuManager::OnUndoRedo, this );
 }
 
 void MenuManager::UpdatePrefs()
@@ -400,7 +405,9 @@ void MenuCreator::CreateMenusAndCommands(AudacityProject &project)
 //"ShowMeterTB,"
 "ShowMixerTB,"
 "ShowEditTB,ShowTranscriptionTB,ShowScrubbingTB,ShowDeviceTB,ShowSelectionTB,"
-"ShowSpectralSelectionTB") }
+"ShowSpectralSelectionTB") },
+         {wxT("/Tracks/Add/Add"), wxT(
+   "NewMonoTrack,NewStereoTrack,NewLabelTrack,NewTimeTrack")},
       }
    };
 
@@ -428,6 +435,8 @@ void MenuCreator::CreateMenusAndCommands(AudacityProject &project)
 void MenuManager::Visit( ToolbarMenuVisitor &visitor )
 {
    static const auto menuTree = MenuTable::Items( MenuPathStart );
+
+   wxLogNull nolog;
    Registry::Visit( visitor, menuTree.get(), &sRegistry() );
 }
 
@@ -501,9 +510,17 @@ void MenuCreator::RebuildMenuBar(AudacityProject &project)
    CreateMenusAndCommands(project);
 }
 
-void MenuManager::OnUndoRedo( wxCommandEvent &evt )
+void MenuManager::OnUndoRedo(UndoRedoMessage message)
 {
-   evt.Skip();
+   switch (message.type) {
+   case UndoRedoMessage::UndoOrRedo:
+   case UndoRedoMessage::Reset:
+   case UndoRedoMessage::Pushed:
+   case UndoRedoMessage::Renamed:
+      break;
+   default:
+      return;
+   }
    ModifyUndoMenuItems( mProject );
    UpdateMenus();
 }
@@ -797,8 +814,7 @@ void MenuManager::TellUserWhyDisallowed(
       return;
 
    // Does not have the warning icon...
-   ShowErrorDialog(
-      NULL,
+   BasicUI::ShowErrorDialog( {},
       untranslatedTitle,
       reason,
       helpPage);
