@@ -20,8 +20,6 @@
 #include "widgets/AudacityMessageBox.h"
 #include "Internat.h"
 #include "MemoryX.h"
-#include "MissingAliasFileDialog.h"
-#include "ondemand/ODManager.h"
 #include "widgets/MultiDialog.h"
 #include "widgets/ProgressDialog.h"
 
@@ -35,6 +33,9 @@
 int ProjectFSCK(
    DirManager &dm, const bool bForceError, const bool bAutoRecoverMode)
 {
+#pragma message( "====================================================================")
+#pragma message( "Don\'t forget to redo ProjectFSCK")
+#pragma message( "====================================================================")
    // In earlier versions of this method, enumerations of errors were
    // all done in sequence, then the user was prompted for each type of error.
    // The enumerations are now interleaved with prompting, because, for example,
@@ -69,7 +70,7 @@ int ProjectFSCK(
       else
          nResult = FSCKstatus_CHANGED | FSCKstatus_SAVE_AUP;
    }
-
+#if 0
    FilePaths filePathArray; // *all* files in the project directory/subdirectories
    auto dirPath = ( dm.GetDataFilesDir() );
    DirManager::RecursivelyEnumerateWithProgress(
@@ -142,8 +143,8 @@ XO("Project check of \"%s\" folder \
                if (action == 2)
                {
                   // silence the blockfiles by yanking the filename
-                  // This is done, eventually, in PCMAliasBlockFile::ReadData()
-                  // and ODPCMAliasBlockFile::ReadData, in the stack of b->Recover().
+                  // This is done, eventually, in PCMAliasBlockFile::ReadData(),
+                  // in the stack of b->Recover().
                   // There, if the mAliasedFileName is bad, it zeroes the data.
                   wxFileNameWrapper dummy;
                   dummy.Clear();
@@ -373,7 +374,7 @@ other projects. \
       }
    }
 
-   if ((nResult != FSCKstatus_CLOSE_REQ) && !ODManager::HasLoadedODFlag())
+   if (nResult != FSCKstatus_CLOSE_REQ)
    {
       // Remove any empty directories.
       ProgressDialog pProgress(
@@ -404,5 +405,6 @@ other projects. \
    }
 
    MissingAliasFilesDialog::SetShouldShow(true);
+#endif
    return nResult;
 }
