@@ -10,12 +10,15 @@ Paul Licameli split from AudacityProject.cpp
 
 #include "ProjectWindow.h"
 
-#include "Experimental.h"
 
+
+#include "ActiveProject.h"
 #include "AllThemeResources.h"
+#include "AudioIO.h"
 #include "Menus.h"
 #include "Project.h"
 #include "ProjectAudioIO.h"
+#include "ProjectWindows.h"
 #include "ProjectStatus.h"
 #include "RefreshCode.h"
 #include "TrackPanelMouseEvent.h"
@@ -24,6 +27,7 @@ Paul Licameli split from AudacityProject.cpp
 #include "ViewInfo.h"
 #include "WaveClip.h"
 #include "WaveTrack.h"
+#include "commands/CommandContext.h"
 #include "prefs/ThemePrefs.h"
 #include "prefs/TracksPrefs.h"
 #include "toolbars/ToolManager.h"
@@ -32,9 +36,38 @@ Paul Licameli split from AudacityProject.cpp
 #include "widgets/wxPanelWrapper.h"
 #include "widgets/WindowAccessible.h"
 
+#include "ThemedWrappers.h"
+#include "RealtimeEffectPanel.h"
+
+#include <wx/app.h>
 #include <wx/display.h>
 #include <wx/scrolbar.h>
 #include <wx/sizer.h>
+#include <wx/splitter.h>
+#include <wx/wupdlock.h>
+
+#include "TrackPanel.h"
+
+namespace
+{
+#ifdef HAS_AUDIOCOM_UPLOAD
+   constexpr int DEFAULT_WINDOW_WIDTH = 1120;
+#else
+   constexpr int DEFAULT_WINDOW_WIDTH = 1060;
+#endif
+   constexpr int DEFAULT_WINDOW_HEIGHT = 674;
+}
+
+BoolSetting ProjectWindowMaximized{ L"/Window/Maximized", false };
+BoolSetting ProjectWindowIconized{ L"/Window/Iconized", false };
+IntSetting ProjectWindowX{ L"/Window/X", 0 };
+IntSetting ProjectWindowY{ L"/Window/Y", 0 };
+IntSetting ProjectWindowWidth{ L"/Window/Width", DEFAULT_WINDOW_WIDTH };
+IntSetting ProjectWindowHeight{ L"/Window/Height", DEFAULT_WINDOW_HEIGHT };
+IntSetting ProjectWindowNormalX{ L"/Window/Normal_X", 0 };
+IntSetting ProjectWindowNormalY{ L"/Window/Normal_Y", 0 };
+IntSetting ProjectWindowNormalWidth{ L"/Window/Normal_Width", DEFAULT_WINDOW_WIDTH };
+IntSetting ProjectWindowNormalHeight{ L"/Window/Normal_Height", DEFAULT_WINDOW_HEIGHT };
 
 // Returns the screen containing a rectangle, or -1 if none does.
 int ScreenContaining( wxRect & r ){
@@ -89,8 +122,8 @@ void GetDefaultWindowRect(wxRect *defRect)
 {
    *defRect = wxGetClientDisplayRect();
 
-   int width = 940;
-   int height = 674;
+   int width = DEFAULT_WINDOW_WIDTH;
+   int height = DEFAULT_WINDOW_HEIGHT;
 
    //These conditional values assist in improving placement and size
    //of NEW windows on different platforms.
@@ -139,20 +172,20 @@ void GetNextWindowPlacement(wxRect *nextRect, bool *pMaximized, bool *pIconized)
    wxRect defaultRect;
    GetDefaultWindowRect(&defaultRect);
 
-   gPrefs->Read(wxT("/Window/Maximized"), pMaximized, false);
-   gPrefs->Read(wxT("/Window/Iconized"), pIconized, false);
+   *pMaximized = ProjectWindowMaximized.Read();
+   *pIconized = ProjectWindowIconized.Read();
 
    wxRect windowRect;
-   gPrefs->Read(wxT("/Window/X"), &windowRect.x, defaultRect.x);
-   gPrefs->Read(wxT("/Window/Y"), &windowRect.y, defaultRect.y);
-   gPrefs->Read(wxT("/Window/Width"), &windowRect.width, defaultRect.width);
-   gPrefs->Read(wxT("/Window/Height"), &windowRect.height, defaultRect.height);
+   windowRect.x = ProjectWindowX.ReadWithDefault(defaultRect.x);
+   windowRect.y = ProjectWindowY.ReadWithDefault(defaultRect.y);
+   windowRect.width = ProjectWindowWidth.ReadWithDefault(defaultRect.width);
+   windowRect.height = ProjectWindowHeight.ReadWithDefault(defaultRect.height);
 
    wxRect normalRect;
-   gPrefs->Read(wxT("/Window/Normal_X"), &normalRect.x, defaultRect.x);
-   gPrefs->Read(wxT("/Window/Normal_Y"), &normalRect.y, defaultRect.y);
-   gPrefs->Read(wxT("/Window/Normal_Width"), &normalRect.width, defaultRect.width);
-   gPrefs->Read(wxT("/Window/Normal_Height"), &normalRect.height, defaultRect.height);
+   normalRect.x = ProjectWindowNormalX.ReadWithDefault(defaultRect.x);
+   normalRect.y = ProjectWindowNormalY.ReadWithDefault(defaultRect.y);
+   normalRect.width = ProjectWindowNormalWidth.ReadWithDefault(defaultRect.width);
+   normalRect.height = ProjectWindowNormalHeight.ReadWithDefault(defaultRect.height);
 
    // Workaround 2.1.1 and earlier bug on OSX...affects only normalRect, but let's just
    // validate for all rects and plats
@@ -352,19 +385,11 @@ END_EVENT_TABLE()
 // Common mouse wheel handling in track panel cells, moved here to avoid
 // compilation dependencies on Track, TrackPanel, and Scrubbing at low levels
 // which made cycles
-static struct MouseWheelHandler {
-
-MouseWheelHandler()
-{
-   CommonTrackPanelCell::InstallMouseWheelHook( *this );
-}
-
+static CommonTrackPanelCell::MouseWheelHook::Scope scope{
 // Need a bit of memory from one call to the next
-mutable double mVertScrollRemainder = 0.0;
-
-unsigned operator()
-   ( const TrackPanelMouseEvent &evt, AudacityProject *pProject ) const
-{
+[mVertScrollRemainder = 0.0](
+   const TrackPanelMouseEvent &evt, AudacityProject *pProject )
+mutable -> unsigned {
    using namespace RefreshCode;
 
    if ( TrackList::Get( *pProject ).empty() )
@@ -491,11 +516,9 @@ unsigned operator()
    }
 
    return result;
-}
+} };
 
-} sMouseWheelHandler;
-
-AudacityProject::AttachedWindows::RegisteredFactory sProjectWindowKey{
+AttachedWindows::RegisteredFactory sProjectWindowKey{
    []( AudacityProject &parent ) -> wxWeakRef< wxWindow > {
       wxRect wndRect;
       bool bMaximized = false;
@@ -530,7 +553,7 @@ AudacityProject::AttachedWindows::RegisteredFactory sProjectWindowKey{
 
 ProjectWindow &ProjectWindow::Get( AudacityProject &project )
 {
-   return project.AttachedWindows::Get< ProjectWindow >( sProjectWindowKey );
+   return GetAttachedWindows(project).Get< ProjectWindow >(sProjectWindowKey);
 }
 
 const ProjectWindow &ProjectWindow::Get( const AudacityProject &project )
@@ -541,7 +564,7 @@ const ProjectWindow &ProjectWindow::Get( const AudacityProject &project )
 ProjectWindow *ProjectWindow::Find( AudacityProject *pProject )
 {
    return pProject
-      ? pProject->AttachedWindows::Find< ProjectWindow >( sProjectWindowKey )
+      ? GetAttachedWindows(*pProject).Find< ProjectWindow >(sProjectWindowKey)
       : nullptr;
 }
 
@@ -550,6 +573,14 @@ const ProjectWindow *ProjectWindow::Find( const AudacityProject *pProject )
    return Find( const_cast< AudacityProject * >( pProject ) );
 }
 
+void ProjectWindow::OnResetWindow(const CommandContext& context)
+{
+   auto& project = context.project;
+   auto& window = ProjectWindow::Get(project);
+
+   window.Reset();
+}
+
 int ProjectWindow::NextWindowID()
 {
    return mNextWindowID++;
@@ -594,45 +625,40 @@ ProjectWindow::ProjectWindow(wxWindow * parent, wxWindowID id,
    mTopPanel->SetBackgroundColour(theTheme.Colour( clrMedium ));
 #endif
 
-   wxWindow    * pPage;
+   auto container = safenew wxSplitterWindow(this, wxID_ANY,
+      wxDefaultPosition,
+      wxDefaultSize,
+      wxNO_BORDER | wxSP_LIVE_UPDATE | wxSP_THIN_SASH);
+   container->Bind(wxEVT_SPLITTER_DOUBLECLICKED, [](wxSplitterEvent& event){
+      //"The default behaviour is to unsplit the window"
+      event.Veto();//do noting instead
+   });
+   mContainerWindow = container;
 
-#ifdef EXPERIMENTAL_NOTEBOOK
-   // We are using a notebook (tabbed panel), so we create the notebook and add pages.
-   GuiFactory Factory;
-   wxNotebook  * pNotebook;
-   mMainPanel = Factory.AddPanel(
-      this, wxPoint( left, top ), wxSize( width, height ) );
-   pNotebook  = Factory.AddNotebook( mMainPanel );
-   /* i18n-hint: This is an experimental feature where the main panel in
-      Audacity is put on a notebook tab, and this is the name on that tab.
-      Other tabs in that notebook may have instruments, patch panels etc.*/
-   pPage = Factory.AddPage( pNotebook, _("Main Mix"));
-#else
-   // Not using a notebook, so we place the track panel inside another panel,
-   // this keeps the notebook code and normal code consistent and also
-   // paves the way for adding additional windows inside the track panel.
-   mMainPanel = safenew wxPanelWrapper(this, -1,
+   mTrackListWindow = safenew wxPanelWrapper(mContainerWindow, wxID_ANY,
       wxDefaultPosition,
       wxDefaultSize,
       wxNO_BORDER);
-   mMainPanel->SetSizer( safenew wxBoxSizer(wxVERTICAL) );
-   mMainPanel->SetLabel("Main Panel");// Not localised.
-   pPage = mMainPanel;
-   // Set the colour here to the track panel background to avoid
-   // flicker when Audacity starts up.
-   // However, that leads to areas next to the horizontal scroller
-   // being painted in background colour and not scroller background
-   // colour, so suppress this for now.
-   //pPage->SetBackgroundColour( theTheme.Colour( clrDark ));
-#endif
-   pPage->SetLayoutDirection(wxLayout_LeftToRight);
+   mTrackListWindow->SetSizer( safenew wxBoxSizer(wxVERTICAL) );
+   mTrackListWindow->SetLabel("Main Panel");// Not localized.
+   mTrackListWindow->SetLayoutDirection(wxLayout_LeftToRight);
+
+   mContainerWindow->Initialize(mTrackListWindow);
+
+   auto effectsPanel = safenew ThemedWindowWrapper<RealtimeEffectPanel>(mProject, mContainerWindow, wxID_ANY);
+   effectsPanel->SetName(_("Realtime effects"));
+   effectsPanel->SetBackgroundColorIndex(clrMedium);
+   effectsPanel->Hide();//initially hidden
+   effectsPanel->Bind(wxEVT_CLOSE_WINDOW, [this](wxCloseEvent&)
+   {
+      HideEffectsPanel();
+   });
+   mEffectsWindow = effectsPanel;
 
 #ifdef EXPERIMENTAL_DA2
-   pPage->SetBackgroundColour(theTheme.Colour( clrMedium ));
+   mTrackListWindow->SetBackgroundColour(theTheme.Colour( clrMedium ));
 #endif
 
-   mMainPage = pPage;
-
    mPlaybackScroller = std::make_unique<PlaybackScroller>( &project );
 
    // PRL: Old comments below.  No longer observing the ordering that it
@@ -644,8 +670,8 @@ ProjectWindow::ProjectWindow(wxWindow * parent, wxWindowID id,
    //      creating the scrollbars after the TrackPanel, we resolve
    //      several focus problems.
 
-   mHsbar = safenew ScrollBar(pPage, HSBarID, wxSB_HORIZONTAL);
-   mVsbar = safenew ScrollBar(pPage, VSBarID, wxSB_VERTICAL);
+   mHsbar = safenew ScrollBar(mTrackListWindow, HSBarID, wxSB_HORIZONTAL);
+   mVsbar = safenew ScrollBar(mTrackListWindow, VSBarID, wxSB_VERTICAL);
 #if wxUSE_ACCESSIBILITY
    // so that name can be set on a standard control
    mHsbar->SetAccessible(safenew WindowAccessible(mHsbar));
@@ -655,12 +681,34 @@ ProjectWindow::ProjectWindow(wxWindow * parent, wxWindowID id,
    mHsbar->SetName(_("Horizontal Scrollbar"));
    mVsbar->SetName(_("Vertical Scrollbar"));
 
-   project.Bind( EVT_UNDO_MODIFIED, &ProjectWindow::OnUndoPushedModified, this );
-   project.Bind( EVT_UNDO_PUSHED, &ProjectWindow::OnUndoPushedModified, this );
-   project.Bind( EVT_UNDO_OR_REDO, &ProjectWindow::OnUndoRedo, this );
-   project.Bind( EVT_UNDO_RESET, &ProjectWindow::OnUndoReset, this );
+   mUndoSubscription = UndoManager::Get(project)
+      .Subscribe([this](UndoRedoMessage message){
+         switch (message.type) {
+         case UndoRedoMessage::Pushed:
+         case UndoRedoMessage::Modified:
+            return OnUndoPushedModified();
+         case UndoRedoMessage::UndoOrRedo:
+            return OnUndoRedo();
+         case UndoRedoMessage::Reset:
+            return OnUndoReset();
+         default:
+            return;
+         }
+      });
+
+   mThemeChangeSubscription =
+      theTheme.Subscribe(*this, &ProjectWindow::OnThemeChange);
+
+   mFocusChangeSubscription = TrackFocus::Get(project)
+      .Subscribe([this](const TrackFocusChangeMessage& msg) {
+         if(mEffectsWindow->IsShown())
+         {
+            auto& project = GetProject();
+            auto& trackFocus = TrackFocus::Get(project);
+            ShowEffectsPanel(trackFocus.Get(), false);
+         }
+      });
 
-   wxTheApp->Bind(EVT_THEME_CHANGE, &ProjectWindow::OnThemeChange, this);
 }
 
 ProjectWindow::~ProjectWindow()
@@ -721,9 +769,10 @@ void ProjectWindow::RedrawProject(const bool bForceWaveTracks /*= false*/)
    });
 }
 
-void ProjectWindow::OnThemeChange(wxCommandEvent& evt)
+void ProjectWindow::OnThemeChange(ThemeChangeMessage message)
 {
-   evt.Skip();
+   if (message.appearance)
+      return;
    auto &project = mProject;
    this->ApplyUpdatedTheme();
    auto &toolManager = ToolManager::Get( project );
@@ -1217,6 +1266,34 @@ bool ProjectWindow::IsIconized() const
    return mIconized;
 }
 
+wxWindow* ProjectWindow::GetEffectsWindow() noexcept
+{
+   return mEffectsWindow;
+}
+
+wxWindow* ProjectWindow::GetTrackListWindow() noexcept
+{
+   return mTrackListWindow;
+}
+
+wxWindow* ProjectWindow::GetContainerWindow() noexcept
+{
+   return mContainerWindow;
+}
+
+wxPanel* ProjectWindow::GetTopPanel() noexcept
+{
+   return mTopPanel;
+}
+
+void ProjectWindow::Reset()
+{
+   wxRect defaultRect;
+   GetDefaultWindowRect(&defaultRect);
+
+   SetSize(defaultRect.width, defaultRect.height);
+}
+
 void ProjectWindow::UpdateStatusWidths()
 {
    enum { nWidths = nStatusBarFields + 1 };
@@ -1387,22 +1464,19 @@ void ProjectWindow::OnToolBarUpdate(wxCommandEvent & event)
    event.Skip(false);             /* No need to propagate any further */
 }
 
-void ProjectWindow::OnUndoPushedModified( wxCommandEvent &evt )
+void ProjectWindow::OnUndoPushedModified()
 {
-   evt.Skip();
    RedrawProject();
 }
 
-void ProjectWindow::OnUndoRedo( wxCommandEvent &evt )
+void ProjectWindow::OnUndoRedo()
 {
-   evt.Skip();
    HandleResize();
    RedrawProject();
 }
 
-void ProjectWindow::OnUndoReset( wxCommandEvent &evt )
+void ProjectWindow::OnUndoReset()
 {
-   evt.Skip();
    HandleResize();
    // RedrawProject();  // Should we do this here too?
 }
@@ -1653,19 +1727,16 @@ void ProjectWindow::TP_HandleResize()
 ProjectWindow::PlaybackScroller::PlaybackScroller(AudacityProject *project)
 : mProject(project)
 {
-   ViewInfo::Get( *mProject ).Bind(EVT_TRACK_PANEL_TIMER,
-      &PlaybackScroller::OnTimer,
-      this);
 }
 
-void ProjectWindow::PlaybackScroller::OnTimer(wxCommandEvent &event)
+void ProjectWindow::PlaybackScroller::OnTimer()
 {
-   // Let other listeners get the notification
-   event.Skip();
+   auto gAudioIO = AudioIO::Get();
+   mRecentStreamTime = gAudioIO->GetStreamTime();
 
    auto cleanup = finally([&]{
       // Propagate the message to other listeners bound to this
-      this->ProcessEvent( event );
+      this->Publish({});
    });
 
    if(!ProjectAudioIO::Get( *mProject ).IsAudioActive())
@@ -1685,7 +1756,7 @@ void ProjectWindow::PlaybackScroller::OnTimer(wxCommandEvent &event)
 
       auto &viewInfo = ViewInfo::Get( *mProject );
       auto &trackPanel = GetProjectPanel( *mProject );
-      const int posX = viewInfo.TimeToPosition(viewInfo.mRecentStreamTime);
+      const int posX = viewInfo.TimeToPosition(mRecentStreamTime);
       auto width = viewInfo.GetTracksUsableWidth();
       int deltaX;
       switch (mMode)
@@ -1714,7 +1785,7 @@ void ProjectWindow::ZoomInByFactor( double ZoomFactor )
    auto &project = mProject;
    auto &viewInfo = ViewInfo::Get( project );
 
-   auto gAudioIO = AudioIOBase::Get();
+   auto gAudioIO = AudioIO::Get();
    // LLL: Handling positioning differently when audio is
    // actively playing.  Don't do this if paused.
    if (gAudioIO->IsStreamActive(
@@ -1841,11 +1912,52 @@ void ProjectWindow::DoZoomFit()
    window.TP_ScrollWindow(start);
 }
 
-static struct InstallTopPanelHook{ InstallTopPanelHook() {
-   ToolManager::SetGetTopPanelHook(
-      []( wxWindow &window ){
-         auto pProjectWindow = dynamic_cast< ProjectWindow* >( &window );
-         return pProjectWindow ? pProjectWindow->GetTopPanel() : nullptr;
-      }
-   );
-}} installTopPanelHook;
+void ProjectWindow::ShowEffectsPanel(Track* track, bool focus)
+{
+   if(track == nullptr)
+   {
+      mEffectsWindow->ResetTrack();
+      return;
+   }
+
+   wxWindowUpdateLocker freeze(this);
+
+   mEffectsWindow->SetTrack(track->shared_from_this());
+
+   if(mContainerWindow->GetWindow1() != mEffectsWindow)
+   {
+      //Restore previous effects window size
+      mContainerWindow->SplitVertically(
+         mEffectsWindow,
+         mTrackListWindow,
+         mEffectsWindow->GetSize().GetWidth());
+   }
+   if(focus)
+      mEffectsWindow->SetFocus();
+   Layout();
+}
+
+void ProjectWindow::HideEffectsPanel()
+{
+   wxWindowUpdateLocker freeze(this);
+
+   if(mContainerWindow->GetWindow2() == nullptr)
+      //only effects panel is present, restore split positions before removing effects panel
+      //Workaround: ::Replace and ::Initialize do not work here...
+      mContainerWindow->SplitVertically(mEffectsWindow, mTrackListWindow);
+
+   mContainerWindow->Unsplit(mEffectsWindow);
+   mTrackListWindow->SetFocus();
+   Layout();
+}
+
+bool ProjectWindow::IsEffectsPanelShown()
+{
+   return mEffectsWindow->IsShown();
+}
+
+static ToolManager::TopPanelHook::Scope scope {
+[]( wxWindow &window ){
+   auto pProjectWindow = dynamic_cast< ProjectWindow* >( &window );
+   return pProjectWindow ? pProjectWindow->GetTopPanel() : nullptr;
+} };
