@@ -26,7 +26,7 @@
   TrackInfo class to draw the controls area on the left of a track,
   and the TrackArtist class to draw the actual waveforms.
 
-  Note that in some of the older code here, e.g., GetLabelWidth(),
+  Note that in some of the older code here,
   "Label" means the TrackInfo plus the vertical ruler.
   Confusing relative to LabelTrack labels.
 
@@ -43,26 +43,27 @@ is time to refresh some aspect of the screen.
 
 *//*****************************************************************/
 
-#include "Audacity.h" // for USE_* macros
-#include "TrackPanel.h"
 
-#include "Experimental.h"
+#include "TrackPanel.h"
+#include "TrackPanelConstants.h"
 
 #include <wx/setup.h> // for wxUSE_* macros
 
 #include "AdornedRulerPanel.h"
+#include "tracks/ui/CommonTrackPanelCell.h"
 #include "KeyboardCapture.h"
 #include "Project.h"
 #include "ProjectAudioIO.h"
 #include "ProjectAudioManager.h"
 #include "ProjectHistory.h"
+#include "ProjectWindows.h"
 #include "ProjectSettings.h"
 #include "ProjectStatus.h"
 #include "ProjectWindow.h"
 #include "Theme.h"
+#include "TrackArt.h"
 #include "TrackPanelMouseEvent.h"
 #include "TrackPanelResizeHandle.h"
-//#define DEBUG_DRAW_TIMING 1
 
 #include "UndoManager.h"
 
@@ -78,8 +79,7 @@ is time to refresh some aspect of the screen.
 #include "TrackPanelResizerCell.h"
 #include "WaveTrack.h"
 
-#include "ondemand/ODManager.h"
-#include "ondemand/ODTask.h"
+#include "FrameStatistics.h"
 
 #include "tracks/ui/TrackControls.h"
 #include "tracks/ui/TrackView.h"
@@ -94,6 +94,11 @@ is time to refresh some aspect of the screen.
 #include <wx/dcclient.h>
 #include <wx/graphics.h>
 
+#include "effects/RealtimeEffectManager.h"
+
+static_assert( kVerticalPadding == kTopMargin + kBottomMargin );
+static_assert( kTrackInfoBtnSize == kAffordancesAreaHeight, "Drag bar is misaligned with the menu button");
+
 /**
 
 \class TrackPanel
@@ -117,8 +122,8 @@ controls, and is a constant.
 GetVRulerWidth() is variable -- all tracks have the same ruler width at any
 time, but that width may be adjusted when tracks change their vertical scales.
 
-GetLabelWidth() counts columns up to and including the VRuler.
-GetLeftOffset() is yet one more -- it counts the "one pixel" column.
+GetLeftOffset() counts columns up to and including the VRuler and one more,
+the "one pixel" column.
 
 Cell for label has a rectangle that OMITS left, top, and bottom
 margins
@@ -198,12 +203,12 @@ std::unique_ptr<wxCursor> MakeCursor( int WXUNUSED(CursorId), const char * const
 
 namespace{
 
-AudacityProject::AttachedWindows::RegisteredFactory sKey{
+AttachedWindows::RegisteredFactory sKey{
    []( AudacityProject &project ) -> wxWeakRef< wxWindow > {
       auto &ruler = AdornedRulerPanel::Get( project );
       auto &viewInfo = ViewInfo::Get( project );
       auto &window = ProjectWindow::Get( project );
-      auto mainPage = window.GetMainPage();
+      auto mainPage = window.GetTrackListWindow();
       wxASSERT( mainPage ); // to justify safenew
 
       auto &tracks = TrackList::Get( project );
@@ -215,7 +220,7 @@ AudacityProject::AttachedWindows::RegisteredFactory sKey{
          &viewInfo,
          &project,
          &ruler);
-      project.SetPanel( result );
+      SetProjectPanel( project, *result );
       return result;
    }
 };
@@ -224,7 +229,7 @@ AudacityProject::AttachedWindows::RegisteredFactory sKey{
 
 TrackPanel &TrackPanel::Get( AudacityProject &project )
 {
-   return project.AttachedWindows::Get< TrackPanel >( sKey );
+   return GetAttachedWindows(project).Get< TrackPanel >( sKey );
 }
 
 const TrackPanel &TrackPanel::Get( const AudacityProject &project )
@@ -234,10 +239,10 @@ const TrackPanel &TrackPanel::Get( const AudacityProject &project )
 
 void TrackPanel::Destroy( AudacityProject &project )
 {
-   auto *pPanel = project.AttachedWindows::Find( sKey );
+   auto *pPanel = GetAttachedWindows(project).Find<TrackPanel>( sKey );
    if (pPanel) {
       pPanel->wxWindow::Destroy();
-      project.AttachedWindows::Assign( sKey, nullptr );
+      GetAttachedWindows(project).Assign(sKey, nullptr);
    }
 }
 
@@ -283,8 +288,6 @@ TrackPanel::TrackPanel(wxWindow * parent, wxWindowID id,
          *this, std::move( pAx ) );
    }
 
-   mRedrawAfterStop = false;
-
    mTrackArtist = std::make_unique<TrackArtist>( this );
 
    mTimeCount = 0;
@@ -294,35 +297,41 @@ TrackPanel::TrackPanel(wxWindow * parent, wxWindowID id,
       &TrackPanel::OnIdle, this);
 
    // Register for tracklist updates
-   mTracks->Bind(EVT_TRACKLIST_RESIZING,
-                    &TrackPanel::OnTrackListResizing,
-                    this);
-   mTracks->Bind(EVT_TRACKLIST_ADDITION,
-                    &TrackPanel::OnTrackListResizing,
-                    this);
-   mTracks->Bind(EVT_TRACKLIST_DELETION,
-                    &TrackPanel::OnTrackListDeletion,
-                    this);
-   mTracks->Bind(EVT_TRACKLIST_TRACK_REQUEST_VISIBLE,
-                    &TrackPanel::OnEnsureVisible,
-                    this);
+   mTrackListSubscription =
+   mTracks->Subscribe([this](const TrackListEvent &event){
+      switch (event.mType) {
+      case TrackListEvent::RESIZING:
+      case TrackListEvent::ADDITION:
+         OnTrackListResizing(event); break;
+      case TrackListEvent::DELETION:
+         OnTrackListDeletion(); break;
+      case TrackListEvent::TRACK_REQUEST_VISIBLE:
+         OnEnsureVisible(event); break;
+      default:
+         break;
+      }
+   });
 
    auto theProject = GetProject();
-   wxTheApp->Bind(EVT_ODTASK_UPDATE, &TrackPanel::OnODTask, this);
-   theProject->Bind(EVT_ODTASK_COMPLETE, &TrackPanel::OnODTask, this);
    theProject->Bind(
       EVT_PROJECT_SETTINGS_CHANGE, &TrackPanel::OnProjectSettingsChange, this);
-   theProject->Bind(
-      EVT_TRACK_FOCUS_CHANGE, &TrackPanel::OnTrackFocusChange, this );
+   mFocusChangeSubscription = TrackFocus::Get(*theProject)
+      .Subscribe(*this, &TrackPanel::OnTrackFocusChange);
 
-   theProject->Bind(EVT_UNDO_RESET, &TrackPanel::OnUndoReset, this);
+   mUndoSubscription = UndoManager::Get(*theProject)
+      .Subscribe(*this, &TrackPanel::OnUndoReset);
+
+   mAudioIOSubscription =
+      AudioIO::Get()->Subscribe(*this, &TrackPanel::OnAudioIO);
+
+   mRealtimeEffectManagerSubscription = RealtimeEffectManager::Get(*theProject)
+      .Subscribe([this](const RealtimeEffectManagerMessage& msg)
+      {
+         if(msg.track)
+            //update "effects" button 
+            RefreshTrack(msg.track.get());
+      });
 
-   wxTheApp->Bind(EVT_AUDIOIO_PLAYBACK,
-                     &TrackPanel::OnAudioIO,
-                     this);
-   wxTheApp->Bind(EVT_AUDIOIO_CAPTURE,
-                     &TrackPanel::OnAudioIO,
-                     this);
    UpdatePrefs();
 }
 
@@ -340,7 +349,7 @@ TrackPanel::~TrackPanel()
 void TrackPanel::UpdatePrefs()
 {
    // All vertical rulers must be recalculated since the minimum and maximum
-   // frequences may have been changed.
+   // frequencies may have been changed.
    UpdateVRulers();
 
    Refresh();
@@ -350,22 +359,16 @@ void TrackPanel::UpdatePrefs()
 /// goes with this track panel.
 AudacityProject * TrackPanel::GetProject() const
 {
-   //JKC casting away constness here.
-   //Do it in two stages in case 'this' is not a wxWindow.
-   //when the compiler will flag the error.
-   wxWindow const * const pConstWind = this;
-   wxWindow * pWind=(wxWindow*)pConstWind;
-#ifdef EXPERIMENTAL_NOTEBOOK
-   pWind = pWind->GetParent(); //Page
-   wxASSERT( pWind );
-   pWind = pWind->GetParent(); //Notebook
-   wxASSERT( pWind );
-#endif
-   pWind = pWind->GetParent(); //MainPanel
-   wxASSERT( pWind );
-   pWind = pWind->GetParent(); //ProjectWindow
-   wxASSERT( pWind );
-   return &static_cast<ProjectWindow*>( pWind )->GetProject();
+   auto window = GetParent();
+
+   while(window != nullptr)
+   {
+      if(const auto projectWindow = dynamic_cast<ProjectWindow*>(window))
+         return &projectWindow->GetProject();
+
+      window = window->GetParent();
+   }
+   return nullptr;
 }
 
 void TrackPanel::OnSize( wxSizeEvent &evt )
@@ -382,7 +385,7 @@ void TrackPanel::OnIdle(wxIdleEvent& event)
    // The window must be ready when the timer fires (#1401)
    if (IsShownOnScreen())
    {
-      mTimer.Start(kTimerInterval, FALSE);
+      mTimer.Start(std::chrono::milliseconds{kTimerInterval}.count(), FALSE);
 
       // Timer is started, we don't need the event anymore
       GetProjectFrame( *GetProject() ).Unbind(wxEVT_IDLE,
@@ -423,18 +426,13 @@ void TrackPanel::OnTimer(wxTimerEvent& )
    {
       projectAudioIO.SetAudioIOToken(0);
       window.RedrawProject();
-
-      mRedrawAfterStop = false;
    }
    if (mLastDrawnSelectedRegion != mViewInfo->selectedRegion) {
       UpdateSelectionDisplay();
    }
 
    // Notify listeners for timer ticks
-   {
-      wxCommandEvent e(EVT_TRACK_PANEL_TIMER);
-      p->ProcessEvent(e);
-   }
+   window.GetPlaybackScroller().OnTimer();
 
    DrawOverlays(false);
    mRuler->DrawOverlays(false);
@@ -443,33 +441,17 @@ void TrackPanel::OnTimer(wxTimerEvent& )
 
       // Periodically update the display while recording
 
-      if (!mRedrawAfterStop) {
-         mRedrawAfterStop = true;
-         MakeParentRedrawScrollbars();
-         mListener->TP_ScrollUpDown( 99999999 );
+      if ((mTimeCount % 5) == 0) {
+         // Must tell OnPaint() to recreate the backing bitmap
+         // since we've not done a full refresh.
+         mRefreshBacking = true;
          Refresh( false );
       }
-      else {
-         if ((mTimeCount % 5) == 0) {
-            // Must tell OnPaint() to recreate the backing bitmap
-            // since we've not done a full refresh.
-            mRefreshBacking = true;
-            Refresh( false );
-         }
-      }
    }
    if(mTimeCount > 1000)
       mTimeCount = 0;
 }
 
-///Handles the redrawing necessary for tasks as they partially update in the
-///background, or finish.
-void TrackPanel::OnODTask(wxCommandEvent & WXUNUSED(event))
-{
-   //todo: add track data to the event - check to see if the project contains it before redrawing.
-   Refresh(false);
-}
-
 void TrackPanel::OnProjectSettingsChange( wxCommandEvent &event )
 {
    event.Skip();
@@ -482,11 +464,12 @@ void TrackPanel::OnProjectSettingsChange( wxCommandEvent &event )
    }
 }
 
-void TrackPanel::OnUndoReset( wxCommandEvent &event )
+void TrackPanel::OnUndoReset(UndoRedoMessage message)
 {
-   event.Skip();
-   TrackFocus::Get( *GetProject() ).Set( nullptr );
-   Refresh( false );
+   if (message.type == UndoRedoMessage::Reset) {
+      TrackFocus::Get( *GetProject() ).Set( nullptr );
+      Refresh( false );
+   }
 }
 
 /// AS: OnPaint( ) is called during the normal course of
@@ -495,9 +478,8 @@ void TrackPanel::OnPaint(wxPaintEvent & /* event */)
 {
    mLastDrawnSelectedRegion = mViewInfo->selectedRegion;
 
-#if DEBUG_DRAW_TIMING
-   wxStopWatch sw;
-#endif
+   auto sw =
+      FrameStatistics::CreateStopwatch(FrameStatistics::SectionID::TrackPanel);
 
    {
       wxPaintDC dc(this);
@@ -533,17 +515,6 @@ void TrackPanel::OnPaint(wxPaintEvent & /* event */)
       dc.DestroyClippingRegion();
       DrawOverlays(true, &dc);
    }
-
-#if DEBUG_DRAW_TIMING
-   sw.Pause();
-   wxLogDebug(wxT("Total: %ld milliseconds"), sw.Time());
-   wxPrintf(wxT("Total: %ld milliseconds\n"), sw.Time());
-#endif
-}
-
-void TrackPanel::MakeParentModifyState(bool bWantsAutoSave)
-{
-   ProjectHistory::Get( *GetProject() ).ModifyState(bWantsAutoSave);
 }
 
 void TrackPanel::MakeParentRedrawScrollbars()
@@ -691,21 +662,20 @@ void TrackPanel::UpdateViewIfNoTracks()
 
 // The tracks positions within the list have changed, so update the vertical
 // ruler size for the track that triggered the event.
-void TrackPanel::OnTrackListResizing(TrackListEvent & e)
+void TrackPanel::OnTrackListResizing(const TrackListEvent &e)
 {
    auto t = e.mpTrack.lock();
    // A deleted track can trigger the event.  In which case do nothing here.
    // A deleted track can have a valid pointer but no owner, bug 2060
    if( t && t->HasOwner() )
       UpdateVRuler(t.get());
-   e.Skip();
 
    // fix for bug 2477
    mListener->TP_RedrawScrollbars();
 }
 
 // Tracks have been removed from the list.
-void TrackPanel::OnTrackListDeletion(wxEvent & e)
+void TrackPanel::OnTrackListDeletion()
 {
    // copy shared_ptr for safety, as in HandleClick
    auto handle = Target();
@@ -718,8 +688,6 @@ void TrackPanel::OnTrackListDeletion(wxEvent & e)
    TrackFocus( *GetProject() ).Get();
 
    UpdateVRulerSize();
-
-   e.Skip();
 }
 
 void TrackPanel::OnKeyDown(wxKeyEvent & event)
@@ -735,7 +703,7 @@ void TrackPanel::OnKeyDown(wxKeyEvent & event)
    case WXK_PAGEDOWN:
       HandlePageDownKey();
       return;
-      
+
    default:
       // fall through to base class handler
       event.Skip();
@@ -751,7 +719,7 @@ void TrackPanel::OnMouseEvent(wxMouseEvent & event)
       // When this timer fires, we call TrackPanel::OnTimer and
       // possibly update the screen for offscreen scrolling.
       mTimer.Stop();
-      mTimer.Start(kTimerInterval, FALSE);
+      mTimer.Start(std::chrono::milliseconds{kTimerInterval}.count(), FALSE);
    }
 
 
@@ -774,7 +742,7 @@ void TrackPanel::OnMouseEvent(wxMouseEvent & event)
 double TrackPanel::GetMostRecentXPos()
 {
    return mViewInfo->PositionToTime(
-      MostRecentXCoord(), mViewInfo->GetLabelWidth());
+      MostRecentXCoord(), mViewInfo->GetLeftOffset());
 }
 
 void TrackPanel::RefreshTrack(Track *trk, bool refreshbacking)
@@ -782,23 +750,30 @@ void TrackPanel::RefreshTrack(Track *trk, bool refreshbacking)
    if (!trk)
       return;
 
+   // Always move to the first channel of the group, and use only
+   // the sum of channel heights, not the height of any channel alone!
    trk = *GetTracks()->FindLeader(trk);
    auto &view = TrackView::Get( *trk );
    auto height =
-      TrackList::Channels(trk).sum( TrackView::GetTrackHeight )
-      - kTopInset - kShadowThickness;
+      TrackList::Channels(trk).sum( TrackView::GetTrackHeight );
 
-   // subtract insets and shadows from the rectangle, but not border
+   // Set rectangle top according to the scrolling position, `vpos`
+   // Subtract the inset (above) and shadow (below) from the height of the
+   // rectangle, but not the border
    // This matters because some separators do paint over the border
-   wxRect rect(kLeftInset,
-            -mViewInfo->vpos + view.GetY() + kTopInset,
-            GetRect().GetWidth() - kLeftInset - kRightInset - kShadowThickness,
-            height);
+   const auto top =
+      -mViewInfo->vpos + view.GetCumulativeHeightBefore() + kTopInset;
+   height -= (kTopInset + kShadowThickness);
+
+   // Width also subtracts insets (left and right) plus shadow (right)
+   const auto left = kLeftInset;
+   const auto width = GetRect().GetWidth()
+      - (kLeftInset + kRightInset + kShadowThickness);
+
+   wxRect rect(left, top, width, height);
 
    if( refreshbacking )
-   {
       mRefreshBacking = true;
-   }
 
    Refresh( false, &rect );
 }
@@ -829,9 +804,10 @@ void TrackPanel::Refresh(bool eraseBackground /* = TRUE */,
    CallAfter([this]{ CellularPanel::HandleCursorForPresentMouseState(); } );
 }
 
-void TrackPanel::OnAudioIO(wxCommandEvent & evt)
+void TrackPanel::OnAudioIO(AudioIOEvent evt)
 {
-   evt.Skip();
+   if (evt.type == AudioIOEvent::MONITOR)
+      return;
    // Some hit tests want to change their cursor to and from the ban symbol
    CallAfter( [this]{ CellularPanel::HandleCursorForPresentMouseState(); } );
 }
@@ -864,6 +840,10 @@ void TrackPanel::DrawTracks(wxDC * dc)
    bool bigPointsFlag  =
       bMultiToolDown || (ToolCodes::drawTool == settings.GetTool());
    bool sliderFlag     = bMultiToolDown;
+   bool brushFlag   = false;
+#ifdef EXPERIMENTAL_BRUSH_TOOL
+   brushFlag   = (ToolCodes::brushTool == settings.GetTool());
+#endif
 
    const bool hasSolo = GetTracks()->Any< PlayableTrack >()
       .any_of( []( const PlayableTrack *pt ) {
@@ -875,22 +855,69 @@ void TrackPanel::DrawTracks(wxDC * dc)
    mTrackArtist->drawEnvelope = envelopeFlag;
    mTrackArtist->bigPoints = bigPointsFlag;
    mTrackArtist->drawSliders = sliderFlag;
+   mTrackArtist->onBrushTool = brushFlag;
    mTrackArtist->hasSolo = hasSolo;
 
    this->CellularPanel::Draw( context, TrackArtist::NPasses );
 }
 
 void TrackPanel::SetBackgroundCell
-(const std::shared_ptr< TrackPanelCell > &pCell)
+(const std::shared_ptr< CommonTrackPanelCell > &pCell)
 {
    mpBackground = pCell;
 }
 
-std::shared_ptr< TrackPanelCell > TrackPanel::GetBackgroundCell()
+std::shared_ptr< CommonTrackPanelCell > TrackPanel::GetBackgroundCell()
 {
    return mpBackground;
 }
 
+namespace {
+std::vector<int> FindAdjustedChannelHeights( Track &t )
+{
+   auto channels = TrackList::Channels(&t);
+   wxASSERT(!channels.empty());
+
+   // Collect heights, and count affordances
+   int nAffordances = 0;
+   int totalHeight = 0;
+   std::vector<int> oldHeights;
+   for (auto channel : channels) {
+      auto &view = TrackView::Get( *channel );
+      const auto height = view.GetHeight();
+      totalHeight += height;
+      oldHeights.push_back( height );
+      if (view.GetAffordanceControls())
+         ++nAffordances;
+   }
+
+   // Allocate results
+   auto nChannels = static_cast<int>(oldHeights.size());
+   std::vector<int> results;
+   results.reserve(nChannels);
+
+   // Now reallocate the channel heights for the presence of affordances
+   // and separators
+   auto availableHeight = totalHeight
+      - nAffordances * kAffordancesAreaHeight
+      - (nChannels - 1) * kChannelSeparatorThickness
+      - kTrackSeparatorThickness;
+   int cumulativeOldHeight = 0;
+   int cumulativeNewHeight = 0;
+   for (const auto &oldHeight : oldHeights) {
+      // Preserve the porportions among the stored heights
+      cumulativeOldHeight += oldHeight;
+      const auto newHeight =
+         cumulativeOldHeight * availableHeight / totalHeight
+            - cumulativeNewHeight;
+      cumulativeNewHeight += newHeight;
+      results.push_back(newHeight);
+   }
+
+   return results;
+}
+}
+
 void TrackPanel::UpdateVRulers()
 {
    for (auto t : GetTracks()->Any< WaveTrack >())
@@ -913,20 +940,22 @@ void TrackPanel::UpdateTrackVRuler(Track *t)
    if (!t)
       return;
 
+   auto heights = FindAdjustedChannelHeights(*t);
+
    wxRect rect(mViewInfo->GetVRulerOffset(),
             0,
             mViewInfo->GetVRulerWidth(),
             0);
 
-
+   auto pHeight = heights.begin();
    for (auto channel : TrackList::Channels(t)) {
       auto &view = TrackView::Get( *channel );
-      const auto height = view.GetHeight() - (kTopMargin + kBottomMargin);
+      const auto height = *pHeight++;
       rect.SetHeight( height );
       const auto subViews = view.GetSubViews( rect );
       if (subViews.empty())
          continue;
-   
+
       auto iter = subViews.begin(), end = subViews.end(), next = iter;
       auto yy = iter->first;
       wxSize vRulerSize{ 0, 0 };
@@ -939,10 +968,10 @@ void TrackPanel::UpdateTrackVRuler(Track *t)
          // This causes ruler size in the track to be reassigned:
          TrackVRulerControls::Get( *iter->second ).UpdateRuler( rect );
          // But we want to know the maximum width and height over all sub-views:
-         vRulerSize.IncTo( t->vrulerSize );
+         vRulerSize.IncTo( {t->vrulerSize.first, t->vrulerSize.second} );
          yy = nextY;
       }
-      t->vrulerSize = vRulerSize;
+      t->vrulerSize = {vRulerSize.x, vRulerSize.y};
    }
 }
 
@@ -952,7 +981,7 @@ void TrackPanel::UpdateVRulerSize()
    if (trackRange) {
       wxSize s { 0, 0 };
       for (auto t : trackRange)
-         s.IncTo(t->vrulerSize);
+         s.IncTo({t->vrulerSize.first, t->vrulerSize.second});
 
       if (mViewInfo->GetVRulerWidth() != s.GetWidth()) {
          mViewInfo->SetVRulerWidth( s.GetWidth() );
@@ -970,10 +999,9 @@ void TrackPanel::OnTrackMenu(Track *t)
 }
 
 // Tracks have been removed from the list.
-void TrackPanel::OnEnsureVisible(TrackListEvent & e)
+void TrackPanel::OnEnsureVisible(const TrackListEvent & e)
 {
-   e.Skip();
-   bool modifyState = e.GetInt();
+   bool modifyState = e.mExtra;
 
    auto pTrack = e.mpTrack.lock();
    auto t = pTrack.get();
@@ -1031,7 +1059,7 @@ void TrackPanel::VerticalScroll( float fracPosition){
    trackTop = range.sum( TrackView::GetChannelGroupHeight );
 
    int delta;
-   
+
    //Get the size of the trackpanel.
    int width, height;
    GetSize(&width, &height);
@@ -1099,7 +1127,11 @@ void DrawTrackName(
    // Tracks more than kTranslucentHeight will have maximum translucency for shields.
    const int kOpaqueHeight = 44;
    const int kTranslucentHeight = 124;
-   int h = rect.GetHeight();
+
+   // PRL:  to do:  reexamine this strange use of TrackView::GetHeight,
+   // ultimately to compute an opacity
+   int h = TrackView::Get( *t ).GetHeight();
+
    // f codes the opacity as a number between 0.0 and 1.0
    float f = wxClip((h-kOpaqueHeight)/(float)(kTranslucentHeight-kOpaqueHeight),0.0,1.0);
    // kOpaque is the shield's alpha for tracks that are not tall
@@ -1157,7 +1189,7 @@ void DrawTrackName(
   The following classes define the subdivision of the area of the TrackPanel
   into cells with differing responses to mouse, keyboard, and scroll wheel
   events.
-  
+
   The classes defining the less inclusive areas are earlier, while those
   defining ever larger hierarchical groupings of cells are later.
 
@@ -1182,7 +1214,7 @@ void DrawTrackName(
   resize the channel views.
 
   Sixthly, divide each channel into one or more vertically stacked sub-views.
-  
+
   Lastly, split the area for each sub-view into a vertical ruler, and an area
   that displays the channel's own contents.
 
@@ -1207,7 +1239,7 @@ struct EmptyCell final : CommonTrackPanelCell {
       if ( iPass == TrackArtist::PassMargins ) {
          // Draw a margin area of TrackPanel
          auto dc = &context.dc;
-         
+
          AColor::TrackPanelBackground( dc, false );
          dc->DrawRectangle( rect );
       }
@@ -1300,7 +1332,65 @@ struct VRulersAndChannels final : TrackPanelGroup {
    wxCoord mLeftOffset;
 };
 
-// n channels with vertical rulers, alternating with n - 1 resizers;
+//Simply fills area using specified brush and outlines borders
+class EmptyPanelRect final : public CommonTrackPanelCell
+{
+   //Required to keep selection behaviour similar to others
+   std::shared_ptr<Track> mTrack;
+   int mFillBrushName;
+public:
+   explicit EmptyPanelRect(const std::shared_ptr<Track>& track, int fillBrushName)
+      : mTrack(track), mFillBrushName(fillBrushName)
+   {
+   }
+
+   ~EmptyPanelRect() { }
+
+   void Draw(TrackPanelDrawingContext& context,
+      const wxRect& rect, unsigned iPass) override
+   {
+      if (iPass == TrackArtist::PassBackground)
+      {
+         context.dc.SetPen(*wxTRANSPARENT_PEN);
+         AColor::UseThemeColour(&context.dc, mFillBrushName);
+         context.dc.DrawRectangle(rect);
+         wxRect bevel(rect.x, rect.y, rect.width - 1, rect.height - 1);
+         AColor::BevelTrackInfo(context.dc, true, bevel, false);
+      }
+   }
+
+   std::shared_ptr<Track> DoFindTrack() override
+   {
+       return mTrack;
+   }
+
+   std::vector<UIHandlePtr> HitTest(const TrackPanelMouseState& state,
+      const AudacityProject* pProject) override
+   {
+      return {};
+   }
+};
+
+//Simply place children one after another horizontally, without any specific logic
+struct HorizontalGroup final : TrackPanelGroup {
+
+   Refinement mRefinement;
+
+   HorizontalGroup(Refinement refinement)
+      : mRefinement(std::move(refinement))
+   {
+   }
+
+   Subdivision Children(const wxRect& /*rect*/) override
+   {
+      return { Axis::X, mRefinement };
+   }
+
+};
+
+
+// optional affordance areas, and n channels with vertical rulers,
+// alternating with n - 1 resizers;
 // each channel-ruler pair might be divided into multiple views
 struct ChannelGroup final : TrackPanelGroup {
    ChannelGroup( const std::shared_ptr< Track > &pTrack, wxCoord leftOffset )
@@ -1313,11 +1403,27 @@ struct ChannelGroup final : TrackPanelGroup {
       const auto channels = TrackList::Channels( mpTrack.get() );
       const auto pLast = *channels.rbegin();
       wxCoord yy = rect.GetTop();
-      for ( auto channel : channels ) {
+      auto heights = FindAdjustedChannelHeights(*mpTrack);
+      auto pHeight = heights.begin();
+      for ( auto channel : channels )
+      {
          auto &view = TrackView::Get( *channel );
-         auto height = view.GetHeight();
+         if (auto affordance = view.GetAffordanceControls())
+         {
+            auto panelRect = std::make_shared<EmptyPanelRect>(
+               channel->shared_from_this(),
+               channel->GetSelected() ? clrTrackInfoSelected : clrTrackInfo);
+            Refinement hgroup {
+               std::make_pair(rect.GetLeft() + 1, panelRect),
+               std::make_pair(mLeftOffset, affordance)
+            };
+            refinement.emplace_back(yy, std::make_shared<HorizontalGroup>(hgroup));
+            yy += kAffordancesAreaHeight;
+         }
+
+         auto height = *pHeight++;
          rect.SetTop( yy );
-         rect.SetHeight( height - kSeparatorThickness );
+         rect.SetHeight( height - kChannelSeparatorThickness );
          refinement.emplace_back( yy,
             std::make_shared< VRulersAndChannels >(
                channel->shared_from_this(),
@@ -1326,13 +1432,43 @@ struct ChannelGroup final : TrackPanelGroup {
          if ( channel != pLast ) {
             yy += height;
             refinement.emplace_back(
-               yy - kSeparatorThickness,
+               yy - kChannelSeparatorThickness,
                TrackPanelResizerCell::Get( *channel ).shared_from_this() );
          }
       }
 
       return { Axis::Y, std::move( refinement ) };
    }
+
+   void Draw(TrackPanelDrawingContext& context, const wxRect& rect, unsigned iPass) override
+   {
+      TrackPanelGroup::Draw(context, rect, iPass);
+      if (iPass == TrackArtist::PassFocus && mpTrack->IsSelected())
+      {
+         const auto channels = TrackList::Channels(mpTrack.get());
+         const auto pLast = *channels.rbegin();
+         wxCoord yy = rect.GetTop();
+         auto heights = FindAdjustedChannelHeights(*mpTrack);
+         auto pHeight = heights.begin();
+         for (auto channel : channels)
+         {
+            auto& view = TrackView::Get(*channel);
+            auto height = *pHeight++;
+            if (auto affordance = view.GetAffordanceControls())
+            {
+               height += kAffordancesAreaHeight;
+            }
+            auto trackRect = wxRect(
+               mLeftOffset,
+               yy,
+               rect.GetRight() - mLeftOffset,
+               height - kChannelSeparatorThickness);
+            TrackArt::DrawCursor(context, trackRect, mpTrack.get());
+            yy += height;
+         }
+      }
+   }
+
    std::shared_ptr< Track > mpTrack;
    wxCoord mLeftOffset;
 };
@@ -1393,15 +1529,15 @@ struct LabeledChannelGroup final : TrackPanelGroup {
             wxRect theRect = rect;
             auto &dc = context.dc;
             dc.SetBrush(*wxTRANSPARENT_BRUSH);
-            
+
             AColor::TrackFocusPen( &dc, 2 );
             dc.DrawRectangle(theRect);
             theRect.Deflate(1);
-            
+
             AColor::TrackFocusPen( &dc, 1 );
             dc.DrawRectangle(theRect);
             theRect.Deflate(1);
-            
+
             AColor::TrackFocusPen( &dc, 0 );
             dc.DrawRectangle(theRect);
          }
@@ -1446,7 +1582,7 @@ struct ResizingChannelGroup final : TrackPanelGroup {
    { return { Axis::Y, Refinement{
       { rect.GetTop(),
          std::make_shared< LabeledChannelGroup >( mpTrack, mLeftOffset ) },
-      { rect.GetTop() + rect.GetHeight() - kSeparatorThickness,
+      { rect.GetTop() + rect.GetHeight() - kTrackSeparatorThickness,
          TrackPanelResizerCell::Get(
             **TrackList::Channels( mpTrack.get() ).rbegin() ).shared_from_this()
       }
@@ -1519,7 +1655,7 @@ wxRect TrackPanel::FindTrackRect( const Track * target )
 {
    auto leader = *GetTracks()->FindLeader( target );
    if (!leader) {
-      return { 0, 0, 0, 0 };
+      return {};
    }
 
    return CellularPanel::FindRect( [&] ( TrackPanelNode &node ) {
@@ -1529,10 +1665,50 @@ wxRect TrackPanel::FindTrackRect( const Track * target )
    } );
 }
 
+wxRect TrackPanel::FindFocusedTrackRect( const Track * target )
+{
+   auto rect = FindTrackRect(target);
+   if (rect != wxRect{}) {
+      // Enlarge horizontally.
+      // PRL:  perhaps it's one pixel too much each side, including some gray
+      // beyond the yellow?
+      rect.x = 0;
+      GetClientSize(&rect.width, nullptr);
+
+      // Enlarge vertically, enough to enclose the yellow focus border pixels
+      // The the outermost ring of gray pixels is included on three sides
+      // but not the top (should that be fixed?)
+
+      // (Note that TrackPanel paints its focus over the "top margin" of the
+      // rectangle allotted to the track, according to TrackView::GetY() and
+      // TrackView::GetHeight(), but also over the margin of the next track.)
+
+      rect.height += kBottomMargin;
+      int dy = kTopMargin - 1;
+      rect.Inflate( 0, dy );
+
+      // Note that this rectangle does not coincide with any one of
+      // the nodes in the subdivision.
+   }
+   return rect;
+}
+
+std::vector<wxRect> TrackPanel::FindRulerRects( const Track *target )
+{
+   std::vector<wxRect> results;
+   if (target)
+      VisitCells( [&]( const wxRect &rect, TrackPanelCell &visited ) {
+         if (auto pRuler = dynamic_cast<const TrackVRulerControls*>(&visited);
+             pRuler && pRuler->FindTrack().get() == target)
+            results.push_back(rect);
+      } );
+   return results;
+}
+
 TrackPanelCell *TrackPanel::GetFocusedCell()
 {
    auto pTrack = TrackFocus::Get( *GetProject() ).Get();
-   return pTrack ? &TrackView::Get( *pTrack ) : nullptr;
+   return pTrack ? &TrackView::Get( *pTrack ) : GetBackgroundCell().get();
 }
 
 void TrackPanel::SetFocusedCell()
@@ -1543,36 +1719,8 @@ void TrackPanel::SetFocusedCell()
    KeyboardCapture::Capture(this);
 }
 
-void TrackPanel::OnTrackFocusChange( wxCommandEvent &event )
+void TrackPanel::OnTrackFocusChange(TrackFocusChangeMessage)
 {
-   event.Skip();
-   auto cell = GetFocusedCell();
-
-   if (cell) {
+   if (auto cell = GetFocusedCell())
       Refresh( false );
-   }
-}
-
-IsVisibleTrack::IsVisibleTrack(AudacityProject *project)
-   : mPanelRect {
-        wxPoint{ 0, ViewInfo::Get( *project ).vpos },
-        wxSize{
-           ViewInfo::Get( *project ).GetTracksUsableWidth(),
-           ViewInfo::Get( *project ).GetHeight()
-        }
-     }
-{}
-
-bool IsVisibleTrack::operator () (const Track *pTrack) const
-{
-   // Need to return true if this track or a later channel intersects
-   // the view
-   return
-   TrackList::Channels(pTrack).StartingWith(pTrack).any_of(
-      [this]( const Track *pT ) {
-         auto &view = TrackView::Get( *pT );
-         wxRect r(0, view.GetY(), 1, view.GetHeight());
-         return r.Intersects(mPanelRect);
-      }
-   );
 }
