@@ -17,24 +17,23 @@ Also acts as a factory.
 
 *//*******************************************************************/
 
-#include "../Audacity.h"
+
 #include "CommandType.h"
 
 #include <wx/string.h>
 
 OldStyleCommandType::OldStyleCommandType()
-   : mSymbol{}, mSignature{}
-{ }
+   : mSignature{}
+{
+}
 
 OldStyleCommandType::~OldStyleCommandType()
 {
 }
 
-ComponentInterfaceSymbol OldStyleCommandType::GetSymbol()
+ComponentInterfaceSymbol OldStyleCommandType::GetSymbol() const
 {
-   if (mSymbol.empty())
-      mSymbol = BuildName();
-   return mSymbol;
+   return BuildName();
 }
 
 CommandSignature &OldStyleCommandType::GetSignature()
