@@ -15,13 +15,12 @@
 
 *//*******************************************************************/
 
-#include "../Audacity.h" // for USE_SOUNDTOUCH
+
 
 #if USE_SOUNDTOUCH
 #include "ChangeTempo.h"
 
 #if USE_SBSMS
-#include "../../../lib-src/header-substitutes/sbsms.h"
 #include <wx/valgen.h>
 #endif
 
@@ -31,7 +30,6 @@
 #include <wx/checkbox.h>
 #include <wx/slider.h>
 
-#include "../Shuttle.h"
 #include "../ShuttleGui.h"
 #include "../widgets/valnum.h"
 #include "TimeWarper.h"
@@ -63,11 +61,13 @@ enum
 
 // Soundtouch is not reasonable below -99% or above 3000%.
 
-// Define keys, defaults, minimums, and maximums for the effect parameters
-//
-//     Name          Type     Key               Def   Min      Max      Scale
-Param( Percentage,   double,  wxT("Percentage"), 0.0,  -95.0,   3000.0,  1  );
-Param( UseSBSMS,     bool,    wxT("SBSMS"),     false, false,   true,    1  );
+const EffectParameterMethods& EffectChangeTempo::Parameters() const
+{
+   static CapturedParameters<EffectChangeTempo,
+      Percentage, UseSBSMS
+   > parameters;
+   return parameters;
+}
 
 // We warp the slider to go up to 400%, but user can enter higher values.
 static const double kSliderMax = 100.0;         // warped above zero to actually go up to 400%
@@ -92,7 +92,9 @@ END_EVENT_TABLE()
 
 EffectChangeTempo::EffectChangeTempo()
 {
-   m_PercentChange = DEF_Percentage;
+   // mUseSBSMS always defaults to false and its value is used only if USE_SBSMS
+   // is defined
+   Parameters().Reset(*this);
    m_FromBPM = 0.0; // indicates not yet set
    m_ToBPM = 0.0; // indicates not yet set
    m_FromLength = 0.0;
@@ -100,12 +102,6 @@ EffectChangeTempo::EffectChangeTempo()
 
    m_bLoopDetect = false;
 
-#if USE_SBSMS
-   mUseSBSMS = DEF_UseSBSMS;
-#else
-   mUseSBSMS = false;
-#endif
-
    SetLinearEffectFlag(true);
 }
 
@@ -115,71 +111,42 @@ EffectChangeTempo::~EffectChangeTempo()
 
 // ComponentInterface implementation
 
-ComponentInterfaceSymbol EffectChangeTempo::GetSymbol()
+ComponentInterfaceSymbol EffectChangeTempo::GetSymbol() const
 {
    return Symbol;
 }
 
-TranslatableString EffectChangeTempo::GetDescription()
+TranslatableString EffectChangeTempo::GetDescription() const
 {
    return XO("Changes the tempo of a selection without changing its pitch");
 }
 
-wxString EffectChangeTempo::ManualPage()
+ManualPageID EffectChangeTempo::ManualPage() const
 {
-   return wxT("Change_Tempo");
+   return L"Change_Tempo";
 }
 
 // EffectDefinitionInterface implementation
 
-EffectType EffectChangeTempo::GetType()
+EffectType EffectChangeTempo::GetType() const
 {
    return EffectTypeProcess;
 }
 
-bool EffectChangeTempo::SupportsAutomation()
+bool EffectChangeTempo::SupportsAutomation() const
 {
    return true;
 }
 
-// EffectClientInterface implementation
-bool EffectChangeTempo::DefineParams( ShuttleParams & S ){
-   S.SHUTTLE_PARAM( m_PercentChange, Percentage );
-   S.SHUTTLE_PARAM( mUseSBSMS, UseSBSMS );
-   return true;
-}
-
-bool EffectChangeTempo::GetAutomationParameters(CommandParameters & parms)
-{
-   parms.Write(KEY_Percentage, m_PercentChange);
-   parms.Write(KEY_UseSBSMS, mUseSBSMS);
-
-   return true;
-}
-
-bool EffectChangeTempo::SetAutomationParameters(CommandParameters & parms)
-{
-   ReadAndVerifyDouble(Percentage);
-   m_PercentChange = Percentage;
-
-#if USE_SBSMS
-   ReadAndVerifyBool(UseSBSMS);
-   mUseSBSMS = UseSBSMS;
-#else
-   mUseSBSMS = false;
-#endif
-
-   return true;
-}
-
 // Effect implementation
 
-double EffectChangeTempo::CalcPreviewInputLength(double previewLength)
+double EffectChangeTempo::CalcPreviewInputLength(
+   const EffectSettings &, double previewLength) const
 {
    return previewLength * (100.0 + m_PercentChange) / 100.0;
 }
 
-bool EffectChangeTempo::CheckWhetherSkipEffect()
+bool EffectChangeTempo::CheckWhetherSkipEffect(const EffectSettings &) const
 {
    return (m_PercentChange == 0.0);
 }
@@ -191,12 +158,10 @@ bool EffectChangeTempo::Init()
    m_FromLength = mT1 - mT0;
    m_ToLength = (m_FromLength * 100.0) / (100.0 + m_PercentChange);
 
-   mSoundTouch.reset();
-
    return true;
 }
 
-bool EffectChangeTempo::Process()
+bool EffectChangeTempo::Process(EffectInstance &, EffectSettings &settings)
 {
    bool success = false;
 
@@ -207,17 +172,20 @@ bool EffectChangeTempo::Process()
       EffectSBSMS proxy;
       proxy.mProxyEffectName = XO("High Quality Tempo Change");
       proxy.setParameters(tempoRatio, 1.0);
-      success = Delegate(proxy, *mUIParent, nullptr);
+      //! Already processing; don't make a dialog
+      success = Delegate(proxy, settings);
    }
    else
 #endif
    {
-      mSoundTouch = std::make_unique<soundtouch::SoundTouch>();
-      mSoundTouch->setTempoChange(m_PercentChange);
+      auto initer = [&](soundtouch::SoundTouch *soundtouch)
+      {
+         soundtouch->setTempoChange(m_PercentChange);
+      };
       double mT1Dashed = mT0 + (mT1 - mT0)/(m_PercentChange/100.0 + 1.0);
       RegionTimeWarper warper{ mT0, mT1,
          std::make_unique<LinearTimeWarper>(mT0, mT0, mT1, mT1Dashed )  };
-      success = EffectSoundTouch::ProcessWithTimeWarper(warper);
+      success = EffectSoundTouch::ProcessWithTimeWarper(initer, warper, false);
    }
 
    if(success)
@@ -226,7 +194,8 @@ bool EffectChangeTempo::Process()
    return success;
 }
 
-void EffectChangeTempo::PopulateOrExchange(ShuttleGui & S)
+std::unique_ptr<EffectUIValidator> EffectChangeTempo::PopulateOrExchange(
+   ShuttleGui & S, EffectInstance &, EffectSettingsAccess &)
 {
    enum { precision = 2 };
 
@@ -242,9 +211,8 @@ void EffectChangeTempo::PopulateOrExchange(ShuttleGui & S)
          m_pTextCtrl_PercentChange = S.Id(ID_PercentChange)
             .Validator<FloatingPointValidator<double>>(
                3, &m_PercentChange, NumValidatorStyle::THREE_TRAILING_ZEROES,
-               MIN_Percentage, MAX_Percentage
-            )
-            .AddTextBox(XXO("Percent C&hange:"), wxT(""), 12);
+               Percentage.min, Percentage.max )
+            .AddTextBox(XXO("Percent C&hange:"), L"", 12);
       }
       S.EndMultiColumn();
 
@@ -254,7 +222,7 @@ void EffectChangeTempo::PopulateOrExchange(ShuttleGui & S)
          m_pSlider_PercentChange = S.Id(ID_PercentChange)
             .Name(XO("Percent Change"))
             .Style(wxSL_HORIZONTAL)
-            .AddSlider( {}, 0, (int)kSliderMax, (int)MIN_Percentage);
+            .AddSlider( {}, 0, (int)kSliderMax, (int)Percentage.min);
       }
       S.EndHorizontalLay();
 
@@ -304,10 +272,9 @@ void EffectChangeTempo::PopulateOrExchange(ShuttleGui & S)
                   2, &m_ToLength, NumValidatorStyle::TWO_TRAILING_ZEROES,
                   // min and max need same precision as what we're validating (bug 963)
                   RoundValue( precision,
-                     (m_FromLength * 100.0) / (100.0 + MAX_Percentage) ),
+                     (m_FromLength * 100.0) / (100.0 + Percentage.max) ),
                   RoundValue( precision,
-                     (m_FromLength * 100.0) / (100.0 + MIN_Percentage) )
-               )
+                     (m_FromLength * 100.0) / (100.0 + Percentage.min) ) )
                /* i18n-hint: changing tempo "from" one value "to" another */
                .AddTextBox(XXC("t&o", "change tempo"), wxT(""), 12);
          }
@@ -328,10 +295,10 @@ void EffectChangeTempo::PopulateOrExchange(ShuttleGui & S)
    }
    S.EndVerticalLay();
 
-   return;
+   return nullptr;
 }
 
-bool EffectChangeTempo::TransferDataToWindow()
+bool EffectChangeTempo::TransferDataToWindow(const EffectSettings &)
 {
    // Reset from length because it can be changed by Preview
    m_FromLength = mT1 - mT0;
@@ -358,7 +325,7 @@ bool EffectChangeTempo::TransferDataToWindow()
    return true;
 }
 
-bool EffectChangeTempo::TransferDataFromWindow()
+bool EffectChangeTempo::TransferDataFromWindow(EffectSettings &)
 {
    if (!mUIParent->Validate() || !mUIParent->TransferDataFromWindow())
    {
