@@ -24,7 +24,7 @@
 
 *//*******************************************************************/
 
-#include "../Audacity.h"
+
 #include "ClickRemoval.h"
 #include "LoadEffects.h"
 
@@ -34,8 +34,7 @@
 #include <wx/slider.h>
 #include <wx/valgen.h>
 
-#include "../Prefs.h"
-#include "../Shuttle.h"
+#include "Prefs.h"
 #include "../ShuttleGui.h"
 #include "../widgets/AudacityMessageBox.h"
 #include "../widgets/valnum.h"
@@ -48,11 +47,13 @@ enum
    ID_Width
 };
 
-// Define keys, defaults, minimums, and maximums for the effect parameters
-//
-//     Name       Type     Key               Def      Min      Max      Scale
-Param( Threshold, int,     wxT("Threshold"),  200,     0,       900,     1  );
-Param( Width,     int,     wxT("Width"),      20,      0,       40,      1  );
+const EffectParameterMethods& EffectClickRemoval::Parameters() const
+{
+   static CapturedParameters<EffectClickRemoval,
+      Threshold, Width
+   > parameters;
+   return parameters;
+}
 
 const ComponentInterfaceSymbol EffectClickRemoval::Symbol
 { XO("Click Removal") };
@@ -68,8 +69,7 @@ END_EVENT_TABLE()
 
 EffectClickRemoval::EffectClickRemoval()
 {
-   mThresholdLevel = DEF_Threshold;
-   mClickWidth = DEF_Width;
+   Parameters().Reset(*this);
 
    SetLinearEffectFlag(false);
 
@@ -83,98 +83,36 @@ EffectClickRemoval::~EffectClickRemoval()
 
 // ComponentInterface implementation
 
-ComponentInterfaceSymbol EffectClickRemoval::GetSymbol()
+ComponentInterfaceSymbol EffectClickRemoval::GetSymbol() const
 {
    return Symbol;
 }
 
-TranslatableString EffectClickRemoval::GetDescription()
+TranslatableString EffectClickRemoval::GetDescription() const
 {
    return XO("Click Removal is designed to remove clicks on audio tracks");
 }
 
-wxString EffectClickRemoval::ManualPage()
+ManualPageID EffectClickRemoval::ManualPage() const
 {
-   return wxT("Click_Removal");
+   return L"Click_Removal";
 }
 
 // EffectDefinitionInterface implementation
 
-EffectType EffectClickRemoval::GetType()
+EffectType EffectClickRemoval::GetType() const
 {
    return EffectTypeProcess;
 }
 
-// EffectClientInterface implementation
-bool EffectClickRemoval::DefineParams( ShuttleParams & S ){
-   S.SHUTTLE_PARAM( mThresholdLevel, Threshold );
-   S.SHUTTLE_PARAM( mClickWidth, Width );
-   return true;
-}
-
-bool EffectClickRemoval::GetAutomationParameters(CommandParameters & parms)
-{
-   parms.Write(KEY_Threshold, mThresholdLevel);
-   parms.Write(KEY_Width, mClickWidth);
-
-   return true;
-}
-
-bool EffectClickRemoval::SetAutomationParameters(CommandParameters & parms)
-{
-   ReadAndVerifyInt(Threshold);
-   ReadAndVerifyInt(Width);
-
-   mThresholdLevel = Threshold;
-   mClickWidth = Width;
-
-   return true;
-}
-
 // Effect implementation
 
-bool EffectClickRemoval::CheckWhetherSkipEffect()
+bool EffectClickRemoval::CheckWhetherSkipEffect(const EffectSettings &) const
 {
    return ((mClickWidth == 0) || (mThresholdLevel == 0));
 }
 
-bool EffectClickRemoval::Startup()
-{
-   wxString base = wxT("/Effects/ClickRemoval/");
-
-   // Migrate settings from 2.1.0 or before
-
-   // Already migrated, so bail
-   if (gPrefs->Exists(base + wxT("Migrated")))
-   {
-      return true;
-   }
-
-   // Load the old "current" settings
-   if (gPrefs->Exists(base))
-   {
-      mThresholdLevel = gPrefs->Read(base + wxT("ClickThresholdLevel"), 200);
-      if ((mThresholdLevel < MIN_Threshold) || (mThresholdLevel > MAX_Threshold))
-      {  // corrupted Prefs?
-         mThresholdLevel = 0;  //Off-skip
-      }
-      mClickWidth = gPrefs->Read(base + wxT("ClickWidth"), 20);
-      if ((mClickWidth < MIN_Width) || (mClickWidth > MAX_Width))
-      {  // corrupted Prefs?
-         mClickWidth = 0;  //Off-skip
-      }
-
-      SaveUserPreset(GetCurrentSettingsGroup());
-
-      // Do not migrate again
-      gPrefs->Write(base + wxT("Migrated"), true);
-      gPrefs->Flush();
-   }
-
-   return true;
-}
-
-bool EffectClickRemoval::Process()
+bool EffectClickRemoval::Process(EffectInstance &, EffectSettings &)
 {
    this->CopyInputTracks(); // Set up mOutputTracks.
    bool bGoodResult = true;
@@ -233,7 +171,7 @@ bool EffectClickRemoval::ProcessOne(int count, WaveTrack * track, sampleCount st
    {
       auto block = limitSampleBufferSize( idealBlockLen, len - s );
 
-      track->Get((samplePtr) buffer.get(), floatSample, start + s, block);
+      track->GetFloats(buffer.get(), start + s, block);
 
       for (decltype(block) i = 0; i + windowSize / 2 < block; i += windowSize / 2)
       {
@@ -335,7 +273,8 @@ bool EffectClickRemoval::RemoveClicks(size_t len, float *buffer)
    return bResult;
 }
 
-void EffectClickRemoval::PopulateOrExchange(ShuttleGui & S)
+std::unique_ptr<EffectUIValidator> EffectClickRemoval::PopulateOrExchange(
+   ShuttleGui & S, EffectInstance &, EffectSettingsAccess &)
 {
    S.AddSpace(0, 5);
    S.SetBorder(10);
@@ -347,8 +286,7 @@ void EffectClickRemoval::PopulateOrExchange(ShuttleGui & S)
       mThreshT = S.Id(ID_Thresh)
          .Validator<IntegerValidator<int>>(
             &mThresholdLevel, NumValidatorStyle::DEFAULT,
-            MIN_Threshold, MAX_Threshold
-         )
+            Threshold.min, Threshold.max )
          .AddTextBox(XXO("&Threshold (lower is more sensitive):"),
                      wxT(""),
                      10);
@@ -358,12 +296,12 @@ void EffectClickRemoval::PopulateOrExchange(ShuttleGui & S)
          .Style(wxSL_HORIZONTAL)
          .Validator<wxGenericValidator>(&mThresholdLevel)
          .MinSize( { 150, -1 } )
-         .AddSlider( {}, mThresholdLevel, MAX_Threshold, MIN_Threshold);
+         .AddSlider( {}, mThresholdLevel, Threshold.max, Threshold.min);
 
       // Click width
       mWidthT = S.Id(ID_Width)
          .Validator<IntegerValidator<int>>(
-            &mClickWidth, NumValidatorStyle::DEFAULT, MIN_Width, MAX_Width)
+            &mClickWidth, NumValidatorStyle::DEFAULT, Width.min, Width.max)
          .AddTextBox(XXO("Max &Spike Width (higher is more sensitive):"),
                      wxT(""),
                      10);
@@ -373,14 +311,14 @@ void EffectClickRemoval::PopulateOrExchange(ShuttleGui & S)
          .Style(wxSL_HORIZONTAL)
          .Validator<wxGenericValidator>(&mClickWidth)
          .MinSize( { 150, -1 } )
-         .AddSlider( {}, mClickWidth, MAX_Width, MIN_Width);
+         .AddSlider( {}, mClickWidth, Width.max, Width.min);
    }
    S.EndMultiColumn();
 
-   return;
+   return nullptr;
 }
 
-bool EffectClickRemoval::TransferDataToWindow()
+bool EffectClickRemoval::TransferDataToWindow(const EffectSettings &)
 {
    if (!mUIParent->TransferDataToWindow())
    {
@@ -390,7 +328,7 @@ bool EffectClickRemoval::TransferDataToWindow()
    return true;
 }
 
-bool EffectClickRemoval::TransferDataFromWindow()
+bool EffectClickRemoval::TransferDataFromWindow(EffectSettings &)
 {
    if (!mUIParent->Validate() || !mUIParent->TransferDataFromWindow())
    {
