@@ -24,7 +24,7 @@
 
 *//*******************************************************************/
 
-#include "../Audacity.h"
+
 #include "Compressor.h"
 #include "LoadEffects.h"
 
@@ -33,21 +33,19 @@
 #include <wx/brush.h>
 #include <wx/checkbox.h>
 #include <wx/dcclient.h>
-#include <wx/dcmemory.h>
 #include <wx/intl.h>
 #include <wx/slider.h>
 #include <wx/stattext.h>
 
-#include "../AColor.h"
-#include "../Prefs.h"
-#include "../Shuttle.h"
+#include "AColor.h"
+#include "Prefs.h"
 #include "../ShuttleGui.h"
-#include "../Theme.h"
-#include "../float_cast.h"
+#include "Theme.h"
+#include "float_cast.h"
 #include "../widgets/Ruler.h"
 
 #include "../WaveTrack.h"
-#include "../AllThemeResources.h"
+#include "AllThemeResources.h"
 
 enum
 {
@@ -58,16 +56,16 @@ enum
    ID_Decay
 };
 
-// Define keys, defaults, minimums, and maximums for the effect parameters
-//
-//     Name          Type     Key                  Def      Min      Max      Scale
-Param( Threshold,    double,  wxT("Threshold"),     -12.0,   -60.0,   -1.0,    1   );
-Param( NoiseFloor,   double,  wxT("NoiseFloor"),    -40.0,   -80.0,   -20.0,   0.2   );
-Param( Ratio,        double,  wxT("Ratio"),         2.0,     1.1,     10.0,    10  );
-Param( AttackTime,   double,  wxT("AttackTime"),    0.2,     0.1,     5.0,     100 );
-Param( ReleaseTime,  double,  wxT("ReleaseTime"),   1.0,     1.0,     30.0,    10  );
-Param( Normalize,    bool,    wxT("Normalize"),     true,    false,   true,    1   );
-Param( UsePeak,      bool,    wxT("UsePeak"),       false,   false,   true,    1   );
+const EffectParameterMethods& EffectCompressor::Parameters() const
+{
+   static CapturedParameters<EffectCompressor,
+      Threshold, NoiseFloor, Ratio, // positive number > 1.0
+      AttackTime, // seconds
+      ReleaseTime, // seconds
+      Normalize, UsePeak
+   > parameters;
+   return parameters;
+}
 
 //----------------------------------------------------------------------------
 // EffectCompressor
@@ -84,13 +82,7 @@ END_EVENT_TABLE()
 
 EffectCompressor::EffectCompressor()
 {
-   mThresholdDB = DEF_Threshold;
-   mNoiseFloorDB = DEF_NoiseFloor;
-   mAttackTime = DEF_AttackTime;          // seconds
-   mDecayTime = DEF_ReleaseTime;          // seconds
-   mRatio = DEF_Ratio;                    // positive number > 1.0
-   mNormalize = DEF_Normalize;
-   mUsePeak = DEF_UsePeak;
+   Parameters().Reset(*this);
 
    mThreshold = 0.25;
    mNoiseFloor = 0.01;
@@ -106,109 +98,30 @@ EffectCompressor::~EffectCompressor()
 
 // ComponentInterface implementation
 
-ComponentInterfaceSymbol EffectCompressor::GetSymbol()
+ComponentInterfaceSymbol EffectCompressor::GetSymbol() const
 {
    return Symbol;
 }
 
-TranslatableString EffectCompressor::GetDescription()
+TranslatableString EffectCompressor::GetDescription() const
 {
    return XO("Compresses the dynamic range of audio");
 }
 
-wxString EffectCompressor::ManualPage()
+ManualPageID EffectCompressor::ManualPage() const
 {
-   return wxT("Compressor");
+   return L"Compressor";
 }
 
 // EffectDefinitionInterface implementation
 
-EffectType EffectCompressor::GetType()
+EffectType EffectCompressor::GetType() const
 {
    return EffectTypeProcess;
 }
 
-// EffectClientInterface implementation
-bool EffectCompressor::DefineParams( ShuttleParams & S ){
-   S.SHUTTLE_PARAM( mThresholdDB, Threshold );
-   S.SHUTTLE_PARAM( mNoiseFloorDB, NoiseFloor );
-   S.SHUTTLE_PARAM( mRatio, Ratio);
-   S.SHUTTLE_PARAM( mAttackTime, AttackTime);
-   S.SHUTTLE_PARAM( mDecayTime, ReleaseTime);
-   S.SHUTTLE_PARAM( mNormalize, Normalize);
-   S.SHUTTLE_PARAM( mUsePeak, UsePeak);
-   return true;
-}
-
-bool EffectCompressor::GetAutomationParameters(CommandParameters & parms)
-{
-   parms.Write(KEY_Threshold, mThresholdDB);
-   parms.Write(KEY_NoiseFloor, mNoiseFloorDB);
-   parms.Write(KEY_Ratio, mRatio);
-   parms.Write(KEY_AttackTime, mAttackTime);
-   parms.Write(KEY_ReleaseTime, mDecayTime);
-   parms.Write(KEY_Normalize, mNormalize);
-   parms.Write(KEY_UsePeak, mUsePeak);
-
-   return true;
-}
-
-bool EffectCompressor::SetAutomationParameters(CommandParameters & parms)
-{
-   ReadAndVerifyDouble(Threshold);
-   ReadAndVerifyDouble(NoiseFloor);
-   ReadAndVerifyDouble(Ratio);
-   ReadAndVerifyDouble(AttackTime);
-   ReadAndVerifyDouble(ReleaseTime);
-   ReadAndVerifyBool(Normalize);
-   ReadAndVerifyBool(UsePeak);
-
-   mThresholdDB = Threshold;
-   mNoiseFloorDB = NoiseFloor;
-   mRatio = Ratio;
-   mAttackTime = AttackTime;
-   mDecayTime = ReleaseTime;
-   mNormalize = Normalize;
-   mUsePeak = UsePeak;
-
-   return true;
-}
-
 // Effect Implementation
 
-bool EffectCompressor::Startup()
-{
-   wxString base = wxT("/Effects/Compressor/");
-
-   // Migrate settings from 2.1.0 or before
-
-   // Already migrated, so bail
-   if (gPrefs->Exists(base + wxT("Migrated")))
-   {
-      return true;
-   }
-
-   // Load the old "current" settings
-   if (gPrefs->Exists(base))
-   {
-      gPrefs->Read(base + wxT("ThresholdDB"), &mThresholdDB, -12.0f );
-      gPrefs->Read(base + wxT("NoiseFloorDB"), &mNoiseFloorDB, -40.0f );
-      gPrefs->Read(base + wxT("Ratio"), &mRatio, 2.0f );
-      gPrefs->Read(base + wxT("AttackTime"), &mAttackTime, 0.2f );
-      gPrefs->Read(base + wxT("DecayTime"), &mDecayTime, 1.0f );
-      gPrefs->Read(base + wxT("Normalize"), &mNormalize, true );
-      gPrefs->Read(base + wxT("UsePeak"), &mUsePeak, false );
-
-      SaveUserPreset(GetCurrentSettingsGroup());
-
-      // Do not migrate again
-      gPrefs->Write(base + wxT("Migrated"), true);
-      gPrefs->Flush();
-   }
-
-   return true;
-}
-
 namespace {
 
 TranslatableString ThresholdFormat( int value )
@@ -243,7 +156,8 @@ TranslatableString RatioLabelFormat( int sliderValue, double value )
 
 }
 
-void EffectCompressor::PopulateOrExchange(ShuttleGui & S)
+std::unique_ptr<EffectUIValidator> EffectCompressor::PopulateOrExchange(
+   ShuttleGui & S, EffectInstance &, EffectSettingsAccess &)
 {
    S.SetBorder(5);
 
@@ -273,9 +187,9 @@ void EffectCompressor::PopulateOrExchange(ShuttleGui & S)
             .Name(XO("Threshold"))
             .Style(wxSL_HORIZONTAL)
             .AddSlider( {},
-               DEF_Threshold * SCL_Threshold,
-               MAX_Threshold * SCL_Threshold,
-               MIN_Threshold * SCL_Threshold);
+               Threshold.def * Threshold.scale,
+               Threshold.max * Threshold.scale,
+               Threshold.min * Threshold.scale);
          mThresholdText = S.AddVariableText(ThresholdFormat(999), true,
             wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL);
 
@@ -285,9 +199,9 @@ void EffectCompressor::PopulateOrExchange(ShuttleGui & S)
             .Name(XO("Noise Floor"))
             .Style(wxSL_HORIZONTAL)
             .AddSlider( {},
-               DEF_NoiseFloor * SCL_NoiseFloor,
-               MAX_NoiseFloor * SCL_NoiseFloor,
-               MIN_NoiseFloor * SCL_NoiseFloor);
+               NoiseFloor.def * NoiseFloor.scale,
+               NoiseFloor.max * NoiseFloor.scale,
+               NoiseFloor.min * NoiseFloor.scale);
          mNoiseFloorText = S.AddVariableText(ThresholdFormat(999),
             true, wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL);
 
@@ -297,9 +211,9 @@ void EffectCompressor::PopulateOrExchange(ShuttleGui & S)
             .Name(XO("Ratio"))
             .Style(wxSL_HORIZONTAL)
             .AddSlider( {},
-               DEF_Ratio * SCL_Ratio,
-               MAX_Ratio * SCL_Ratio,
-               MIN_Ratio * SCL_Ratio);
+               Ratio.def * Ratio.scale,
+               Ratio.max * Ratio.scale,
+               Ratio.min * Ratio.scale);
          mRatioSlider->SetPageSize(5);
          mRatioText = S.AddVariableText(RatioTextFormat( 1, 99.9 ), true,
             wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL);
@@ -316,9 +230,9 @@ void EffectCompressor::PopulateOrExchange(ShuttleGui & S)
             .Name(XO("Attack Time"))
             .Style(wxSL_HORIZONTAL)
             .AddSlider( {},
-               DEF_AttackTime * SCL_AttackTime,
-               MAX_AttackTime * SCL_AttackTime,
-               MIN_AttackTime * SCL_AttackTime);
+               AttackTime.def * AttackTime.scale,
+               AttackTime.max * AttackTime.scale,
+               AttackTime.min * AttackTime.scale);
          mAttackText = S.AddVariableText(
             AttackTimeFormat(9.99),
             true, wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL);
@@ -335,9 +249,9 @@ void EffectCompressor::PopulateOrExchange(ShuttleGui & S)
             .Name(XO("Release Time"))
             .Style(wxSL_HORIZONTAL)
             .AddSlider( {},
-               DEF_ReleaseTime * SCL_ReleaseTime,
-               MAX_ReleaseTime * SCL_ReleaseTime,
-               MIN_ReleaseTime * SCL_ReleaseTime);
+               ReleaseTime.def * ReleaseTime.scale,
+               ReleaseTime.max * ReleaseTime.scale,
+               ReleaseTime.min * ReleaseTime.scale);
 
          mDecayText = S.AddVariableText(
             DecayTimeFormat(99.9),
@@ -351,22 +265,23 @@ void EffectCompressor::PopulateOrExchange(ShuttleGui & S)
    {
       /* i18n-hint: Make-up, i.e. correct for any reduction, rather than fabricate it.*/
       mGainCheckBox = S.AddCheckBox(XXO("Ma&ke-up gain for 0 dB after compressing"),
-                                    DEF_Normalize);
+            Normalize.def);
       /* i18n-hint: "Compress" here means reduce variations of sound volume,
        NOT related to file-size compression; Peaks means extremes in volume */
       mPeakCheckBox = S.AddCheckBox(XXO("C&ompress based on Peaks"),
-                                    DEF_UsePeak);
+            UsePeak.def);
    }
    S.EndHorizontalLay();
+   return nullptr;
 }
 
-bool EffectCompressor::TransferDataToWindow()
+bool EffectCompressor::TransferDataToWindow(const EffectSettings &)
 {
    mThresholdSlider->SetValue(lrint(mThresholdDB));
-   mNoiseFloorSlider->SetValue(lrint(mNoiseFloorDB * SCL_NoiseFloor));
-   mRatioSlider->SetValue(lrint(mRatio * SCL_Ratio));
-   mAttackSlider->SetValue(lrint(mAttackTime * SCL_AttackTime));
-   mDecaySlider->SetValue(lrint(mDecayTime * SCL_ReleaseTime));
+   mNoiseFloorSlider->SetValue(lrint(mNoiseFloorDB * NoiseFloor.scale));
+   mRatioSlider->SetValue(lrint(mRatio * Ratio.scale));
+   mAttackSlider->SetValue(lrint(mAttackTime * AttackTime.scale));
+   mDecaySlider->SetValue(lrint(mDecayTime * ReleaseTime.scale));
    mGainCheckBox->SetValue(mNormalize);
    mPeakCheckBox->SetValue(mUsePeak);
 
@@ -375,18 +290,23 @@ bool EffectCompressor::TransferDataToWindow()
    return true;
 }
 
-bool EffectCompressor::TransferDataFromWindow()
+bool EffectCompressor::TransferDataFromWindow(EffectSettings &)
 {
    if (!mUIParent->Validate())
    {
       return false;
    }
+   return DoTransferDataFromWindow();
+}
 
+bool EffectCompressor::DoTransferDataFromWindow()
+{
+   // To do:  eliminate this by using control validators instead
    mThresholdDB = (double) mThresholdSlider->GetValue();
-   mNoiseFloorDB = (double) mNoiseFloorSlider->GetValue() / SCL_NoiseFloor;
-   mRatio = (double) mRatioSlider->GetValue() / SCL_Ratio;
-   mAttackTime = (double) mAttackSlider->GetValue() / 100.0; //SCL_AttackTime;
-   mDecayTime = (double) mDecaySlider->GetValue() / SCL_ReleaseTime;
+   mNoiseFloorDB = (double) mNoiseFloorSlider->GetValue() / NoiseFloor.scale;
+   mRatio = (double) mRatioSlider->GetValue() / Ratio.scale;
+   mAttackTime = (double) mAttackSlider->GetValue() / 100.0; //AttackTime.scale;
+   mDecayTime = (double) mDecaySlider->GetValue() / ReleaseTime.scale;
    mNormalize = mGainCheckBox->GetValue();
    mUsePeak = mPeakCheckBox->GetValue();
 
@@ -663,7 +583,7 @@ float EffectCompressor::DoCompression(float value, double env)
 
 void EffectCompressor::OnSlider(wxCommandEvent & WXUNUSED(evt))
 {
-   TransferDataFromWindow();
+   DoTransferDataFromWindow();
    UpdateUI();
 }
 
