@@ -12,6 +12,9 @@
 #define __AUDACITY_EFFECT_COMPRESSOR__
 
 #include "TwoPassSimpleMono.h"
+#include "../ShuttleAutomation.h"
+#include "MemoryX.h"
+#include "../widgets/wxPanelWrapper.h"
 
 class wxCheckBox;
 class wxSlider;
@@ -19,9 +22,15 @@ class wxStaticText;
 class EffectCompressorPanel;
 class ShuttleGui;
 
+using Floats = ArrayOf<float>;
+using Doubles = ArrayOf<double>;
+
 class EffectCompressor final : public EffectTwoPassSimpleMono
 {
 public:
+   static inline EffectCompressor *
+   FetchParameters(EffectCompressor &e, EffectSettings &) { return &e; }
+
    static const ComponentInterfaceSymbol Symbol;
 
    EffectCompressor();
@@ -29,26 +38,22 @@ public:
 
    // ComponentInterface implementation
 
-   ComponentInterfaceSymbol GetSymbol() override;
-   TranslatableString GetDescription() override;
-   wxString ManualPage() override;
+   ComponentInterfaceSymbol GetSymbol() const override;
+   TranslatableString GetDescription()  const override;
+   ManualPageID ManualPage() const override;
 
    // EffectDefinitionInterface implementation
 
-   EffectType GetType() override;
-
-   // EffectClientInterface implementation
-
-   bool DefineParams( ShuttleParams & S ) override;
-   bool GetAutomationParameters(CommandParameters & parms) override;
-   bool SetAutomationParameters(CommandParameters & parms) override;
+   EffectType GetType() const override;
 
    // Effect implementation
 
-   bool Startup() override;
-   void PopulateOrExchange(ShuttleGui & S) override;
-   bool TransferDataToWindow() override;
-   bool TransferDataFromWindow() override;
+   std::unique_ptr<EffectUIValidator> PopulateOrExchange(
+      ShuttleGui & S, EffectInstance &instance, EffectSettingsAccess &access)
+   override;
+   bool DoTransferDataFromWindow();
+   bool TransferDataToWindow(const EffectSettings &settings) override;
+   bool TransferDataFromWindow(EffectSettings &settings) override;
 
 protected:
    // EffectTwoPassSimpleMono implementation
@@ -124,7 +129,23 @@ private:
    wxCheckBox *mGainCheckBox;
    wxCheckBox *mPeakCheckBox;
 
+   const EffectParameterMethods& Parameters() const override;
    DECLARE_EVENT_TABLE()
+
+static constexpr EffectParameter Threshold{ &EffectCompressor::mThresholdDB,
+   L"Threshold",     -12.0,   -60.0,   -1.0,    1   };
+static constexpr EffectParameter NoiseFloor{ &EffectCompressor::mNoiseFloorDB,
+   L"NoiseFloor",    -40.0,   -80.0,   -20.0,   0.2   };
+static constexpr EffectParameter Ratio{ &EffectCompressor::mRatio,
+   L"Ratio",         2.0,     1.1,     10.0,    10  };
+static constexpr EffectParameter AttackTime{ &EffectCompressor::mAttackTime,
+   L"AttackTime",    0.2,     0.1,     5.0,     100 };
+static constexpr EffectParameter ReleaseTime{ &EffectCompressor::mDecayTime,
+   L"ReleaseTime",   1.0,     1.0,     30.0,    10  };
+static constexpr EffectParameter Normalize{ &EffectCompressor::mNormalize,
+   L"Normalize",     true,    false,   true,    1   };
+static constexpr EffectParameter UsePeak{ &EffectCompressor::mUsePeak,
+   L"UsePeak",       false,   false,   true,    1   };
 };
 
 class EffectCompressorPanel final : public wxPanelWrapper
