@@ -15,12 +15,10 @@
 *//*******************************************************************/
 
 
-#include "../Audacity.h" // for rint from configwin.h
+
 #include "Normalize.h"
 #include "LoadEffects.h"
 
-#include "../Experimental.h"
-
 #include <math.h>
 
 #include <wx/checkbox.h>
@@ -28,21 +26,20 @@
 #include <wx/stattext.h>
 #include <wx/valgen.h>
 
-#include "../Prefs.h"
+#include "Prefs.h"
 #include "../ProjectFileManager.h"
-#include "../Shuttle.h"
 #include "../ShuttleGui.h"
 #include "../WaveTrack.h"
 #include "../widgets/valnum.h"
 #include "../widgets/ProgressDialog.h"
 
-// Define keys, defaults, minimums, and maximums for the effect parameters
-//
-//     Name         Type     Key                        Def      Min      Max   Scale
-Param( PeakLevel,   double,  wxT("PeakLevel"),           -1.0,    -145.0,  0.0,  1  );
-Param( RemoveDC,    bool,    wxT("RemoveDcOffset"),      true,    false,   true, 1  );
-Param( ApplyGain,   bool,    wxT("ApplyGain"),           true,    false,   true, 1  );
-Param( StereoInd,   bool,    wxT("StereoIndependent"),   false,   false,   true, 1  );
+const EffectParameterMethods& EffectNormalize::Parameters() const
+{
+   static CapturedParameters<EffectNormalize,
+      PeakLevel, ApplyGain, RemoveDC, StereoInd
+   > parameters;
+   return parameters;
+}
 
 const ComponentInterfaceSymbol EffectNormalize::Symbol
 { XO("Normalize") };
@@ -56,11 +53,7 @@ END_EVENT_TABLE()
 
 EffectNormalize::EffectNormalize()
 {
-   mPeakLevel = DEF_PeakLevel;
-   mDC = DEF_RemoveDC;
-   mGain = DEF_ApplyGain;
-   mStereoInd = DEF_StereoInd;
-
+   Parameters().Reset(*this);
    SetLinearEffectFlag(false);
 }
 
@@ -70,105 +63,36 @@ EffectNormalize::~EffectNormalize()
 
 // ComponentInterface implementation
 
-ComponentInterfaceSymbol EffectNormalize::GetSymbol()
+ComponentInterfaceSymbol EffectNormalize::GetSymbol() const
 {
    return Symbol;
 }
 
-TranslatableString EffectNormalize::GetDescription()
+TranslatableString EffectNormalize::GetDescription() const
 {
    return XO("Sets the peak amplitude of one or more tracks");
 }
 
-wxString EffectNormalize::ManualPage()
+ManualPageID EffectNormalize::ManualPage() const
 {
-   return wxT("Normalize");
+   return L"Normalize";
 }
 
 // EffectDefinitionInterface implementation
 
-EffectType EffectNormalize::GetType()
+EffectType EffectNormalize::GetType() const
 {
    return EffectTypeProcess;
 }
 
-// EffectClientInterface implementation
-bool EffectNormalize::DefineParams( ShuttleParams & S ){
-   S.SHUTTLE_PARAM( mPeakLevel, PeakLevel );
-   S.SHUTTLE_PARAM( mGain, ApplyGain );
-   S.SHUTTLE_PARAM( mDC, RemoveDC );
-   S.SHUTTLE_PARAM( mStereoInd, StereoInd );
-   return true;
-}
-
-bool EffectNormalize::GetAutomationParameters(CommandParameters & parms)
-{
-   parms.Write(KEY_PeakLevel, mPeakLevel);
-   parms.Write(KEY_ApplyGain, mGain);
-   parms.Write(KEY_RemoveDC, mDC);
-   parms.Write(KEY_StereoInd, mStereoInd);
-
-   return true;
-}
-
-bool EffectNormalize::SetAutomationParameters(CommandParameters & parms)
-{
-   ReadAndVerifyDouble(PeakLevel);
-   ReadAndVerifyBool(ApplyGain);
-   ReadAndVerifyBool(RemoveDC);
-   ReadAndVerifyBool(StereoInd);
-
-   mPeakLevel = PeakLevel;
-   mGain = ApplyGain;
-   mDC = RemoveDC;
-   mStereoInd = StereoInd;
-
-   return true;
-}
-
 // Effect implementation
 
-bool EffectNormalize::CheckWhetherSkipEffect()
+bool EffectNormalize::CheckWhetherSkipEffect(const EffectSettings &) const
 {
    return ((mGain == false) && (mDC == false));
 }
 
-bool EffectNormalize::Startup()
-{
-   wxString base = wxT("/Effects/Normalize/");
-
-   // Migrate settings from 2.1.0 or before
-
-   // Already migrated, so bail
-   if (gPrefs->Exists(base + wxT("Migrated")))
-   {
-      return true;
-   }
-
-   // Load the old "current" settings
-   if (gPrefs->Exists(base))
-   {
-      int boolProxy = gPrefs->Read(base + wxT("RemoveDcOffset"), 1);
-      mDC = (boolProxy == 1);
-      boolProxy = gPrefs->Read(base + wxT("Normalize"), 1);
-      mGain = (boolProxy == 1);
-      gPrefs->Read(base + wxT("Level"), &mPeakLevel, -1.0);
-      if(mPeakLevel > 0.0)  // this should never happen
-         mPeakLevel = -mPeakLevel;
-      boolProxy = gPrefs->Read(base + wxT("StereoIndependent"), 0L);
-      mStereoInd = (boolProxy == 1);
-
-      SaveUserPreset(GetCurrentSettingsGroup());
-
-      // Do not migrate again
-      gPrefs->Write(base + wxT("Migrated"), true);
-      gPrefs->Flush();
-   }
-
-   return true;
-}
-
-bool EffectNormalize::Process()
+bool EffectNormalize::Process(EffectInstance &, EffectSettings &)
 {
    if (mGain == false && mDC == false)
       return true;
@@ -177,7 +101,7 @@ bool EffectNormalize::Process()
    if( mGain )
    {
       // same value used for all tracks
-      ratio = DB_TO_LINEAR(TrapDouble(mPeakLevel, MIN_PeakLevel, MAX_PeakLevel));
+      ratio = DB_TO_LINEAR(std::clamp<double>(mPeakLevel, PeakLevel.min, PeakLevel.max));
    }
    else {
       ratio = 1.0;
@@ -287,7 +211,8 @@ bool EffectNormalize::Process()
    return bGoodResult;
 }
 
-void EffectNormalize::PopulateOrExchange(ShuttleGui & S)
+std::unique_ptr<EffectUIValidator> EffectNormalize::PopulateOrExchange(
+   ShuttleGui & S, EffectInstance &, EffectSettingsAccess &)
 {
    mCreating = true;
 
@@ -315,10 +240,9 @@ void EffectNormalize::PopulateOrExchange(ShuttleGui & S)
                      2,
                      &mPeakLevel,
                      NumValidatorStyle::ONE_TRAILING_ZERO,
-                     MIN_PeakLevel,
-                     MAX_PeakLevel
-                  )
-                  .AddTextBox( {}, wxT(""), 10);
+                     PeakLevel.min,
+                     PeakLevel.max )
+                  .AddTextBox( {}, L"", 10);
                mLeveldB = S.AddVariableText(XO("dB"), false,
                   wxALIGN_CENTER_VERTICAL | wxALIGN_LEFT);
                mWarning = S.AddVariableText( {}, false,
@@ -337,9 +261,10 @@ void EffectNormalize::PopulateOrExchange(ShuttleGui & S)
    }
    S.EndVerticalLay();
    mCreating = false;
+   return nullptr;
 }
 
-bool EffectNormalize::TransferDataToWindow()
+bool EffectNormalize::TransferDataToWindow(const EffectSettings &)
 {
    if (!mUIParent->TransferDataToWindow())
    {
@@ -351,7 +276,7 @@ bool EffectNormalize::TransferDataToWindow()
    return true;
 }
 
-bool EffectNormalize::TransferDataFromWindow()
+bool EffectNormalize::TransferDataFromWindow(EffectSettings &)
 {
    if (!mUIParent->Validate() || !mUIParent->TransferDataFromWindow())
    {
@@ -371,17 +296,6 @@ bool EffectNormalize::AnalyseTrack(const WaveTrack * track, const TranslatableSt
 
    if(mGain)
    {
-      // Since we need complete summary data, we need to block until the OD tasks are done for this track
-      // This is needed for track->GetMinMax
-      // TODO: should we restrict the flags to just the relevant block files (for selections)
-      while (ProjectFileManager::GetODFlags( *track )) {
-         // update the gui
-         if (ProgressResult::Cancelled == mProgress->Update(
-            0, XO("Waiting for waveform to finish computing...")) )
-            return false;
-         wxMilliSleep(100);
-      }
-
       // set mMin, mMax.  No progress bar here as it's fast.
       auto pair = track->GetMinMax(mCurT0, mCurT1); // may throw
       min = pair.first, max = pair.second;
@@ -448,7 +362,7 @@ bool EffectNormalize::AnalyseTrackData(const WaveTrack * track, const Translatab
       );
 
       //Get the samples from the track and put them in the buffer
-      track->Get((samplePtr) buffer.get(), floatSample, s, block, fillZero, true, &blockSamples);
+      track->GetFloats(buffer.get(), s, block, fillZero, true, &blockSamples);
       totalSamples += blockSamples;
 
       //Process the buffer.
@@ -508,7 +422,7 @@ bool EffectNormalize::ProcessOne(
       );
 
       //Get the samples from the track and put them in the buffer
-      track->Get((samplePtr) buffer.get(), floatSample, s, block);
+      track->GetFloats(buffer.get(), s, block);
 
       //Process the buffer.
       ProcessData(buffer.get(), block, offset);
