@@ -11,7 +11,7 @@ effect that uses SBSMS to do its processing (TimeScale)
 
 **********************************************************************/
 
-#include "../Audacity.h" // for USE_* macros
+
 
 #if USE_SBSMS
 #include "SBSMSEffect.h"
@@ -19,6 +19,8 @@ effect that uses SBSMS to do its processing (TimeScale)
 #include <math.h>
 
 #include "../LabelTrack.h"
+#include "../SyncLock.h"
+#include "../WaveClip.h"
 #include "../WaveTrack.h"
 #include "TimeWarper.h"
 
@@ -68,7 +70,7 @@ public:
    SBSMSEffectInterface(Resampler *resampler,
                         Slide *rateSlide, Slide *pitchSlide,
                         bool bReferenceInput,
-                        long samples, long preSamples,
+                        const SampleCountType samples, long preSamples,
                         SBSMSQuality *quality)
       : SBSMSInterfaceSliding(rateSlide,pitchSlide,bReferenceInput,samples,preSamples,quality)
    {
@@ -98,10 +100,10 @@ long resampleCB(void *cb_data, SBSMSFrame *data)
    // does not seem to let us report error codes, so use this roundabout to
    // stop the effect early.
    try {
-      r->leftTrack->Get(
-         (samplePtr)(r->leftBuffer.get()), floatSample, r->offset, blockSize);
-      r->rightTrack->Get(
-         (samplePtr)(r->rightBuffer.get()), floatSample, r->offset, blockSize);
+      r->leftTrack->GetFloats(
+         (r->leftBuffer.get()), r->offset, blockSize);
+      r->rightTrack->GetFloats(
+         (r->rightBuffer.get()), r->offset, blockSize);
    }
    catch ( ... ) {
       // Save the exception object for re-throw when out of the library
@@ -208,7 +210,7 @@ double EffectSBSMS::getRate(double rateStart, double rateEnd, SlideType slideTyp
    return slide.getRate(t);
 }
 
-bool EffectSBSMS::Process()
+bool EffectSBSMS::Process(EffectInstance &, EffectSettings &)
 {
    bool bGoodResult = true;
 
@@ -227,7 +229,8 @@ bool EffectSBSMS::Process()
 
    mOutputTracks->Leaders().VisitWhile( bGoodResult,
       [&](LabelTrack *lt, const Track::Fallthrough &fallthrough) {
-         if (!(lt->GetSelected() || (mustSync && lt->IsSyncLockSelected())))
+         if (!(lt->GetSelected() ||
+               (mustSync && SyncLock::IsSyncLockSelected(lt))))
             return fallthrough();
          if (!ProcessLabelTrack(lt))
             bGoodResult = false;
@@ -236,9 +239,9 @@ bool EffectSBSMS::Process()
          if (!leftTrack->GetSelected())
             return fallthrough();
 
-         //Get start and end times from track
-         mCurT0 = leftTrack->GetStartTime();
-         mCurT1 = leftTrack->GetEndTime();
+         //Get start and end times from selection
+         mCurT0 = mT0;
+         mCurT1 = mT1;
 
          //Set the current bounds to whichever left marker is
          //greater and whichever right marker is less
@@ -272,10 +275,6 @@ bool EffectSBSMS::Process()
 
                mCurTrackNum++; // Increment for rightTrack, too.
             }
-            const auto trackStart =
-               leftTrack->TimeToLongSamples(leftTrack->GetStartTime());
-            const auto trackEnd =
-               leftTrack->TimeToLongSamples(leftTrack->GetEndTime());
 
             // SBSMS has a fixed sample rate - we just convert to its sample rate and then convert back
             float srTrack = leftTrack->GetRate();
@@ -327,30 +326,13 @@ bool EffectSBSMS::Process()
               rb.sbsms = std::make_unique<SBSMS>(rightTrack ? 2 : 1, rb.quality.get(), true);
               rb.SBSMSBlockSize = rb.sbsms->getInputFrameSize();
               rb.SBSMSBuf.reinit(static_cast<size_t>(rb.SBSMSBlockSize), true);
-
-              // Note: width of getMaxPresamples() is only long.  Widen it
-              decltype(start) processPresamples = rb.quality->getMaxPresamples();
-              processPresamples =
-                 std::min(processPresamples,
-                          decltype(processPresamples)
-                             (( start - trackStart ).as_float() *
-                                 (srProcess/srTrack)));
-              auto trackPresamples = start - trackStart;
-              trackPresamples =
-                  std::min(trackPresamples,
-                           decltype(trackPresamples)
-                              (processPresamples.as_float() *
-                                  (srTrack/srProcess)));
-              rb.offset = start - trackPresamples;
-              rb.end = trackEnd;
+              rb.offset = start;
+              rb.end = end;
               rb.iface = std::make_unique<SBSMSEffectInterface>
                   (rb.resampler.get(), &rateSlide, &pitchSlide,
                    bPitchReferenceInput,
-                   // UNSAFE_SAMPLE_COUNT_TRUNCATION
-                   // The argument type is only long!
-                   static_cast<long> ( samplesToProcess.as_long_long() ),
-                   // This argument type is also only long!
-                   static_cast<long> ( processPresamples.as_long_long() ),
+                   static_cast<_sbsms_::SampleCountType>( samplesToProcess.as_long_long() ),
+                   0,
                    rb.quality.get());
             }
             
@@ -426,17 +408,14 @@ bool EffectSBSMS::Process()
             if(rightTrack)
                rb.outputRightTrack->Flush();
 
-            leftTrack->ClearAndPaste(mCurT0, mCurT1, rb.outputLeftTrack.get(),
-                                     true, false, warper.get());
-
+            Finalize(leftTrack, rb.outputLeftTrack.get(), warper.get());
             if(rightTrack)
-               rightTrack->ClearAndPaste(mCurT0, mCurT1, rb.outputRightTrack.get(),
-                                         true, false, warper.get());
+               Finalize(rightTrack, rb.outputRightTrack.get(), warper.get());
          }
          mCurTrackNum++;
       },
       [&](Track *t) {
-         if (mustSync && t->IsSyncLockSelected())
+         if (mustSync && SyncLock::IsSyncLockSelected(t))
          {
             t->SyncLockAdjust(mCurT1, mCurT0 + (mCurT1 - mCurT0) * mTotalStretch);
          }
@@ -445,13 +424,51 @@ bool EffectSBSMS::Process()
 
    if (bGoodResult) {
       ReplaceProcessedTracks(bGoodResult);
-
-      // Update selection
-      mT0 = mCurT0;
-      mT1 = mCurT0 + maxDuration;
    }
 
    return bGoodResult;
 }
 
+void EffectSBSMS::Finalize(WaveTrack* orig, WaveTrack* out, const TimeWarper *warper)
+{
+   // Silenced samples will be inserted in gaps between clips, so capture where these
+   // gaps are for later deletion
+   std::vector<std::pair<double, double>> gaps;
+   double last = mCurT0;
+   auto clips = orig->SortedClipArray();
+   auto front = clips.front();
+   auto back = clips.back();
+   for (auto &clip : clips) {
+      auto st = clip->GetPlayStartTime();
+      auto et = clip->GetPlayEndTime();
+
+      if (st >= mCurT0 || et < mCurT1) {
+         if (mCurT0 < st && clip == front) {
+            gaps.push_back(std::make_pair(mCurT0, st));
+         }
+         else if (last < st && mCurT0 <= last ) {
+            gaps.push_back(std::make_pair(last, st));
+         }
+
+         if (et < mCurT1 && clip == back) {
+            gaps.push_back(std::make_pair(et, mCurT1));
+         }
+      }
+      last = et;
+   }
+
+   // Take the output track and insert it in place of the original sample data
+   orig->ClearAndPaste(mCurT0, mCurT1, out, true, true, warper);
+
+   // Finally, recreate the gaps
+   for (auto gap : gaps) {
+      auto st = orig->LongSamplesToTime(orig->TimeToLongSamples(gap.first));
+      auto et = orig->LongSamplesToTime(orig->TimeToLongSamples(gap.second));
+      if (st >= mCurT0 && et <= mCurT1 && st != et)
+      {
+         orig->SplitDelete(warper->Warp(st), warper->Warp(et));
+      }
+   }
+}
+
 #endif
