@@ -11,11 +11,12 @@
 #ifndef __AUDACITY_EFFECT_TIMESCALE__
 #define __AUDACITY_EFFECT_TIMESCALE__
 
-#include "../Audacity.h" // for USE_* macros
+
 
 #if USE_SBSMS
 
 #include "SBSMSEffect.h"
+#include "../ShuttleAutomation.h"
 
 class wxSlider;
 class wxTextCtrl;
@@ -24,6 +25,8 @@ class ShuttleGui;
 class EffectTimeScale final : public EffectSBSMS
 {
 public:
+   static inline EffectTimeScale *
+   FetchParameters(EffectTimeScale &e, EffectSettings &) { return &e; }
    static const ComponentInterfaceSymbol Symbol;
 
    EffectTimeScale();
@@ -31,36 +34,32 @@ public:
 
    // ComponentInterface implementation
 
-   ComponentInterfaceSymbol GetSymbol() override;
-   TranslatableString GetDescription() override;
-   wxString ManualPage() override;
+   ComponentInterfaceSymbol GetSymbol() const override;
+   TranslatableString GetDescription() const override;
+   ManualPageID ManualPage() const override;
 
    // EffectDefinitionInterface implementation
 
-   EffectType GetType() override;
-
-   // EffectClientInterface implementation
-
-   bool DefineParams( ShuttleParams & S ) override;
-   bool GetAutomationParameters(CommandParameters & parms) override;
-   bool SetAutomationParameters(CommandParameters & parms) override;
+   EffectType GetType() const override;
 
    // Effect implementation
 
-   bool Init() override;
-   void Preview(bool dryOnly) override;
-   bool Process() override;
-   void PopulateOrExchange(ShuttleGui & S) override;
-   bool TransferDataToWindow() override;
-   bool TransferDataFromWindow() override;
-   double CalcPreviewInputLength(double previewLength) override;
+   void Preview(EffectSettingsAccess &access, bool dryOnly) override;
+   bool Process(EffectInstance &instance, EffectSettings &settings) override;
+   std::unique_ptr<EffectUIValidator> PopulateOrExchange(
+      ShuttleGui & S, EffectInstance &instance, EffectSettingsAccess &access)
+   override;
+   bool TransferDataToWindow(const EffectSettings &settings) override;
+   bool TransferDataFromWindow(EffectSettings &settings) override;
+   double CalcPreviewInputLength(
+      const EffectSettings &settings, double previewLength) const override;
 
 private:
    // EffectTimeScale implementation
 
-   inline double PercentChangeToRatio(double percentChange);
-   inline double HalfStepsToPercentChange(double halfSteps);
-   inline double PercentChangeToHalfSteps(double percentChange);
+   static inline double PercentChangeToRatio(double percentChange);
+   static inline double HalfStepsToPercentChange(double halfSteps);
+   static inline double PercentChangeToHalfSteps(double percentChange);
 
    void OnText_RatePercentChangeStart(wxCommandEvent & evt);
    void OnText_RatePercentChangeEnd(wxCommandEvent & evt);
@@ -81,7 +80,6 @@ private:
    void Update_Slider_RatePercentChangeStart();
    void Update_Slider_RatePercentChangeEnd();
 
-private:
    bool bPreview;
    double previewSelectedDuration;
    SlideType slideTypeRate;
@@ -102,7 +100,21 @@ private:
    wxTextCtrl *m_pTextCtrl_PitchPercentChangeStart;
    wxTextCtrl *m_pTextCtrl_PitchPercentChangeEnd;
 
+   const EffectParameterMethods& Parameters() const override;
    DECLARE_EVENT_TABLE()
+
+static constexpr EffectParameter RatePercentStart{ &EffectTimeScale::m_RatePercentChangeStart,
+   L"RatePercentChangeStart",  0.0,  -90.0,   500,   1  };
+static constexpr EffectParameter RatePercentEnd{ &EffectTimeScale::m_RatePercentChangeEnd,
+   L"RatePercentChangeEnd",    0.0,  -90.0,   500,   1  };
+static constexpr EffectParameter HalfStepsStart{ &EffectTimeScale::m_PitchHalfStepsStart,
+   L"PitchHalfStepsStart",     0.0,  -12.0,   12.0,  1  };
+static constexpr EffectParameter HalfStepsEnd{ &EffectTimeScale::m_PitchHalfStepsEnd,
+   L"PitchHalfStepsEnd",       0.0,  -12.0,   12.0,  1  };
+static constexpr EffectParameter PitchPercentStart{ &EffectTimeScale::m_PitchPercentChangeStart,
+   L"PitchPercentChangeStart", 0.0,  -50.0,   100.0, 1  };
+static constexpr EffectParameter PitchPercentEnd{ &EffectTimeScale::m_PitchPercentChangeEnd,
+   L"PitchPercentChangeEnd",   0.0,  -50.0,   100.0, 1  };
 };
 
 #endif // __AUDACITY_EFFECT_TIMESCALE
