@@ -18,6 +18,7 @@
 #define __AUDACITY_EFFECT_TRUNC_SILENCE__
 
 #include "Effect.h"
+#include "../ShuttleAutomation.h"
 
 class ShuttleGui;
 class wxChoice;
@@ -26,9 +27,11 @@ class wxCheckBox;
 
 class RegionList;
 
-class EffectTruncSilence final : public Effect
+class EffectTruncSilence final : public StatefulEffect
 {
 public:
+   static inline EffectTruncSilence *
+   FetchParameters(EffectTruncSilence &e, EffectSettings &) { return &e; }
    static const ComponentInterfaceSymbol Symbol;
 
    EffectTruncSilence();
@@ -36,24 +39,20 @@ public:
 
    // ComponentInterface implementation
 
-   ComponentInterfaceSymbol GetSymbol() override;
-   TranslatableString GetDescription() override;
-   wxString ManualPage() override;
+   ComponentInterfaceSymbol GetSymbol() const override;
+   TranslatableString GetDescription() const override;
+   ManualPageID ManualPage() const override;
 
    // EffectDefinitionInterface implementation
 
-   EffectType GetType() override;
-
-   // EffectClientInterface implementation
-
-   bool DefineParams( ShuttleParams & S ) override;
-   bool GetAutomationParameters(CommandParameters & parms) override;
-   bool SetAutomationParameters(CommandParameters & parms) override;
+   EffectType GetType() const override;
+   bool LoadSettings(
+      const CommandParameters & parms, EffectSettings &settings) const override;
 
    // Effect implementation
 
-   double CalcPreviewInputLength(double previewLength) override;
-   bool Startup() override;
+   double CalcPreviewInputLength(
+      const EffectSettings &settings, double previewLength) const override;
 
    // Analyze a single track to find silences
    // If inputLength is not NULL we are calculating the minimum
@@ -65,12 +64,14 @@ public:
                         sampleCount* index,
                         int whichTrack,
                         double* inputLength = NULL,
-                        double* minInputLength = NULL);
+                        double* minInputLength = NULL) const;
 
-   bool Process() override;
-   void PopulateOrExchange(ShuttleGui & S) override;
-   bool TransferDataToWindow() override;
-   bool TransferDataFromWindow() override;
+   bool Process(EffectInstance &instance, EffectSettings &settings) override;
+   std::unique_ptr<EffectUIValidator> PopulateOrExchange(
+      ShuttleGui & S, EffectInstance &instance, EffectSettingsAccess &access)
+   override;
+   bool TransferDataToWindow(const EffectSettings &settings) override;
+   bool TransferDataFromWindow(EffectSettings &settings) override;
 
 private:
    // EffectTruncSilence implementation
@@ -91,8 +92,6 @@ private:
       (const RegionList &silences, unsigned iGroup, unsigned nGroups, Track *firstTrack, Track *lastTrack,
        double &totalCutLen);
 
-private:
-
    double mThresholdDB {} ;
    int mActionIndex;
    double mInitialAllowedSilence;
@@ -109,7 +108,30 @@ private:
    wxTextCtrl *mSilenceCompressPercentT;
    wxCheckBox *mIndependent;
 
+   const EffectParameterMethods& Parameters() const override;
    DECLARE_EVENT_TABLE()
+
+   enum kActions
+   {
+      kTruncate,
+      kCompress,
+      nActions
+   };
+
+   static const EnumValueSymbol kActionStrings[nActions];
+
+static constexpr EffectParameter Threshold{ &EffectTruncSilence::mThresholdDB,
+   L"Threshold",  -20.0,      -80.0,   -20.0,                     1  };
+static constexpr EnumParameter ActIndex{ &EffectTruncSilence::mActionIndex,
+   L"Action",     (int)kTruncate,  0,       nActions - 1,           1, kActionStrings, nActions };
+static constexpr EffectParameter Minimum{ &EffectTruncSilence::mInitialAllowedSilence,
+   L"Minimum",    0.5,        0.001,   10000.0,                   1  };
+static constexpr EffectParameter Truncate{ &EffectTruncSilence::mTruncLongestAllowedSilence,
+   L"Truncate",   0.5,        0.0,     10000.0,                   1  };
+static constexpr EffectParameter Compress{ &EffectTruncSilence::mSilenceCompressPercent,
+   L"Compress",   50.0,       0.0,     99.9,                      1  };
+static constexpr EffectParameter Independent{ &EffectTruncSilence::mbIndependent,
+   L"Independent", false,     false,   true,                      1  };
 };
 
 #endif
