@@ -59,7 +59,7 @@
 
 *//********************************************************************/
 
-#include "../Audacity.h" // for USE_* macros
+
 #include "ExportMP3.h"
 
 #include <wx/app.h>
@@ -79,19 +79,22 @@
 #include <wx/utils.h>
 #include <wx/window.h>
 
-#include "../FileNames.h"
-#include "../float_cast.h"
-#include "../Mix.h"
-#include "../Prefs.h"
+#include "FileNames.h"
+#include "float_cast.h"
+#include "Mix.h"
+#include "Prefs.h"
+#include "ProjectRate.h"
 #include "../ProjectSettings.h"
 #include "../ProjectWindow.h"
+#include "../SelectFile.h"
 #include "../ShuttleGui.h"
 #include "../Tags.h"
-#include "../Track.h"
+#include "Track.h"
 #include "../widgets/HelpSystem.h"
 #include "../widgets/AudacityMessageBox.h"
 #include "../widgets/ProgressDialog.h"
-#include "../wxFileNameWrapper.h"
+#include "wxFileNameWrapper.h"
+#include "Project.h"
 
 #include "Export.h"
 
@@ -114,8 +117,8 @@ enum MP3ChannelMode : unsigned {
 enum : int {
    QUALITY_2 = 2,
 
-   ROUTINE_FAST = 0,
-   ROUTINE_STANDARD = 1,
+   //ROUTINE_FAST = 0,
+   //ROUTINE_STANDARD = 1,
 
    PRESET_INSANE = 0,
    PRESET_EXTREME = 1,
@@ -180,12 +183,12 @@ static const TranslatableStrings varRateNames {
    XO("65-105 kbps"),
    XO("45-85 kbps (Smaller files)"),
 };
-
+/*
 static const TranslatableStrings varModeNames {
    XO("Fast"),
    XO("Standard"),
 };
-
+*/
 static const TranslatableStrings setRateNames {
    /* i18n-hint: Slightly humorous - as in use an insane precision with MP3.*/
    XO("Insane, 320 kbps"),
@@ -251,7 +254,7 @@ private:
    wxRadioButton *mABR;
    wxRadioButton *mCBR;
    wxChoice *mRate;
-   wxChoice *mMode;
+   //wxChoice *mMode;
 
    long mSetRate;
    long mVbrRate;
@@ -332,7 +335,7 @@ void ExportMP3Options::PopulateOrExchange(ShuttleGui & S)
 
    const TranslatableStrings *choices = nullptr;
    const std::vector< int > *codes = nullptr;
-   bool enable;
+   //bool enable;
    int defrate;
 
    S.StartVerticalLay();
@@ -345,18 +348,25 @@ void ExportMP3Options::PopulateOrExchange(ShuttleGui & S)
             S.StartTwoColumn();
             {
                S.AddPrompt(XXO("Bit Rate Mode:"));
-               S.StartHorizontalLay();
+
+               // Bug 2692: Place button group in panel so tabbing will work and,
+               // on the Mac, VoiceOver will announce as radio buttons.
+               S.StartPanel();
                {
-                  S.StartRadioButtonGroup(MP3RateModeSetting);
+                  S.StartHorizontalLay();
                   {
-                     mSET = S.Id(ID_SET).TieRadioButton();
-                     mVBR = S.Id(ID_VBR).TieRadioButton();
-                     mABR = S.Id(ID_ABR).TieRadioButton();
-                     mCBR = S.Id(ID_CBR).TieRadioButton();
+                     S.StartRadioButtonGroup(MP3RateModeSetting);
+                     {
+                        mSET = S.Id(ID_SET).TieRadioButton();
+                        mVBR = S.Id(ID_VBR).TieRadioButton();
+                        mABR = S.Id(ID_ABR).TieRadioButton();
+                        mCBR = S.Id(ID_CBR).TieRadioButton();
+                     }
+                     S.EndRadioButtonGroup();
                   }
-                  S.EndRadioButtonGroup();
+                  S.EndHorizontalLay();
                }
-               S.EndHorizontalLay();
+               S.EndPanel();
 
                /* PRL: unfortunately this bit of procedural code must
                 interrupt the mostly-declarative dialog description, until
@@ -367,20 +377,20 @@ void ExportMP3Options::PopulateOrExchange(ShuttleGui & S)
                switch( MP3RateModeSetting.ReadEnum() ) {
                   case MODE_SET:
                      choices = &setRateNames;
-                     enable = true;
+                     //enable = true;
                      defrate = mSetRate;
                      break;
 
                   case MODE_VBR:
                      choices = &varRateNames;
-                     enable = true;
+                     //enable = true;
                      defrate = mVbrRate;
                      break;
 
                   case MODE_ABR:
                      choices = &fixRateNames;
                      codes = &fixRateValues;
-                     enable = false;
+                     //enable = false;
                      defrate = mAbrRate;
                      break;
 
@@ -388,39 +398,51 @@ void ExportMP3Options::PopulateOrExchange(ShuttleGui & S)
                   default:
                      choices = &fixRateNames;
                      codes = &fixRateValues;
-                     enable = false;
+                     //enable = false;
                      defrate = mCbrRate;
                      break;
                }
 
+               IntSetting Setting{ L"/FileFormats/MP3Bitrate", defrate };
+
                mRate = S.Id(ID_QUALITY).TieNumberAsChoice(
                   XXO("Quality"),
-                  { wxT("/FileFormats/MP3Bitrate"), defrate },
+                  Setting,
                   *choices,
                   codes
                );
-
+               /*
                mMode = S.Disable(!enable)
                   .TieNumberAsChoice(
                      XXO("Variable Speed:"),
                      { wxT("/FileFormats/MP3VarMode"), ROUTINE_FAST },
                      varModeNames );
-   
+               */
                S.AddPrompt(XXO("Channel Mode:"));
-               S.StartMultiColumn(3, wxEXPAND);
+               S.StartMultiColumn(2, wxEXPAND);
                {
-                  S.StartRadioButtonGroup(MP3ChannelModeSetting);
+                  // Bug 2692: Place button group in panel so tabbing will work and,
+                  // on the Mac, VoiceOver will announce as radio buttons.
+                  S.StartPanel();
                   {
-                     mJoint = S.Disable(mono)
-                        .TieRadioButton();
-                     mStereo = S.Disable(mono)
-                        .TieRadioButton();
+                     S.StartHorizontalLay();
+                     {
+                        S.StartRadioButtonGroup(MP3ChannelModeSetting);
+                        {
+                           mJoint = S.Disable(mono)
+                              .TieRadioButton();
+                           mStereo = S.Disable(mono)
+                              .TieRadioButton();
+                        }
+                        S.EndRadioButtonGroup();
+                     }
+                     S.EndHorizontalLay();
                   }
-                  S.EndRadioButtonGroup();
+                  S.EndPanel();
 
                   mMono = S.Id(ID_MONO).AddCheckBox(XXO("Force export to mono"), mono);
                }
-               S.EndTwoColumn();
+               S.EndMultiColumn();
             }
             S.EndTwoColumn();
          }
@@ -483,7 +505,7 @@ void ExportMP3Options::OnSET(wxCommandEvent& WXUNUSED(event))
 
    mRate->SetSelection(ValidateValue(setRateNames.size(), mSetRate, 2));
    mRate->Refresh();
-   mMode->Enable(true);
+   //mMode->Enable(true);
 }
 
 ///
@@ -494,7 +516,7 @@ void ExportMP3Options::OnVBR(wxCommandEvent& WXUNUSED(event))
 
    mRate->SetSelection(ValidateValue(varRateNames.size(), mVbrRate, 2));
    mRate->Refresh();
-   mMode->Enable(true);
+   //mMode->Enable(true);
 }
 
 ///
@@ -505,7 +527,7 @@ void ExportMP3Options::OnABR(wxCommandEvent& WXUNUSED(event))
 
    mRate->SetSelection(ValidateIndex(fixRateValues, mAbrRate, 10));
    mRate->Refresh();
-   mMode->Enable(false);
+   //mMode->Enable(false);
 }
 
 ///
@@ -516,7 +538,7 @@ void ExportMP3Options::OnCBR(wxCommandEvent& WXUNUSED(event))
 
    mRate->SetSelection(ValidateIndex(fixRateValues, mCbrRate, 10));
    mRate->Refresh();
-   mMode->Enable(false);
+   //mMode->Enable(false);
 }
 
 void ExportMP3Options::OnQuality(wxCommandEvent& WXUNUSED(event))
@@ -641,7 +663,7 @@ public:
        * "Where would I find the file %s" instead if you want. */
       auto question = XO("Where is %s?").Format( mName );
 
-      wxString path = FileNames::SelectFile(FileNames::Operation::_None,
+      wxString path = SelectFile(FileNames::Operation::_None,
          question,
             mLibPath.GetPath(),
             mLibPath.GetName(),
@@ -657,7 +679,7 @@ public:
 
    void OnDownload(wxCommandEvent & WXUNUSED(event))
    {
-      HelpSystem::ShowHelp(this, wxT("FAQ:Installing_the_LAME_MP3_Encoder"));
+      HelpSystem::ShowHelp(this, L"FAQ:Installing_the_LAME_MP3_Encoder");
    }
 
    wxString GetLibPath()
@@ -800,7 +822,7 @@ public:
    /* These global settings keep state over the life of the object */
    void SetMode(int mode);
    void SetBitrate(int rate);
-   void SetQuality(int q, int r);
+   void SetQuality(int q/*, int r*/);
    void SetChannel(int mode);
 
    /* Virtual methods that must be supplied by library interfaces */
@@ -854,7 +876,7 @@ private:
    int mMode;
    int mBitrate;
    int mQuality;
-   int mRoutine;
+   //int mRoutine;
    int mChannel;
 
 #ifndef DISABLE_DYNAMIC_LOADING_LAME
@@ -906,7 +928,7 @@ MP3Exporter::MP3Exporter()
 // We could use #defines rather than this variable.
 // The idea of the variable is that if we wanted, we could allow
 // a dynamic override of the library, e.g. with a newer faster version,
-// or to fix CVEs in the underlying librray.
+// or to fix CVEs in the underlying library.
 // for now though the 'variable' is a constant.
 #ifdef MP3_EXPORT_BUILT_IN
    mLibIsExternal = false;
@@ -930,7 +952,7 @@ MP3Exporter::MP3Exporter()
    mQuality = QUALITY_2;
    mChannel = CHANNEL_STEREO;
    mMode = MODE_CBR;
-   mRoutine = ROUTINE_FAST;
+   //mRoutine = ROUTINE_FAST;
 }
 
 MP3Exporter::~MP3Exporter()
@@ -1055,10 +1077,10 @@ void MP3Exporter::SetBitrate(int rate)
    mBitrate = rate;
 }
 
-void MP3Exporter::SetQuality(int q, int r)
+void MP3Exporter::SetQuality(int q/*, int r*/)
 {
    mQuality = q;
-   mRoutine = r;
+   //mRoutine = r;
 }
 
 void MP3Exporter::SetChannel(int mode)
@@ -1295,8 +1317,8 @@ int MP3Exporter::InitializeStream(unsigned channels, int sampleRate)
          if (mQuality == PRESET_INSANE) {
             preset = INSANE;
          }
-         else if (mRoutine == ROUTINE_FAST) {
-            if (mQuality == PRESET_EXTREME) {
+         //else if (mRoutine == ROUTINE_FAST) {
+            else if (mQuality == PRESET_EXTREME) {
                preset = EXTREME_FAST;
             }
             else if (mQuality == PRESET_STANDARD) {
@@ -1305,7 +1327,8 @@ int MP3Exporter::InitializeStream(unsigned channels, int sampleRate)
             else {
                preset = 1007;    // Not defined until 3.96
             }
-         }
+         //}
+         /*
          else {
             if (mQuality == PRESET_EXTREME) {
                preset = EXTREME;
@@ -1317,13 +1340,13 @@ int MP3Exporter::InitializeStream(unsigned channels, int sampleRate)
                preset = 1006;    // Not defined until 3.96
             }
          }
-
+         */
          lame_set_preset(mGF, preset);
       }
       break;
 
       case MODE_VBR:
-         lame_set_VBR(mGF, (mRoutine == ROUTINE_STANDARD ? vbr_rh : vbr_mtrh ));
+         lame_set_VBR(mGF, vbr_mtrh );
          lame_set_VBR_q(mGF, mQuality);
       break;
 
@@ -1592,7 +1615,7 @@ static void dump_config( 	lame_global_flags*	gfp )
       case JOINT_STEREO: wxPrintf(wxT( "Joint-Stereo\n" )); break;
       case DUAL_CHANNEL: wxPrintf(wxT( "Forced Stereo\n" )); break;
       case MONO:         wxPrintf(wxT( "Mono\n" )); break;
-      case NOT_SET:      /* FALLTROUGH */
+      case NOT_SET:      /* FALLTHROUGH */
       default:           wxPrintf(wxT( "Error (unknown)\n" )); break;
    }
 
@@ -1681,7 +1704,7 @@ public:
 
    void OptionsCreate(ShuttleGui &S, int format) override;
    ProgressResult Export(AudacityProject *project,
-               std::unique_ptr<ProgressDialog> &pDialog,
+               std::unique_ptr<BasicUI::ProgressDialog> &pDialog,
                unsigned channels,
                const wxFileNameWrapper &fName,
                bool selectedOnly,
@@ -1739,7 +1762,7 @@ int ExportMP3::SetNumExportChannels()
 
 
 ProgressResult ExportMP3::Export(AudacityProject *project,
-                       std::unique_ptr<ProgressDialog> &pDialog,
+                       std::unique_ptr<BasicUI::ProgressDialog> &pDialog,
                        unsigned channels,
                        const wxFileNameWrapper &fName,
                        bool selectionOnly,
@@ -1749,7 +1772,7 @@ ProgressResult ExportMP3::Export(AudacityProject *project,
                        const Tags *metadata,
                        int WXUNUSED(subformat))
 {
-   int rate = lrint( ProjectSettings::Get( *project ).GetRate());
+   int rate = lrint( ProjectRate::Get( *project ).GetRate());
 #ifndef DISABLE_DYNAMIC_LOADING_LAME
    wxWindow *parent = ProjectWindow::Find( project );
 #endif // DISABLE_DYNAMIC_LOADING_LAME
@@ -1787,34 +1810,33 @@ ProgressResult ExportMP3::Export(AudacityProject *project,
    int lowrate = 8000;
    int bitrate = 0;
    int brate;
-   int vmode;
+   //int vmode;
    bool forceMono;
 
    gPrefs->Read(wxT("/FileFormats/MP3Bitrate"), &brate, 128);
    auto rmode = MP3RateModeSetting.ReadEnumWithDefault( MODE_CBR );
-   gPrefs->Read(wxT("/FileFormats/MP3VarMode"), &vmode, ROUTINE_FAST);
+   //gPrefs->Read(wxT("/FileFormats/MP3VarMode"), &vmode, ROUTINE_FAST);
    auto cmode = MP3ChannelModeSetting.ReadEnumWithDefault( CHANNEL_STEREO );
    gPrefs->Read(wxT("/FileFormats/MP3ForceMono"), &forceMono, 0);
 
    // Set the bitrate/quality and mode
    if (rmode == MODE_SET) {
       brate = ValidateValue(setRateNames.size(), brate, PRESET_STANDARD);
-      int r = ValidateValue( varModeNames.size(), vmode, ROUTINE_FAST );
+      //int r = ValidateValue( varModeNames.size(), vmode, ROUTINE_FAST );
       exporter.SetMode(MODE_SET);
-      exporter.SetQuality(brate, r);
+      exporter.SetQuality(brate/*, r*/);
    }
    else if (rmode == MODE_VBR) {
       brate = ValidateValue( varRateNames.size(), brate, QUALITY_2 );
-      int r = ValidateValue( varModeNames.size(), vmode, ROUTINE_FAST );
+      //int r = ValidateValue( varModeNames.size(), vmode, ROUTINE_FAST );
       exporter.SetMode(MODE_VBR);
-      exporter.SetQuality(brate, r);
+      exporter.SetQuality(brate/*, r*/);
    }
    else if (rmode == MODE_ABR) {
       brate = ValidateIndex( fixRateValues, brate, 6 /* 128 kbps */ );
       bitrate = fixRateValues[ brate ];
       exporter.SetMode(MODE_ABR);
       exporter.SetBitrate(bitrate);
-
       if (bitrate > 160) {
          lowrate = 32000;
       }
@@ -1839,11 +1861,32 @@ ProgressResult ExportMP3::Export(AudacityProject *project,
    // Verify sample rate
    if (!make_iterator_range( sampRates ).contains( rate ) ||
       (rate < lowrate) || (rate > highrate)) {
-      rate = AskResample(bitrate, rate, lowrate, highrate);
-      if (rate == 0) {
-         return ProgressResult::Cancelled;
-      }
-   }
+        // Force valid sample rate in macros.
+		if (project->mBatchMode) {
+			if (!make_iterator_range( sampRates ).contains( rate )) {
+				auto const bestRateIt = std::lower_bound(sampRates.begin(),
+				sampRates.end(), rate);
+				rate = (bestRateIt == sampRates.end()) ? highrate : *bestRateIt;
+			}
+			if (rate < lowrate) {
+				rate = lowrate;
+			}
+			else if (rate > highrate) {
+				rate = highrate;
+			}
+		}
+		// else validate or prompt
+		else {
+			if (!make_iterator_range( sampRates ).contains( rate ) ||
+				(rate < lowrate) || (rate > highrate)) {
+				rate = AskResample(bitrate, rate, lowrate, highrate);
+			}
+			if (rate == 0) {
+				return ProgressResult::Cancelled;
+       
+			}
+		}
+	}
 
    // Set the channel mode
    if (forceMono) {
@@ -1879,7 +1922,7 @@ ProgressResult ExportMP3::Export(AudacityProject *project,
    if (id3len && !endOfFile) {
       if (id3len > outFile.Write(id3buffer.get(), id3len)) {
          // TODO: more precise message
-         AudacityMessageBox( XO("Unable to export") );
+         ShowExportErrorDialog("MP3:1882");
          return ProgressResult::Cancelled;
       }
    }
@@ -1891,7 +1934,7 @@ ProgressResult ExportMP3::Export(AudacityProject *project,
    size_t bufferSize = std::max(0, exporter.GetOutBufferSize());
    if (bufferSize <= 0) {
       // TODO: more precise message
-      AudacityMessageBox( XO("Unable to export") );
+      ShowExportErrorDialog("MP3:1849");
       return ProgressResult::Cancelled;
    }
 
@@ -1902,7 +1945,7 @@ ProgressResult ExportMP3::Export(AudacityProject *project,
       auto mixer = CreateMixer(tracks, selectionOnly,
          t0, t1,
          channels, inSamples, true,
-         rate, floatSample, true, mixerSpec);
+         rate, floatSample, mixerSpec);
 
       TranslatableString title;
       if (rmode == MODE_SET) {
@@ -1928,11 +1971,9 @@ ProgressResult ExportMP3::Export(AudacityProject *project,
       auto &progress = *pDialog;
 
       while (updateResult == ProgressResult::Success) {
-         auto blockLen = mixer->Process(inSamples);
-
-         if (blockLen == 0) {
+         auto blockLen = mixer->Process();
+         if (blockLen == 0)
             break;
-         }
 
          float *mixed = (float *)mixer->GetBuffer();
 
@@ -1963,12 +2004,12 @@ ProgressResult ExportMP3::Export(AudacityProject *project,
 
          if (bytes > (int)outFile.Write(buffer.get(), bytes)) {
             // TODO: more precise message
-            AudacityMessageBox( XO("Unable to export") );
+            ShowDiskFullExportErrorDialog(fName);
             updateResult = ProgressResult::Cancelled;
             break;
          }
 
-         updateResult = progress.Update(mixer->MixGetCurrentTime() - t0, t1 - t0);
+         updateResult = progress.Poll(mixer->MixGetCurrentTime() - t0, t1 - t0);
       }
    }
 
@@ -1978,14 +2019,14 @@ ProgressResult ExportMP3::Export(AudacityProject *project,
 
       if (bytes < 0) {
          // TODO: more precise message
-         AudacityMessageBox( XO("Unable to export") );
+         ShowExportErrorDialog("MP3:1981");
          return ProgressResult::Cancelled;
       }
 
       if (bytes > 0) {
          if (bytes > (int)outFile.Write(buffer.get(), bytes)) {
             // TODO: more precise message
-            AudacityMessageBox( XO("Unable to export") );
+            ShowExportErrorDialog("MP3:1988");
             return ProgressResult::Cancelled;
          }
       }
@@ -1994,7 +2035,7 @@ ProgressResult ExportMP3::Export(AudacityProject *project,
       if (id3len > 0 && endOfFile) {
          if (bytes > (int)outFile.Write(id3buffer.get(), id3len)) {
             // TODO: more precise message
-            AudacityMessageBox( XO("Unable to export") );
+            ShowExportErrorDialog("MP3:1997");
             return ProgressResult::Cancelled;
          }
       }
@@ -2009,7 +2050,7 @@ ProgressResult ExportMP3::Export(AudacityProject *project,
           !outFile.Flush() ||
           !outFile.Close()) {
          // TODO: more precise message
-         AudacityMessageBox( XO("Unable to export") );
+         ShowExportErrorDialog("MP3:2012");
          return ProgressResult::Cancelled;
       }
    }
