@@ -14,13 +14,13 @@
 
 *//*******************************************************************/
 
-#include "../Audacity.h"
+
 #include "TracksBehaviorsPrefs.h"
+#include "ViewInfo.h"
 
-#include "../Experimental.h"
-
-#include "../Prefs.h"
+#include "Prefs.h"
 #include "../ShuttleGui.h"
+#include "../WaveTrack.h"
 
 TracksBehaviorsPrefs::TracksBehaviorsPrefs(wxWindow * parent, wxWindowID winid)
 /* i18n-hint: i.e. the behaviors of tracks */
@@ -33,27 +33,21 @@ TracksBehaviorsPrefs::~TracksBehaviorsPrefs()
 {
 }
 
-ComponentInterfaceSymbol TracksBehaviorsPrefs::GetSymbol()
+ComponentInterfaceSymbol TracksBehaviorsPrefs::GetSymbol() const
 {
    return TRACKS_BEHAVIORS_PREFS_PLUGIN_SYMBOL;
 }
 
-TranslatableString TracksBehaviorsPrefs::GetDescription()
+TranslatableString TracksBehaviorsPrefs::GetDescription() const
 {
    return XO("Preferences for TracksBehaviors");
 }
 
-wxString TracksBehaviorsPrefs::HelpPageName()
+ManualPageID TracksBehaviorsPrefs::HelpPageName()
 {
    return "Tracks_Behaviors_Preferences";
 }
 
-const wxChar *TracksBehaviorsPrefs::ScrollingPreferenceKey()
-{
-   static auto string = wxT("/GUI/ScrollBeyondZero");
-   return string;
-}
-
 void TracksBehaviorsPrefs::Populate()
 {
    //------------------------- Main section --------------------
@@ -90,9 +84,9 @@ void TracksBehaviorsPrefs::PopulateOrExchange(ShuttleGui & S)
       S.TieCheckBox(XXO("Enable &dragging selection edges"),
                     {wxT("/GUI/AdjustSelectionEdges"),
                      true});
-      S.TieCheckBox(XXO("Editing a clip can &move other clips"),
-                    {wxT("/GUI/EditClipCanMove"),
-                     true});
+      S
+         .TieCheckBox(XXO("Editing a clip can &move other clips"),
+            EditClipsCanMove);
       S.TieCheckBox(XXO("\"Move track focus\" c&ycles repeatedly through tracks"),
                     {wxT("/GUI/CircularTrackNavigation"),
                      false});
@@ -104,8 +98,7 @@ void TracksBehaviorsPrefs::PopulateOrExchange(ShuttleGui & S)
                      false});
 #ifdef EXPERIMENTAL_SCROLLING_LIMITS
       S.TieCheckBox(XXO("Enable scrolling left of &zero"),
-                    {ScrollingPreferenceKey(),
-                     ScrollingPreferenceDefault()});
+                    ScrollingPreference);
 #endif
       S.TieCheckBox(XXO("Advanced &vertical zooming"),
                     {wxT("/GUI/VerticalZooming"),
@@ -143,18 +136,3 @@ PrefsPanel::Registration sAttachment{ "TracksBehaviors",
    { "Tracks" }
 };
 }
-
-// Bug 825 is essentially that SyncLock requires EditClipsCanMove.
-// SyncLock needs rethinking, but meanwhile this function
-// fixes the issues of Bug 825 by allowing clips to move when in
-// SyncLock.
-bool GetEditClipsCanMove()
-{
-   bool mIsSyncLocked;
-   gPrefs->Read(wxT("/GUI/SyncLockTracks"), &mIsSyncLocked, false);
-   if( mIsSyncLocked )
-      return true;
-   bool editClipsCanMove;
-   gPrefs->Read(wxT("/GUI/EditClipCanMove"), &editClipsCanMove, true);
-   return editClipsCanMove;
-}
