@@ -24,10 +24,9 @@
 
 *//**********************************************************************/
 
-#include "../Audacity.h"
+
 #include "ToolManager.h"
 
-#include "../Experimental.h"
 #include "../commands/CommandContext.h"
 
 // For compilers that support precompilation, includes "wx/wx.h".
@@ -53,15 +52,16 @@
 #include <wx/minifram.h>
 #include <wx/popupwin.h>
 
-#include "../AColor.h"
-#include "../AllThemeResources.h"
-#include "../ImageManipulation.h"
-#include "../Prefs.h"
-#include "../Project.h"
-#include "../widgets/AButton.h"
-#include "../widgets/ASlider.h"
-#include "../widgets/MeterPanelBase.h"
-#include "../widgets/Grabber.h"
+#include "AColor.h"
+#include "AllThemeResources.h"
+#include "ImageManipulation.h"
+#include "Prefs.h"
+#include "Project.h"
+#include "ProjectWindows.h"
+#include "widgets/AButton.h"
+#include "widgets/ASlider.h"
+#include "widgets/MeterPanelBase.h"
+#include "widgets/Grabber.h"
 
 ////////////////////////////////////////////////////////////
 /// Methods for ToolFrame
@@ -95,7 +95,15 @@ ToolFrame::ToolFrame
 
    // Transfer the bar to the ferry
    bar->Reparent(this);
-   SetMinSize(bar->GetDockedSize());
+
+   // Bug 2120 (comment 6 residual): No need to set a minimum size
+   // if the toolbar is not resizable. On GTK, setting a minimum
+   // size will prevent the frame from shrinking if the toolbar gets
+   // reconfigured and needs to resize smaller.
+   if (bar->IsResizable())
+   {
+      SetMinSize(bar->GetDockedSize());
+   }
 
    {
       // We use a sizer to maintain proper spacing
@@ -214,7 +222,7 @@ void ToolFrame::OnMotion( wxMouseEvent & event )
 
       rect.SetBottomRight( pos );
 
-      // Keep it within max size, if specificed
+      // Keep it within max size, if specified
       wxSize maxsz = mBar->GetMaxSize();
       if (maxsz != wxDefaultSize)
       {
@@ -325,21 +333,6 @@ BEGIN_EVENT_TABLE( ToolManager, wxEvtHandler )
    EVT_TIMER( wxID_ANY, ToolManager::OnTimer )
 END_EVENT_TABLE()
 
-static ToolManager::GetTopPanelHook &getTopPanelHook()
-{
-   static ToolManager::GetTopPanelHook theHook;
-   return theHook;
-}
-
-auto ToolManager::SetGetTopPanelHook( const GetTopPanelHook &hook )
-   -> GetTopPanelHook
-{
-   auto &theHook = getTopPanelHook();
-   auto result = theHook;
-   theHook = hook;
-   return result;
-}
-
 static const AudacityProject::AttachedObjects::RegisteredFactory key{
   []( AudacityProject &parent ){
      return std::make_shared< ToolManager >( &parent ); }
@@ -443,7 +436,8 @@ void ToolManager::CreateWindows()
                      &ToolManager::OnCaptureLost,
                      this );
 
-   wxWindow *topDockParent = getTopPanelHook()( window );
+   wxWindow *topDockParent = TopPanelHook::Call( window );
+   wxASSERT(topDockParent);
 
    // Create the top and bottom docks
    mTopDock = safenew ToolDock( this, topDockParent, TopDockID );
@@ -523,17 +517,15 @@ static struct DefaultConfigEntry {
    // Top dock row, may wrap
    { TransportBarID,         NoBarID,                NoBarID                },
    { ToolsBarID,             TransportBarID,         NoBarID                },
-   { RecordMeterBarID,       ToolsBarID,             NoBarID                },
-   { PlayMeterBarID,         RecordMeterBarID,       NoBarID                },
-   { MixerBarID,             PlayMeterBarID,         NoBarID                },
-   { EditBarID,              MixerBarID,             NoBarID                },
-
-// DA: Transcription Toolbar not docked, by default.
-#ifdef EXPERIMENTAL_DA
-   { TranscriptionBarID,     NoBarID,                NoBarID                },
+   { EditBarID,              ToolsBarID,             NoBarID                },
+   { AudioSetupBarID,        EditBarID,              NoBarID                },
+#ifdef HAS_AUDIOCOM_UPLOAD
+   { ShareAudioBarID,        AudioSetupBarID,        NoBarID                },
+   { RecordMeterBarID,       ShareAudioBarID,        NoBarID                },
 #else
-   { TranscriptionBarID,     EditBarID,              NoBarID                },
+   { RecordMeterBarID,       AudioSetupBarID,        NoBarID                },
 #endif
+   { PlayMeterBarID,         RecordMeterBarID,       NoBarID                },
 
    // start another top dock row
    { ScrubbingBarID,         NoBarID,                TransportBarID         },
@@ -546,6 +538,13 @@ static struct DefaultConfigEntry {
    { SelectionBarID,         NoBarID,                NoBarID                },
    { TimeBarID,              SelectionBarID,         NoBarID                },
 
+// DA: Transcription Toolbar not docked, by default.
+#ifdef EXPERIMENTAL_DA
+   { TranscriptionBarID,     NoBarID,                NoBarID                },
+#else
+   { TranscriptionBarID,     TimeBarID,              NoBarID                },
+#endif
+
    // Hidden by default in bottom dock
    { SpectralSelectionBarID, NoBarID,                NoBarID                },
 };
@@ -596,6 +595,7 @@ void ToolManager::Reset()
          || ndx == SpectralSelectionBarID
 #endif
          || ndx == TimeBarID
+         || ndx == TranscriptionBarID
          )
          dock = mBotDock;
       else
@@ -626,9 +626,9 @@ void ToolManager::Reset()
          || ndx == SpectralSelectionBarID
 #endif
          || ndx == ScrubbingBarID
-// DA: Hides three more toolbars.
-#ifdef EXPERIMENTAL_DA
          || ndx == DeviceBarID
+// DA: Hides two more toolbars.
+#ifdef EXPERIMENTAL_DA
          || ndx == TranscriptionBarID
          || ndx == SelectionBarID
 #endif
@@ -766,6 +766,8 @@ void ToolManager::ReadConfig()
          bShownByDefault = false;
       if( ndx == ScrubbingBarID )
          bShownByDefault = false;
+      if( ndx == DeviceBarID )
+         bShownByDefault = false;
       if( ndx == TimeBarID )
          defaultDock = BotDockID;
 
@@ -1123,8 +1125,15 @@ void ToolManager::Expose( int type, bool show )
 void ToolManager::LayoutToolBars()
 {
    // Update the layout
-   mTopDock->LayoutToolBars();
-   mBotDock->LayoutToolBars();
+   if (mTopDock)
+   {
+      mTopDock->LayoutToolBars();
+   }
+
+   if (mBotDock)
+   {
+      mBotDock->LayoutToolBars();
+   }
 }
 
 //
@@ -1195,6 +1204,8 @@ void ToolManager::OnMouse( wxMouseEvent & event )
             if (mPrevDock)
                mPrevDock->GetConfiguration().Remove( mDragBar );
             UndockBar(mp);
+            // Rearrange the remaining toolbars before trying to re-insert this one.
+            LayoutToolBars();
          }
       }
 
@@ -1512,7 +1523,12 @@ void ToolManager::HandleEscapeKey()
 
 void ToolManager::DoneDragging()
 {
-   // Done dragging
+   // Done dragging - ensure grabber button isn't pushed
+   if( mDragBar )
+   {
+      mDragBar->SetDocked( mDragBar->GetDock(), false );
+   }
+
    // Release capture
    auto &window = GetProjectFrame( *mParent );
    if( window.HasCapture() )
