@@ -21,24 +21,23 @@
 
 *//*******************************************************************/
 
-#include "../Audacity.h"
+
 #include "AButton.h"
 
-#include "../AColor.h"
+#include "AColor.h"
+#include "TrackArt.h"
 
 #include <wx/setup.h> // for wxUSE_* macros
 
 #include <wx/app.h>
-#include <wx/dcclient.h>
-#include <wx/dcmemory.h>
 #include <wx/dcbuffer.h>
 #include <wx/eventfilter.h>
 #include <wx/image.h>
 #include <wx/timer.h>
 
 //This is needed for tooltips
-#include "../Project.h"
-#include "../ProjectStatus.h"
+#include "Project.h"
+#include "ProjectStatus.h"
 #include "../ProjectWindowBase.h"
 #include <wx/tooltip.h>
 
@@ -193,21 +192,28 @@ int AButton::Listener::FilterEvent(wxEvent &event)
    return Event_Skip;
 }
 
+AButton::AButton(wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, bool toggle)
+{
+   Init(parent, id, pos, size, toggle);
+}
+
 AButton::AButton(wxWindow * parent,
                  wxWindowID id,
                  const wxPoint & pos,
                  const wxSize & size,
-                 ImageRoll up,
-                 ImageRoll over,
-                 ImageRoll down,
-                 ImageRoll overDown,
-                 ImageRoll dis,
-                 bool toggle):
-   wxWindow()
+                 const wxImage& up,
+                 const wxImage& over,
+                 const wxImage& down,
+                 const wxImage& overDown,
+                 const wxImage& dis,
+                 bool toggle)
 {
-   Init(parent, id, pos, size,
-        up, over, down, overDown, dis,
-        toggle);
+   Init(parent, id, pos, size, toggle);
+
+   SetAlternateImages(0, up, over, down, overDown, dis);
+
+   SetMinSize(mImages[0][AButtonUp].GetSize());
+   SetMaxSize(mImages[0][AButtonUp].GetSize());
 }
 
 AButton::~AButton()
@@ -216,46 +222,31 @@ AButton::~AButton()
       ReleaseMouse();
 }
 
-void AButton::Init(wxWindow * parent,
-                   wxWindowID id,
-                   const wxPoint & pos,
-                   const wxSize & size,
-                   ImageRoll up,
-                   ImageRoll over,
-                   ImageRoll down,
-                   ImageRoll overDown,
-                   ImageRoll dis,
-                   bool toggle)
+void AButton::SetButtonType(Type type)
 {
+   if(mType != type)
+   {
+      mType = type;
+      InvalidateBestSize();
+      Refresh(false);
+      PostSizeEventToParent();
+   }
+}
+
+
+void AButton::Init(wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, bool toggle)
+{
+   SetBackgroundStyle(wxBG_STYLE_PAINT);
+
    // Bug in wxWidgets 2.8.12: by default pressing Enter on an AButton is interpreted as
    // a navigation event - move to next control. As a workaround, the style wxWANTS_CHARS
    // results in all characters being available in the OnKeyDown function below. Note
    // that OnKeyDown now has to handle navigation.
    Create(parent, id, pos, size, wxWANTS_CHARS);
 
-   mWasShiftDown = false;
-   mWasControlDown = false;
-   mButtonIsDown = false;
-   mIsClicking = false;
-   mEnabled = true;
-   mCursorIsInWindow = false;
    mToggle = toggle;
-   mUseDisabledAsDownHiliteImage = false;
-
-   mImages.resize(1);
-   mImages[0].mArr[0] = up;
-   mImages[0].mArr[1] = over;
-   mImages[0].mArr[2] = down;
-   mImages[0].mArr[3] = overDown;
-   mImages[0].mArr[4] = dis;
-
-   mAlternateIdx = 0;
 
    mFocusRect = GetClientRect().Deflate( 3, 3 );
-   mForceFocusRect = false;
-
-   SetMinSize(mImages[0].mArr[0].GetMinSize());
-   SetMaxSize(mImages[0].mArr[0].GetMaxSize());
 
 #if wxUSE_ACCESSIBILITY
    SetName( wxT("") );
@@ -276,6 +267,8 @@ void AButton::SetToolTip( const TranslatableString &toolTip )
 void AButton::SetLabel( const TranslatableString &toolTip )
 {
    wxWindow::SetLabel( toolTip.Stripped().Translation() );
+   if(mType == FrameButton)
+      InvalidateBestSize();
 }
 
 // This compensates for a but in wxWidgets 3.0.2 for mac:
@@ -287,36 +280,31 @@ void AButton::SetFocusFromKbd()
    SetFocus();
 }
 
-void AButton::SetAlternateImages(unsigned idx,
-                                 wxImage up,
-                                 wxImage over,
-                                 wxImage down,
-                                 wxImage overDown,
-                                 wxImage dis)
+void AButton::SetImages(const wxImage& up, const wxImage& over, const wxImage& down, const wxImage& overDown, const wxImage& dis)
 {
-   if (1 + idx > mImages.size())
-      mImages.resize(1 + idx);
-   mImages[idx].mArr[0] = ImageRoll(up);
-   mImages[idx].mArr[1] = ImageRoll(over);
-   mImages[idx].mArr[2] = ImageRoll(down);
-   mImages[idx].mArr[3] = ImageRoll(overDown);
-   mImages[idx].mArr[4] = ImageRoll(dis);
+   SetAlternateImages(0, up, over, down, overDown, dis);
 }
 
 void AButton::SetAlternateImages(unsigned idx,
-                                 ImageRoll up,
-                                 ImageRoll over,
-                                 ImageRoll down,
-                                 ImageRoll overDown,
-                                 ImageRoll dis)
+                                 const wxImage& up,
+                                 const wxImage& over,
+                                 const wxImage& down,
+                                 const wxImage& overDown,
+                                 const wxImage& dis)
 {
    if (1 + idx > mImages.size())
       mImages.resize(1 + idx);
-   mImages[idx].mArr[0] = up;
-   mImages[idx].mArr[1] = over;
-   mImages[idx].mArr[2] = down;
-   mImages[idx].mArr[3] = overDown;
-   mImages[idx].mArr[4] = dis;
+   mImages[idx][AButtonUp] = up;
+   mImages[idx][AButtonOver] = over;
+   mImages[idx][AButtonDown] = down;
+   mImages[idx][AButtonOverDown] = overDown;
+   mImages[idx][AButtonDis] = dis;
+}
+
+void AButton::SetIcon(const wxImage& icon)
+{
+   mIcon = icon;
+   Refresh(false);
 }
 
 void AButton::SetAlternateIdx(unsigned idx)
@@ -327,6 +315,7 @@ void AButton::SetAlternateIdx(unsigned idx)
       return;
    mAlternateIdx = idx;
    Refresh(false);
+   PostSizeEventToParent();
 }
 
 void AButton::FollowModifierKeys()
@@ -335,7 +324,7 @@ void AButton::FollowModifierKeys()
       mListener = std::make_unique<Listener>(this);
 }
 
-void AButton::SetFocusRect(wxRect & r)
+void AButton::SetFocusRect(const wxRect & r)
 {
    mFocusRect = r;
    mForceFocusRect = true;
@@ -373,12 +362,7 @@ AButton::AButtonState AButton::GetState()
       }
    }
    else {
-      //if (mToggle) {
-         state = mButtonIsDown ? AButtonDown : AButtonUp;
-      //}
-      //else {
-         //state = mButtonIsDown ? AButtonDown : AButtonUp;
-      //}
+      state = mButtonIsDown ? AButtonDown : AButtonUp;
    }
 
    return state;
@@ -388,14 +372,69 @@ void AButton::OnPaint(wxPaintEvent & WXUNUSED(event))
 {
    wxBufferedPaintDC dc(this);
 
-   AButtonState buttonState = GetState();
+   dc.SetPen(*wxTRANSPARENT_PEN);
+   dc.SetBrush(GetBackgroundColour());
+   dc.Clear();
 
-   mImages[mAlternateIdx].mArr[buttonState].Draw(dc, GetClientRect());
-
-   if( this == wxWindow::FindFocus() )
+   const auto buttonRect = GetClientRect();
+   if(HasAlternateImages(mAlternateIdx))
    {
-      AColor::DrawFocus( dc, mFocusRect );
+      AButtonState buttonState = GetState();
+      if(mType == ImageButton)
+         dc.DrawBitmap(mImages[mAlternateIdx][buttonState], buttonRect.GetTopLeft());
+      else if(mType == FrameButton)
+      {
+         wxBitmap bitmap = mImages[mAlternateIdx][buttonState];
+         AColor::DrawFrame(dc, buttonRect, bitmap);
+
+         const auto border = bitmap.GetSize() / 4;
+
+         if(!GetLabel().IsEmpty())
+         {
+            dc.SetFont(GetFont());
+            auto textRect = buttonRect;
+            if(mIcon.IsOk())
+            {
+               auto fontMetrics = dc.GetFontMetrics();
+               auto sumHeight = fontMetrics.height + mIcon.GetHeight() + border.y;
+               dc.DrawBitmap(mIcon,
+                  buttonRect.x + (buttonRect.width - mIcon.GetWidth()) / 2,
+                  buttonRect.y + (buttonRect.height - sumHeight) / 2);
+               textRect = wxRect(
+                     buttonRect.x,
+                     buttonRect.y + buttonRect.height / 2 + sumHeight / 2 - fontMetrics.height,
+                     buttonRect.width,
+                     fontMetrics.height);
+            }
+            dc.SetPen(GetForegroundColour());
+            dc.DrawLabel(GetLabel(), textRect, wxALIGN_CENTER);
+         }
+         else if(mIcon.IsOk())
+         {
+            dc.DrawBitmap(mIcon,
+               buttonRect.x + (buttonRect.width - mIcon.GetWidth() / 2),
+                  buttonRect.y + (buttonRect.height - mIcon.GetHeight() / 2));
+         }
+      }
+      else
+      {
+         wxBitmap bitmap = mImages[mAlternateIdx][buttonState];
+         AColor::DrawHStretch(dc, GetClientRect(), bitmap);
+         if(!GetLabel().IsEmpty())
+         {
+            dc.SetFont(GetFont());
+            const auto text = TrackArt::TruncateText(dc, GetLabel(), GetClientSize().GetWidth() - 6);
+            if(!text.IsEmpty())
+            {
+               dc.SetPen(GetForegroundColour());
+               dc.DrawLabel(text, GetClientRect(), wxALIGN_CENTER);
+            }
+         }
+      }
    }
+   
+   if(HasFocus())
+      AColor::DrawFocus( dc, mFocusRect );
 }
 
 void AButton::OnErase(wxEraseEvent & WXUNUSED(event))
@@ -414,13 +453,12 @@ void AButton::OnSize(wxSizeEvent & WXUNUSED(event))
 
 bool AButton::s_AcceptsFocus{ false };
 
-bool AButton::HasAlternateImages(unsigned idx)
+bool AButton::HasAlternateImages(unsigned idx) const
 {
    if (mImages.size() <= idx)
       return false;
 
-   const ImageArr &images = mImages[idx];
-   const ImageRoll (&arr)[5] = images.mArr;
+   const auto &arr = mImages[idx];
    return (arr[0].Ok() &&
            arr[1].Ok() &&
            arr[2].Ok() &&
@@ -465,8 +503,6 @@ void AButton::OnMouseEvent(wxMouseEvent & event)
          if (mCursorIsInWindow && (mToggle || !mButtonIsDown)) {
             if (mToggle)
                mButtonIsDown = !mButtonIsDown;
-            else
-               mButtonIsDown = true;
 
             mWasShiftDown = event.ShiftDown();
             mWasControlDown = event.ControlDown();
@@ -527,15 +563,17 @@ void AButton::OnKeyDown(wxKeyEvent & event)
    switch( event.GetKeyCode() )
    {
    case WXK_RIGHT:
+   case WXK_DOWN:
       Navigate(wxNavigationKeyEvent::IsForward);
       break;
    case WXK_LEFT:
+   case WXK_UP:
       Navigate(wxNavigationKeyEvent::IsBackward);
       break;
    case WXK_TAB:
-      Navigate(event.ShiftDown()
+      Navigate(wxNavigationKeyEvent::FromTab | (event.ShiftDown()
                ? wxNavigationKeyEvent::IsBackward
-               : wxNavigationKeyEvent::IsForward);
+               : wxNavigationKeyEvent::IsForward));
       break;
    case WXK_RETURN:
    case WXK_NUMPAD_ENTER:
@@ -543,6 +581,11 @@ void AButton::OnKeyDown(wxKeyEvent & event)
          break;
       mWasShiftDown = event.ShiftDown();
       mWasControlDown = event.ControlDown();
+      if(mToggle)
+      {
+         mButtonIsDown = !mButtonIsDown;
+         Refresh(false);
+      }
       Click();
       break;
    default:
@@ -635,6 +678,45 @@ void AButton::SetControl(bool control)
    mWasControlDown = control;
 }
 
+wxSize AButton::DoGetBestClientSize() const
+{
+   if(HasAlternateImages(mAlternateIdx))
+   {
+      const auto& image = mImages[mAlternateIdx][AButtonUp];
+      switch(mType)
+      {
+      case FrameButton:
+         {
+            const auto border = image.GetSize() / 4;
+            wxSize bestSize { -1 , -1 };
+            if(!GetLabel().IsEmpty())
+            {
+               wxMemoryDC dc;
+               dc.SetFont(GetFont());
+               bestSize = dc.GetTextExtent(GetLabel());
+            }
+            if(mIcon.IsOk())
+            {
+               bestSize.x = std::max(bestSize.x, mIcon.GetWidth());
+               bestSize.y = bestSize.y > 0
+                  ? bestSize.y + border.y + mIcon.GetHeight()
+                  : mIcon.GetHeight();
+            }
+            if(bestSize.x > 0)
+               bestSize.x += border.x * 2;
+            if(bestSize.y > 0)
+               bestSize.y += border.y * 2;
+            return bestSize;
+         }
+      case TextButton:
+         return {-1, image.GetHeight() };
+      default:
+         return image.GetSize();
+      }
+   }
+   return wxWindow::DoGetBestClientSize();
+}
+
 auto AButton::TemporarilyAllowFocus() -> TempAllowFocus {
    s_AcceptsFocus = true;
    return TempAllowFocus{ &s_AcceptsFocus };
@@ -662,6 +744,11 @@ wxAccStatus AButtonAx::DoDefaultAction(int WXUNUSED(childId))
    if(ab && ab->IsEnabled()) {
       ab->mWasShiftDown = false;
       ab->mWasControlDown = false;
+      if(ab->mToggle)
+      {
+         ab->mButtonIsDown = !ab->mButtonIsDown;
+         ab->Refresh(false);
+      }
       ab->Click();
    }
 
@@ -811,33 +898,21 @@ wxAccStatus AButtonAx::GetSelections( wxVariant * WXUNUSED(selections) )
 wxAccStatus AButtonAx::GetState(int WXUNUSED(childId), long* state)
 {
    AButton *ab = wxDynamicCast( GetWindow(), AButton );
-
-   switch( ab->GetState() )
+   *state = 0;
+   if(!ab->IsEnabled())
+       *state = wxACC_STATE_SYSTEM_UNAVAILABLE;
+   else
    {
-      case AButton::AButtonDown:
-         *state = wxACC_STATE_SYSTEM_PRESSED | wxACC_STATE_SYSTEM_FOCUSABLE;
-      break;
+      if(ab->mButtonIsDown)
+         *state |= wxACC_STATE_SYSTEM_PRESSED;
 
-      case AButton::AButtonOver:
-         *state = wxACC_STATE_SYSTEM_HOTTRACKED | wxACC_STATE_SYSTEM_FOCUSABLE;
-      break;
+      if(ab->mCursorIsInWindow)
+         *state |= wxACC_STATE_SYSTEM_HOTTRACKED;
 
-      case AButton::AButtonOverDown:
-         *state = wxACC_STATE_SYSTEM_HOTTRACKED | wxACC_STATE_SYSTEM_PRESSED |
-            wxACC_STATE_SYSTEM_FOCUSABLE;
-      break;
-
-      case AButton::AButtonDis:
-         *state = wxACC_STATE_SYSTEM_UNAVAILABLE;
-      break;
-
-      default:
-         *state = wxACC_STATE_SYSTEM_FOCUSABLE;
-      break;
+      *state |= wxACC_STATE_SYSTEM_FOCUSABLE;
+      if(ab->HasFocus())
+         *state |= wxACC_STATE_SYSTEM_FOCUSED;
    }
-
-   *state |= ( ab == wxWindow::FindFocus() ? wxACC_STATE_SYSTEM_FOCUSED : 0 );
-
    return wxACC_OK;
 }
 
